import { Component, Host, h, Prop, Event, State } from '@stencil/core';
export class K2btSplitDatepicker {
  constructor() {
    this.value = null;
    this.readonlyclass = '';
    this.enabled = true;
    this.validationerrorcode = K2btSplitDatepicker.VALIDATION_ERROR_NOERROR;
    this.yearControl = null;
    this.monthControl = null;
    this.dayControl = null;
    this.minuteControl = null;
    this.hourControl = null;
  }
  connectedCallback() {
    if (this.value) {
      this.selectedYear = this.value.getFullYear();
      this.selectedMonth = this.value.getMonth() + 1;
      this.selectedDay = this.value.getDate();
      this.selectedHour = this.value.getHours();
      this.selectedMinute = this.value.getMinutes();
    }
    else {
      this.selectedYear = 0;
      this.selectedMonth = 0;
      this.selectedDay = 0;
      this.selectedHour = -1;
      this.selectedMinute = -1;
      this.validationerrorcode = K2btSplitDatepicker.VALIDATION_ERROR_EMPTYYEAR;
    }
    this.validateDate();
  }
  render() {
    var readonlyClassString = this.readonlyclass;
    return (h(Host, null,
      h("p", { class: "form-control-static", style: this.enabled ? { display: 'none' } : {} },
        h("span", { class: readonlyClassString, "data-gx-readonly": "" }, this.getReadonlyContents())),
      h("div", { style: !this.enabled ? { display: 'none' } : {} }, this.renderDatepickerContents())));
  }
  getReadonlyContents() {
    switch (this.fieldorder) {
      case K2btSplitDatepicker.ORDER_DMY:
        return `${this.normalizeToAmountOfDigits(this.selectedDay, 2)}/${this.getSelectedMonthString()}/${this.normalizeToAmountOfDigits(this.selectedYear, 4)} ${this.normalizeToAmountOfDigits(this.selectedHour, 2)}:${this.normalizeToAmountOfDigits(this.selectedMinute, 2)}`;
      case K2btSplitDatepicker.ORDER_YMD:
        return `${this.normalizeToAmountOfDigits(this.selectedYear, 4)}/${this.getSelectedMonthString()}/${this.normalizeToAmountOfDigits(this.selectedDay, 2)} ${this.normalizeToAmountOfDigits(this.selectedHour, 2)}:${this.normalizeToAmountOfDigits(this.selectedMinute, 2)}`;
      case K2btSplitDatepicker.ORDER_MDY:
        return `${this.getSelectedMonthString()}/${this.normalizeToAmountOfDigits(this.selectedDay, 2)}/${this.normalizeToAmountOfDigits(this.selectedYear, 4)} ${this.normalizeToAmountOfDigits(this.selectedHour, 2)}:${this.normalizeToAmountOfDigits(this.selectedMinute, 2)}`;
    }
  }
  getSelectedMonthString() {
    var month = this.selectedMonth;
    if (month === 0)
      return this.normalizeToAmountOfDigits(0, 2);
    switch (this.showmonthas) {
      case K2btSplitDatepicker.SHOWMONTHAS_NUMBER:
        return this.normalizeToAmountOfDigits(month, 2);
      case K2btSplitDatepicker.SHOWMONTHAS_NAME:
        //@ts-ignore
        return k2btools.getMonthName(month - 1);
      case K2btSplitDatepicker.SHOWMONTHAS_SHORTNAME:
        //@ts-ignore
        return k2btools.getMonthShortName(month - 1);
    }
  }
  normalizeToAmountOfDigits(value, digits) {
    return '0'.repeat(digits - value.toString().length) + value.toString();
  }
  renderDatepickerContents() {
    switch (this.fieldorder) {
      case K2btSplitDatepicker.ORDER_DMY:
        return (h("div", { class: "K2BT_SplitDatepicker" },
          this.renderDayField(),
          this.renderMonthField(),
          this.renderYearField(),
          this.renderTimeFields()));
      case K2btSplitDatepicker.ORDER_YMD:
        return (h("div", { class: "K2BT_SplitDatepicker" },
          this.renderYearField(),
          this.renderMonthField(),
          this.renderDayField(),
          this.renderTimeFields()));
      case K2btSplitDatepicker.ORDER_MDY:
        return (h("div", { class: "K2BT_SplitDatepicker" },
          this.renderMonthField(),
          this.renderDayField(),
          this.renderYearField(),
          this.renderTimeFields()));
    }
  }
  renderTimeFields() {
    return (h("div", { class: "K2BT_SplitDatepickerTime" },
      this.renderHourField(),
      this.renderMinuteField()));
  }
  renderYearField() {
    if (this.fieldcontroltype == K2btSplitDatepicker.FIELDCONTROLTYPE_COMBO) {
      return this.renderYearFieldAsCombo();
    }
    else {
      if (this.minimumdate) {
        return this.renderYearFieldAsSuggest();
      }
      else {
        return this.renderYearFieldAsInputWithNoSuggest();
      }
    }
  }
  renderYearFieldAsInputWithNoSuggest() {
    return (h("input", { type: "text", class: "form-control Attribute_Trn", size: Math.max(4, this.yearinvitemessage.length), placeholder: this.yearinvitemessage, ref: c => (this.yearControl = c), value: this.selectedYear != 0 ? this.selectedYear.toString() : '', onChange: () => {
        if (this.yearControl.value)
          this.selectedYear = parseInt(this.yearControl.value);
        else
          this.selectedYear = 0;
        if (isNaN(this.selectedYear))
          this.selectedYear = 0;
        this.validateDate();
      } }));
  }
  renderYearFieldAsSuggest() {
    const AMOUNT_OF_SUGGEST_ITEMS = 15;
    return (h("k2bt-enhancedsuggest", { inputSize: Math.max(4, this.yearinvitemessage.length), onSuggestValueChanged: this.processYearChange.bind(this), enableadditem: false, placeholder: this.yearinvitemessage, noresultsfoundtext: this.yearnotfoundmessage, value: this.selectedYear != 0 ? [this.selectedYear.toString()] : [], values: this.getCandidateYears(AMOUNT_OF_SUGGEST_ITEMS), maxSelectionSize: 1, suggestmaxitems: AMOUNT_OF_SUGGEST_ITEMS, ref: c => (this.yearControl = c), clientSideFiltering: "Disabled", onInputValueChanged: () => {
        this.yearControl.values = this.getCandidateYears(AMOUNT_OF_SUGGEST_ITEMS);
      } }));
  }
  renderYearFieldAsCombo() {
    var values = Array.from(Array(this.maximumdate.getFullYear() - this.minimumdate.getFullYear() + 1).keys()).map(i => {
      var year = i + this.minimumdate.getFullYear();
      return { value: year.toString(), description: this.normalizeToAmountOfDigits(year, 4) };
    });
    return (h("k2bt-enhancedcombo", { optionswidth: Math.max(4, this.yearinvitemessage.length) + 'ch !important', showemptyiteminlist: false, onChange: this.processYearChange.bind(this), includesearch: false, showselectionastags: false, enableadditem: false, includeemptyitem: true, emptyitemtext: this.yearinvitemessage, headermaxvisibleitems: 1, values: values, value: this.selectedYear != 0 ? [this.selectedYear.toString()] : [], maxSelectionSize: 1, ref: c => (this.yearControl = c) }));
  }
  processYearChange() {
    var _a, _b;
    if (((_b = (_a = this.yearControl.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) > 0)
      this.selectedYear = parseInt(this.yearControl.value[0]);
    else
      this.selectedYear = 0;
    if (isNaN(this.selectedYear) ||
      (this.maximumdate && this.selectedYear > this.maximumdate.getFullYear()) ||
      (this.minimumdate && this.selectedYear < this.minimumdate.getFullYear()))
      this.selectedYear = 0;
    this.validateDate();
  }
  getCandidateYears(max_candidates) {
    var _a, _b;
    var prefix = (_b = (_a = this.yearControl) === null || _a === void 0 ? void 0 : _a.searchvalue) !== null && _b !== void 0 ? _b : '';
    var start = this.getStartYearForSuggestions(prefix);
    max_candidates = Math.min(max_candidates, Math.pow(10, 4 - prefix.length));
    if (this.minimumdate || prefix.length >= 2)
      return Array.from(Array(max_candidates).keys())
        .map(i => i + start)
        .filter(year => this.maximumdate == null || year <= this.maximumdate.getFullYear())
        .map(y => {
        return { value: y.toString(), description: y.toString() };
      });
    else
      return [];
  }
  getStartYearForSuggestions(prefix) {
    var start = 0;
    if (prefix !== '') {
      const missingDigits = prefix.length <= 4 ? 4 - prefix.length : 0;
      start = parseInt(prefix) * Math.pow(10, missingDigits);
    }
    if (this.minimumdate != null)
      start = Math.max(start, this.minimumdate.getFullYear(), this.selectedYear);
    return start;
  }
  renderMonthField() {
    var values = this.getMonthValues();
    if (this.fieldcontroltype == K2btSplitDatepicker.FIELDCONTROLTYPE_COMBO) {
      return this.renderMonthAsCombo(values);
    }
    else {
      return this.renderMonthAsSuggest(values);
    }
  }
  renderMonthAsSuggest(values) {
    return (h("k2bt-enhancedsuggest", { 
      //@ts-ignore
      inputSize: Math.max(Math.max(...values.map(v => v.description.length)), this.monthinvitemessage.length), onSuggestValueChanged: this.processMonthChange.bind(this), enableadditem: false, placeholder: this.monthinvitemessage, noresultsfoundtext: this.monthnotfoundmessage, value: this.selectedMonth != 0 ? [this.selectedMonth.toString()] : [], values: values, suggestmaxitems: 12, maxSelectionSize: 1, ref: c => (this.monthControl = c), onInputValueChanged: () => (this.monthControl.values = this.getMonthValues()) }));
  }
  renderMonthAsCombo(values) {
    return (h("k2bt-enhancedcombo", { optionswidth: Math.max(Math.max(...values.map(v => v.description.length)), this.monthinvitemessage.length) + 'ch !important', showemptyiteminlist: false, onChange: this.processMonthChange.bind(this), includesearch: false, showselectionastags: false, enableadditem: false, includeemptyitem: true, emptyitemtext: this.monthinvitemessage, headermaxvisibleitems: 1, values: values, value: this.selectedMonth != 0 ? [this.selectedMonth.toString()] : [], maxSelectionSize: 1, ref: c => (this.monthControl = c) }));
  }
  processMonthChange() {
    var _a, _b;
    if (((_b = (_a = this.monthControl.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) > 0)
      this.selectedMonth = parseInt(this.monthControl.value[0]);
    else
      this.selectedMonth = 0;
    if (isNaN(this.selectedMonth) || this.selectedMonth > 12 || this.selectedMonth < 1)
      this.selectedMonth = 0;
    this.validateDate();
  }
  getMonthValues() {
    switch (this.showmonthas) {
      case K2btSplitDatepicker.SHOWMONTHAS_NUMBER:
        return Array.from(Array(12).keys()).map(i => {
          var month = i + 1;
          return { value: month.toString(), description: this.normalizeToAmountOfDigits(month, 2) };
        });
      case K2btSplitDatepicker.SHOWMONTHAS_NAME:
        return Array.from(Array(12).keys()).map(i => {
          var month = i + 1;
          //@ts-ignore
          return { value: month.toString(), description: k2btools.getMonthName(month - 1) };
        });
      case K2btSplitDatepicker.SHOWMONTHAS_SHORTNAME:
        return Array.from(Array(12).keys()).map(i => {
          var month = i + 1;
          //@ts-ignore
          return { value: month.toString(), description: k2btools.getMonthShortName(month - 1) };
        });
    }
  }
  renderDayField() {
    var values = Array.from(Array(31).keys()).map(i => {
      var day = i + 1;
      return { value: day.toString(), description: this.normalizeToAmountOfDigits(day, 2) };
    });
    if (this.fieldcontroltype == K2btSplitDatepicker.FIELDCONTROLTYPE_COMBO) {
      return this.renderDayFieldAsCombo(values);
    }
    else {
      return this.renderDayFieldAsSuggest(values);
    }
  }
  renderDayFieldAsSuggest(values) {
    return (h("k2bt-enhancedsuggest", { inputSize: Math.max(2, this.dayinvitemessage.length), onSuggestValueChanged: this.processDayChange.bind(this), enableadditem: false, placeholder: this.dayinvitemessage, noresultsfoundtext: this.daynotfoundmessage, value: this.selectedDay != 0 ? [this.selectedDay.toString()] : [], values: values, maxSelectionSize: 1, suggestmaxitems: 31, ref: c => (this.dayControl = c), onInputValueChanged: () => (this.dayControl.values = [...values]) }));
  }
  renderDayFieldAsCombo(values) {
    return (h("k2bt-enhancedcombo", { optionswidth: Math.max(2, this.dayinvitemessage.length) + 'ch !important', showemptyiteminlist: false, onChange: this.processDayChange.bind(this), includesearch: false, showselectionastags: false, enableadditem: false, includeemptyitem: true, emptyitemtext: this.dayinvitemessage, headermaxvisibleitems: 1, values: values, value: this.selectedDay != 0 ? [this.selectedDay.toString()] : [], maxSelectionSize: 1, ref: c => (this.dayControl = c) }));
  }
  processDayChange() {
    var _a, _b;
    if (((_b = (_a = this.dayControl.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) > 0)
      this.selectedDay = parseInt(this.dayControl.value[0]);
    else
      this.selectedDay = 0;
    if (isNaN(this.selectedDay) || this.selectedDay > 31 || this.selectedDay < 1)
      this.selectedDay = 0;
    this.validateDate();
  }
  getMaxDayForSelectedMonth() {
    var year = this.selectedYear;
    var month = this.selectedMonth;
    // This is the first date of next month (as month parameter is 0 based)
    const nextMonth = new Date(year, month, 1);
    nextMonth.setDate(nextMonth.getDate() - 1);
    return nextMonth.getDate();
  }
  renderHourField() {
    if (!this.showtimepart)
      return null;
    var values = this.getHourFieldValues();
    if (this.fieldcontroltype == K2btSplitDatepicker.FIELDCONTROLTYPE_COMBO) {
      return this.renderHourFieldAsCombo(values);
    }
    else {
      return this.renderHourFieldAsSuggest(values);
    }
  }
  getHourFieldValues() {
    return Array.from(Array(this.endhour + 1 - this.starthour).keys()).map(i => {
      var hour = i + this.starthour; // +1 to avoid "empty" values
      if (this.hourformat == K2btSplitDatepicker.HOURFORMAT_24H)
        return { value: (hour + 1).toString(), description: this.normalizeToAmountOfDigits(hour, 2) };
      else {
        var ampmHour = hour;
        if (hour === 0)
          ampmHour = 12;
        if (hour > 12)
          ampmHour = hour - 12;
        return { value: (hour + 1).toString(), description: ampmHour.toString() + ' ' + (hour >= 12 ? 'PM' : 'AM') };
      }
    });
  }
  renderHourFieldAsSuggest(values) {
    return (h("k2bt-enhancedsuggest", { 
      //@ts-ignore
      inputSize: Math.max(Math.max(...values.map(v => v.description.length)), this.hourinvitemessage.length), onSuggestValueChanged: this.processHourChange.bind(this), enableadditem: false, placeholder: this.hourinvitemessage, noresultsfoundtext: this.hournotfoundmessage, values: values, suggestmaxitems: 24, value: this.selectedHour != -1 ? [(this.selectedHour + 1).toString()] : [], maxSelectionSize: 1, ref: c => (this.hourControl = c), onInputValueChanged: () => (this.hourControl.values = [...values]) }));
  }
  renderHourFieldAsCombo(values) {
    return (h("k2bt-enhancedcombo", { optionswidth: Math.max(Math.max(...values.map(v => v.description.length)), this.hourinvitemessage.length) + 'ch !important', showemptyiteminlist: false, onChange: this.processHourChange.bind(this), includesearch: false, showselectionastags: false, enableadditem: false, includeemptyitem: true, emptyitemvalue: "-1", emptyitemtext: this.hourinvitemessage, headermaxvisibleitems: 1, values: values, value: this.selectedHour != -1 ? [(this.selectedHour + 1).toString()] : [], maxSelectionSize: 1, ref: c => (this.hourControl = c) }));
  }
  processHourChange() {
    var _a, _b;
    // Decrease 1 because of values being 1-based and not 0-based
    if (((_b = (_a = this.hourControl.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) > 0)
      this.selectedHour = parseInt(this.hourControl.value[0]) - 1;
    else
      this.selectedYear = 0;
    this.validateDate();
  }
  renderMinuteField() {
    if (!this.showtimepart)
      return null;
    var values = [];
    for (var m = 0; m < 60; m += this.minutestep) {
      // starting in 1 to avoid "empty" values
      values.push({ value: (m + 1).toString(), description: this.normalizeToAmountOfDigits(m, 2) });
    }
    if (this.fieldcontroltype == K2btSplitDatepicker.FIELDCONTROLTYPE_COMBO) {
      return this.renderMinuteFieldAsCombo(values);
    }
    else {
      return this.renderMinuteFieldAsSuggest(values);
    }
  }
  renderMinuteFieldAsSuggest(values) {
    return (h("k2bt-enhancedsuggest", { 
      //@ts-ignore
      inputSize: Math.max(Math.max(...values.map(v => v.description.length)), this.minuteinvitemessage.length), onSuggestValueChanged: this.processMinuteChange.bind(this), suggestmaxitems: 60, enableadditem: false, placeholder: this.minuteinvitemessage, noresultsfoundtext: this.minutenotfoundmessage, values: values, value: this.selectedMinute != -1 ? [(this.selectedMinute + 1).toString()] : [], maxSelectionSize: 1, ref: c => (this.minuteControl = c), onInputValueChanged: () => (this.minuteControl.values = [...values]) }));
  }
  processMinuteChange() {
    var _a, _b;
    // Decrease 1 because of values being 1-based and not 0-based
    if (((_b = (_a = this.minuteControl.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) > 0)
      this.selectedMinute = parseInt(this.minuteControl.value[0]) - 1;
    else
      this.selectedMinute = 0;
    this.validateDate();
  }
  renderMinuteFieldAsCombo(values) {
    return (h("k2bt-enhancedcombo", { optionswidth: Math.max(Math.max(...values.map(v => v.description.length)), this.minuteinvitemessage.length) + 'ch !important', showemptyiteminlist: false, onChange: this.processMinuteChange.bind(this), includesearch: false, showselectionastags: false, enableadditem: false, includeemptyitem: true, emptyitemvalue: "-1", emptyitemtext: this.minuteinvitemessage, headermaxvisibleitems: 1, values: values, value: this.selectedMinute != -1 ? [(this.selectedMinute + 1).toString()] : [], maxSelectionSize: 1, ref: c => (this.minuteControl = c) }));
  }
  componentDidRender() {
    var _a, _b, _c, _d, _e;
    if (this.fieldcontroltype == K2btSplitDatepicker.FIELDCONTROLTYPE_SUGGEST) {
      //@ts-ignore
      if ((_a = this.yearControl) === null || _a === void 0 ? void 0 : _a.updateDescription)
        this.yearControl.updateDescription();
      //@ts-ignore
      if ((_b = this.monthControl) === null || _b === void 0 ? void 0 : _b.updateDescription)
        this.monthControl.updateDescription();
      //@ts-ignore
      if ((_c = this.dayControl) === null || _c === void 0 ? void 0 : _c.updateDescription)
        this.dayControl.updateDescription();
      //@ts-ignore
      if ((_d = this.hourControl) === null || _d === void 0 ? void 0 : _d.updateDescription)
        this.hourControl.updateDescription();
      //@ts-ignore
      if ((_e = this.minuteControl) === null || _e === void 0 ? void 0 : _e.updateDescription)
        this.minuteControl.updateDescription();
    }
  }
  validateDate() {
    this.validationerrorcode = K2btSplitDatepicker.VALIDATION_ERROR_NOERROR;
    // Validate not empty
    if (this.validationerrorcode === K2btSplitDatepicker.VALIDATION_ERROR_NOERROR && this.selectedDay == 0)
      this.validationerrorcode = K2btSplitDatepicker.VALIDATION_ERROR_EMPTYDAY;
    if (this.validationerrorcode === K2btSplitDatepicker.VALIDATION_ERROR_NOERROR && this.selectedMonth == 0)
      this.validationerrorcode = K2btSplitDatepicker.VALIDATION_ERROR_EMPTYMONTH;
    if (this.validationerrorcode === K2btSplitDatepicker.VALIDATION_ERROR_NOERROR && this.selectedYear == 0)
      this.validationerrorcode = K2btSplitDatepicker.VALIDATION_ERROR_EMPTYYEAR;
    if (this.showtimepart) {
      if (this.validationerrorcode === K2btSplitDatepicker.VALIDATION_ERROR_NOERROR && this.selectedHour == -1)
        this.validationerrorcode = K2btSplitDatepicker.VALIDATION_ERROR_EMPTYHOUR;
      if (this.validationerrorcode === K2btSplitDatepicker.VALIDATION_ERROR_NOERROR && this.selectedMinute == -1)
        this.validationerrorcode = K2btSplitDatepicker.VALIDATION_ERROR_EMPTYMINUTE;
    }
    // Validate month boundary
    var day = this.selectedDay;
    const maxDayForMonth = this.getMaxDayForSelectedMonth();
    if (this.validationerrorcode === K2btSplitDatepicker.VALIDATION_ERROR_NOERROR && day > maxDayForMonth)
      this.validationerrorcode = K2btSplitDatepicker.VALIDATION_ERROR_INVALIDDAYFORMONTH;
    const candidateDate = this.showtimepart
      ? new Date(this.selectedYear, this.selectedMonth - 1, day, this.selectedHour, this.selectedMinute)
      : new Date(this.selectedYear, this.selectedMonth - 1, day);
    // Validate accepted date ranges
    if (this.validationerrorcode === K2btSplitDatepicker.VALIDATION_ERROR_NOERROR && this.minimumdate && candidateDate < this.minimumdate)
      this.validationerrorcode = K2btSplitDatepicker.VALIDATION_ERROR_DATETOOLOW;
    if (this.validationerrorcode === K2btSplitDatepicker.VALIDATION_ERROR_NOERROR && this.maximumdate && candidateDate > this.maximumdate)
      this.validationerrorcode = K2btSplitDatepicker.VALIDATION_ERROR_DATETOOHIGH;
    if (this.validationerrorcode === K2btSplitDatepicker.VALIDATION_ERROR_NOERROR)
      this.value = candidateDate;
    else
      this.value = null;
    this.inputEvent.emit({});
    this.changeEvent.emit({});
  }
  static get is() { return "k2bt-split-datepicker"; }
  static get properties() { return {
    "showtimepart": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "showtimepart",
      "reflect": false
    },
    "minimumdate": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Date",
        "resolved": "Date",
        "references": {
          "Date": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      }
    },
    "maximumdate": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Date",
        "resolved": "Date",
        "references": {
          "Date": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      }
    },
    "dayinvitemessage": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "dayinvitemessage",
      "reflect": false
    },
    "monthinvitemessage": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "monthinvitemessage",
      "reflect": false
    },
    "yearinvitemessage": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "yearinvitemessage",
      "reflect": false
    },
    "hourinvitemessage": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "hourinvitemessage",
      "reflect": false
    },
    "minuteinvitemessage": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "minuteinvitemessage",
      "reflect": false
    },
    "daynotfoundmessage": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "daynotfoundmessage",
      "reflect": false
    },
    "monthnotfoundmessage": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "monthnotfoundmessage",
      "reflect": false
    },
    "yearnotfoundmessage": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "yearnotfoundmessage",
      "reflect": false
    },
    "hournotfoundmessage": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "hournotfoundmessage",
      "reflect": false
    },
    "minutenotfoundmessage": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "minutenotfoundmessage",
      "reflect": false
    },
    "showmonthas": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "showmonthas",
      "reflect": false
    },
    "fieldcontroltype": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "fieldcontroltype",
      "reflect": false
    },
    "fieldorder": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "fieldorder",
      "reflect": false
    },
    "hourformat": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "hourformat",
      "reflect": false
    },
    "starthour": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "starthour",
      "reflect": false
    },
    "endhour": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "endhour",
      "reflect": false
    },
    "minutestep": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "minutestep",
      "reflect": false
    },
    "value": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Date",
        "resolved": "Date",
        "references": {
          "Date": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "defaultValue": "null"
    },
    "readonlyclass": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "readonlyclass",
      "reflect": false,
      "defaultValue": "''"
    },
    "enabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "enabled",
      "reflect": false,
      "defaultValue": "true"
    },
    "validationerrorcode": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "validationerrorcode",
      "reflect": false,
      "defaultValue": "K2btSplitDatepicker.VALIDATION_ERROR_NOERROR"
    }
  }; }
  static get states() { return {
    "selectedYear": {},
    "selectedMonth": {},
    "selectedDay": {},
    "selectedHour": {},
    "selectedMinute": {}
  }; }
  static get events() { return [{
      "method": "inputEvent",
      "name": "dateinput",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "object",
        "resolved": "object",
        "references": {}
      }
    }, {
      "method": "changeEvent",
      "name": "datechange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "object",
        "resolved": "object",
        "references": {}
      }
    }]; }
}
K2btSplitDatepicker.ORDER_DMY = 'DMY';
K2btSplitDatepicker.ORDER_YMD = 'YMD';
K2btSplitDatepicker.ORDER_MDY = 'MDY';
K2btSplitDatepicker.SHOWMONTHAS_NAME = 'Name';
K2btSplitDatepicker.SHOWMONTHAS_SHORTNAME = 'ShortName';
K2btSplitDatepicker.SHOWMONTHAS_NUMBER = 'Number';
K2btSplitDatepicker.HOURFORMAT_AMPM = 'AM/PM';
K2btSplitDatepicker.HOURFORMAT_24H = '24H';
K2btSplitDatepicker.FIELDCONTROLTYPE_COMBO = 'ComboBox';
K2btSplitDatepicker.FIELDCONTROLTYPE_SUGGEST = 'Suggest';
K2btSplitDatepicker.VALIDATION_ERROR_NOERROR = '0';
K2btSplitDatepicker.VALIDATION_ERROR_EMPTYDAY = '1';
K2btSplitDatepicker.VALIDATION_ERROR_EMPTYMONTH = '2';
K2btSplitDatepicker.VALIDATION_ERROR_EMPTYYEAR = '3';
K2btSplitDatepicker.VALIDATION_ERROR_INVALIDDAYFORMONTH = '4';
K2btSplitDatepicker.VALIDATION_ERROR_DATETOOLOW = '5';
K2btSplitDatepicker.VALIDATION_ERROR_DATETOOHIGH = '6';
K2btSplitDatepicker.VALIDATION_ERROR_EMPTYHOUR = '7';
K2btSplitDatepicker.VALIDATION_ERROR_EMPTYMINUTE = '8';
