import { HTMLElement as HTMLElement$1, createEvent, h, Host, proxyCustomElement } from '@stencil/core/internal/client';
export { setAssetPath, setPlatformOptions } from '@stencil/core/internal/client';

const K2btBaseColorPicker$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.selectionChangedEvent = createEvent(this, "selectionChanged", 7);
    this.selectionErrorEvent = createEvent(this, "selectionError", 7);
    this.columns = 0;
    this.containerclass = '';
    this.maxSelectionSize = 1;
    this.enabled = true;
  }
  getColorRows() {
    var result = [];
    if (this.columns > 0) {
      for (let i = 0; i < this.colorOptions.length; i += this.columns) {
        const row = this.colorOptions.slice(i, i + this.columns);
        result.push(row);
      }
    }
    else {
      result.push(this.colorOptions);
    }
    return result;
  }
  colorIsSelected(c) {
    return this.selectedIds.indexOf(c.id) != -1;
  }
  selectColor(c) {
    debugger;
    if (this.enabled) {
      var id = c.id;
      if (this.selectedIds.includes(id)) {
        // if only one item, it cannot be unselected
        if (this.maxSelectionSize != 1) {
          this.selectedIds = this.selectedIds.filter(i => i != id);
          this.selectionChangedEvent.emit({});
        }
      }
      else {
        // Before selecting this item, check that the amount of items is OK
        if (this.maxSelectionSize == 1) {
          this.selectedIds = [id];
          this.selectionChangedEvent.emit({});
        }
        else if (this.maxSelectionSize == 0 || this.maxSelectionSize > this.selectedIds.length) {
          this.selectedIds = this.selectedIds.concat(id);
          this.selectionChangedEvent.emit({});
        }
        else {
          this.errorCode = K2btBaseColorPicker$1.ERROR_SELECTION_FULL;
          this.selectionErrorEvent.emit({});
        }
      }
    }
  }
  componentWillRender() {
    if (this.maxSelectionSize > 0 && this.selectedIds.length > this.maxSelectionSize)
      this.selectedIds = this.selectedIds.slice(0, this.maxSelectionSize);
  }
  render() {
    var sectionClass = 'K2BT_BaseColorPicker ' + this.containerclass;
    if (!this.enabled)
      sectionClass += ' K2BT_BaseColorPickerDisabled';
    return (h("div", { class: sectionClass }, this.getColorRows().map(r => {
      return (h("div", { class: "K2BT_BaseColorPickerRow" }, r.map(c => {
        var style = { backgroundColor: c.colorCode };
        var itemClass = 'K2BT_BaseColorPickerItem' + (this.colorIsSelected(c) ? ' K2BT_BaseColorPickerItemSelected' : '');
        return h("div", { class: itemClass, style: style, title: c.description, onClick: () => this.selectColor(c) });
      })));
    })));
  }
};
K2btBaseColorPicker$1.ERROR_SELECTION_FULL = 'SELECTION_FULL';

class ControlInfoValue {
  static getAtomicValues_impl(values) {
    var _a;
    return (_a = values === null || values === void 0 ? void 0 : values.map(value => { var _a; return (((_a = value.items) === null || _a === void 0 ? void 0 : _a.length) > 0 ? ControlInfoValue.getAtomicValues_impl(value.items) : [value]); }).reduce((a, b) => a.concat(b), [])) !== null && _a !== void 0 ? _a : [];
  }
  static containsDetails(values) {
    var _a;
    return (_a = (values === null || values === void 0 ? void 0 : values.filter(value => value.detail != '' && value.detail != undefined).length) != 0) !== null && _a !== void 0 ? _a : false;
  }
  static containsIcons(values) {
    var _a;
    return (_a = (values === null || values === void 0 ? void 0 : values.filter(value => value.imageUrl != '' && value.imageUrl != undefined).length) != 0) !== null && _a !== void 0 ? _a : false;
  }
  static containsTrailingText(values) {
    var _a;
    return (_a = (values === null || values === void 0 ? void 0 : values.filter(value => value.trailingText != '' && value.trailingText != undefined).length) != 0) !== null && _a !== void 0 ? _a : false;
  }
  static containsBadges(values) {
    var _a;
    return (_a = (values === null || values === void 0 ? void 0 : values.filter(value => value.badgeClass != '' && value.badgeClass != undefined).length) != 0) !== null && _a !== void 0 ? _a : false;
  }
  static containsTooltip(values) {
    var _a;
    return (_a = (values === null || values === void 0 ? void 0 : values.filter(value => value.tooltip != '' && value.tooltip != undefined).length) != 0) !== null && _a !== void 0 ? _a : false;
  }
}
class ImageRegion {
}
ImageRegion.AVAILABLE = 'AVAILABLE';
ImageRegion.UNAVAILABLE = 'UNAVAILABLE';
class CalendarItem {
  constructor() {
    this.isDraft = false;
  }
  static calendarItemsCollide(item1, item2) {
    var first = item1.dateFrom < item2.dateFrom ? item1 : item2;
    var second = first == item1 ? item2 : item1;
    return second.dateFrom < first.dateTo;
  }
  static isInDay(i, dateFrom, dateTo) {
    // Check if event should be shown on this day
    return ((dateFrom <= i.dateFrom && dateTo > i.dateFrom) ||
      (dateFrom <= i.dateTo && dateTo > i.dateTo) || // Option 1: event starts or ends in this day
      (i.dateFrom < dateFrom && i.dateTo > dateTo) // Option 2: events starts before and ends after this day
    );
  }
  static allDayItemIsInDay(i, day) {
    var from = new Date(day.getFullYear(), day.getMonth(), day.getDate(), 0, 0, 0);
    var to = new Date(day.getFullYear(), day.getMonth(), day.getDate() + 1, 0, 0, 0);
    return this.isInDay(i, from, to);
  }
  static itemIsInPast(item, currentDate) {
    return !item.allDay ? item.dateFrom < currentDate && item.dateTo < currentDate : currentDate > item.dateTo && !this.allDayItemIsInDay(item, currentDate);
  }
  static addItemClasses(classList, item, currentDate) {
    if (item.isDraft)
      classList.push('K2BT_CalendarDayItemDraft');
    if (item.class)
      classList.push(item.class);
    if (CalendarItem.itemIsInPast(item, currentDate))
      classList.push(' K2BT_CalendarDayItemPast');
  }
}
class EnumUtils {
  static getEnumKeyByEnumValue(myEnum, enumValue) {
    let keys = Object.keys(myEnum).filter(x => myEnum[x] == enumValue);
    return keys.length > 0 ? keys[0] : null;
  }
}
class LanguageUtils {
  static getTranslatedMessage(msg) {
    //@ts-ignore
    return k2btools.getTranslatedMessage(msg);
  }
}

const K2btCalendarActionMenu$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.actionClicked = createEvent(this, "actionClicked", 7);
    this.actions = [];
    this.isOpen = false;
  }
  toggleMenu() {
    this.isOpen = !this.isOpen;
  }
  executeAction(action) {
    this.isOpen = false;
    this.actionClicked.emit(action.id);
  }
  closeMenu(ev) {
    if (this.isOpen && this.quickDaySelectorContainer && !this.quickDaySelectorContainer.contains(ev.target))
      this.isOpen = false;
  }
  render() {
    var _a, _b;
    const menuClass = `K2BT_CalendarActionMenu ${this.isOpen ? 'open' : ''}`;
    return ((_b = (_a = this.actions) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) > 0 ? (h("div", { class: menuClass, ref: c => (this.quickDaySelectorContainer = c) }, h("span", { class: "K2BT_CalendarActionMenu_ToggleButton", onClick: e => {
        e.stopPropagation();
        this.toggleMenu();
      } }), h("div", { class: "K2BT_CalendarActionMenuContents" }, this.isOpen && (h("ul", { class: "K2BT_CalendarActionMenu_Dropdown" }, this.actions.map(action => (h("li", { onClick: e => {
        e.stopPropagation();
        this.executeAction(action);
      } }, h("img", { class: 'K2BT_CalendarActionMenu_Image GX_Image_' + action.imageName + '_Class', src: action.imageUrl, alt: LanguageUtils.getTranslatedMessage(action.name) }), h("span", null, LanguageUtils.getTranslatedMessage(action.name)))))))))) : ('');
  }
};

class DateUtils {
  static equalsDate(date1, date2) {
    return date1.getFullYear() === date2.getFullYear() && date1.getMonth() === date2.getMonth() && date1.getDate() === date2.getDate();
  }
  static getDayOfWeekShortStringFromDayNumber(number) {
    //@ts-ignore
    return k2btools.getDayOfWeekShortStringFromDayNumber(number);
  }
  static getDayOfWeekShortString(day) {
    //@ts-ignore
    return k2btools.getDayOfWeekShortString(day);
  }
  static getMonthName(monthNumber) {
    //@ts-ignore
    return k2btools.getMonthName(monthNumber);
  }
  static getDateAtMidnight(date) {
    return new Date(date.getFullYear(), date.getMonth(), date.getDate(), 0, 0, 0);
  }
  static datePeriodInBound(lowerBound, upperBound, periodFrom, periodTo) {
    return ((lowerBound <= periodFrom && upperBound >= periodFrom) ||
      (lowerBound <= periodTo && upperBound >= periodTo) ||
      (periodFrom <= lowerBound && periodTo >= lowerBound) ||
      (periodFrom <= upperBound && periodTo >= upperBound));
  }
  static formatItemDate(dateFrom) {
    if (dateFrom.getMinutes() !== 0)
      return dateFrom.getHours() + ':' + this.ensureTwoDigits(dateFrom.getMinutes());
    else
      return dateFrom.getHours();
  }
  static ensureTwoDigits(arg0) {
    if (arg0 < 10)
      return '0' + arg0;
    else
      return arg0.toString();
  }
  static getWeeksOfMonth(year, month, weekstartday) {
    const firstDayOfMonth = new Date(year, month - 1, 1);
    const lastDayOfMonth = new Date(year, month, 0);
    var startOfFirstWeek = new Date(firstDayOfMonth.getTime());
    if (weekstartday == WeekStartDay.Sunday) {
      startOfFirstWeek.setDate(firstDayOfMonth.getDate() - firstDayOfMonth.getDay());
    }
    else {
      startOfFirstWeek.setDate(firstDayOfMonth.getDate() - firstDayOfMonth.getDay() + 1);
    }
    var endOfLastWeek = new Date(lastDayOfMonth.getTime());
    if (weekstartday == WeekStartDay.Sunday) {
      endOfLastWeek.setDate(lastDayOfMonth.getDate() - lastDayOfMonth.getDay() + 7);
    }
    else {
      endOfLastWeek.setDate(lastDayOfMonth.getDate() - lastDayOfMonth.getDay() + 8);
    }
    const weeks = [];
    var day = new Date(startOfFirstWeek.getTime());
    while (day < endOfLastWeek) {
      if ((weekstartday == WeekStartDay.Sunday && day.getDay() === 0) || (weekstartday == WeekStartDay.Monday && day.getDay() === 1)) {
        weeks.push([]);
      }
      weeks[weeks.length - 1].push(day);
      day = new Date(day.getTime());
      day.setDate(day.getDate() + 1);
    }
    return weeks;
  }
}
var CalendarPeriodView;
(function (CalendarPeriodView) {
  CalendarPeriodView["Day"] = "day";
  CalendarPeriodView["Week"] = "week";
  CalendarPeriodView["WorkWeek"] = "workweek";
  CalendarPeriodView["Month"] = "month";
})(CalendarPeriodView || (CalendarPeriodView = {}));
var WeekStartDay;
(function (WeekStartDay) {
  WeekStartDay["Sunday"] = "Sunday";
  WeekStartDay["Monday"] = "Monday";
})(WeekStartDay || (WeekStartDay = {}));

const K2btCalendarDayInMonthPicker$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.dayClicked = createEvent(this, "dayClicked", 7);
    this.weekstartday = WeekStartDay.Sunday;
  }
  componentWillRender() {
    this.currentMonth = this.selectedDate.getMonth();
    this.currentYear = this.selectedDate.getFullYear();
  }
  render() {
    const currentDate = new Date();
    const monthName = DateUtils.getMonthName(this.currentMonth);
    const weeks = DateUtils.getWeeksOfMonth(this.currentYear, this.currentMonth + 1, this.weekstartday);
    const firstDayOfMonth = new Date(this.currentYear, this.currentMonth, 1);
    const lastDayOfMonth = new Date(this.currentYear, this.currentMonth + 1, 0);
    return (h("div", { class: "K2BT_CalendarPickerContainer" }, h("div", { class: "K2BT_CalendarPickerHeader" }, h("div", { class: "K2BT_CalendarPickerPreviousMonth", onClick: () => this.previousMonth() }), h("div", { class: "K2BT_CalendarPickerCurrentMonth" }, monthName, " ", this.currentYear), h("div", { class: "K2BT_CalendarPickerNextMonth", onClick: () => this.nextMonth() })), h("div", { class: "K2BT_CalendarPickerWeekdays" }, weeks[0]
      .map(d => d.getDay())
      .map(weekdayNum => (h("div", { class: "K2BT_CalendarPickerWeekday" }, DateUtils.getDayOfWeekShortStringFromDayNumber(weekdayNum))))), h("div", { class: "K2BT_CalendarPickerCalendar" }, weeks.map(week => (h("div", { class: "K2BT_CalendarPickerWeek" }, week.map(date => {
      const dayClass = DateUtils.equalsDate(date, currentDate) ? 'K2BT_CalendarPickerDay K2BT_CalendarPickerToday' : 'K2BT_CalendarPickerDay';
      return date >= firstDayOfMonth && date <= lastDayOfMonth ? (h("div", { class: dayClass, onClick: () => this.dayClickHandler(date) }, date.getDate())) : (h("div", { class: "K2BT_CalendarPickerDayPlaceholder" }));
    })))))));
  }
  previousMonth() {
    if (this.currentMonth == 0) {
      this.currentMonth = 11;
      this.currentYear = this.currentYear - 1;
    }
    else {
      this.currentMonth = this.currentMonth - 1;
    }
    this.updateSelectedDate();
  }
  nextMonth() {
    if (this.currentMonth == 11) {
      this.currentMonth = 0;
      this.currentYear = this.currentYear + 1;
    }
    else {
      this.currentMonth = this.currentMonth + 1;
    }
    this.updateSelectedDate();
  }
  updateSelectedDate() {
    this.selectedDate = new Date(this.currentYear, this.currentMonth, 1, 0, 0, 0);
  }
  dayClickHandler(date) {
    if (!date)
      return; // Ignore clicks on empty days
    this.dayClicked.emit(date);
  }
};

const K2btCalendarDayView$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.creatingNewDraft = createEvent(this, "creatingNewDraft", 7);
    this.newDraftCreated = createEvent(this, "newDraftCreated", 7);
    this.draftCancelled = createEvent(this, "draftCancelled", 7);
    this.itemClicked = createEvent(this, "itemClicked", 7);
    this.actionClicked = createEvent(this, "actionClicked", 7);
    this.calendars = [];
    this.year = 2023;
    this.month = 1;
    this.day = 1;
    this.readonly = true;
    this.draftItemDescription = '(no title)';
    this.showhours = true;
    this.alldayrows = 10;
    this.draftItem = null;
    this.currentDate = new Date();
    this.creatingDraft = false;
    this.mouseMovedWhileCreating = false;
    this.draftBaseDate = null;
    this.intervalId = null;
    this.scopedfunction = this.onMouseUp.bind(this);
  }
  cancelDraft() {
    if (this.draftItem != null) {
      this.draftCancelled.emit(this.draftItem);
      this.draftItem = null;
    }
  }
  onMouseDown(event) {
    if (this.readonly) {
      if (this.draftItem) {
        this.draftCancelled.emit(this.draftItem);
        this.draftItem = null;
      }
      var mousePos = this.getMousePosition(event, this.baseGridControl);
      var hour = Math.floor((mousePos.y / this.baseGridControl.offsetHeight) * this.getAmountOfHours() + this.startHour);
      var min = (Math.round(((mousePos.y / this.baseGridControl.offsetHeight) * this.getAmountOfHours() + this.startHour - hour) * 4) / 4) * 60;
      this.draftItem = new CalendarItem();
      this.draftItem.class = '';
      this.draftItem.isDraft = true;
      this.draftItem.description = this.draftItemDescription;
      this.draftItem.dateFrom = new Date(this.year, this.month, this.day, hour, min, 0, 0);
      this.draftItem.dateTo = new Date(this.year, this.month, this.day, hour + 1, min, 0, 0);
      this.creatingDraft = true;
      this.mouseMovedWhileCreating = false;
      this.draftBaseDate = this.draftItem.dateFrom;
      this.creatingNewDraft.emit(null);
    }
  }
  onMouseMove(event) {
    if (this.readonly && this.creatingDraft) {
      this.processMouseMove(event);
    }
  }
  processMouseMove(event) {
    this.mouseMovedWhileCreating = true;
    var mousePos = this.getMousePosition(event, this.baseGridControl);
    var { hour, min } = this.getSelectedHourAndMinuteFromMousePosition(mousePos);
    if (hour > this.draftBaseDate.getHours() || (hour === this.draftBaseDate.getHours() && min > this.draftBaseDate.getMinutes())) {
      // Set draft to previous dates
      if (this.draftItem.dateFrom !== this.draftBaseDate || hour !== this.draftItem.dateTo.getHours() || min !== this.draftItem.dateTo.getMinutes()) {
        this.draftItem = Object.assign({}, this.draftItem);
        this.draftItem.dateFrom = this.draftBaseDate;
        this.draftItem.dateTo = new Date(this.year, this.month, this.day, hour, min, 0, 0);
      }
    }
    else if (hour < this.draftBaseDate.getHours() || (hour === this.draftBaseDate.getHours() && min < this.draftBaseDate.getMinutes())) {
      // Set draft to preceding dates
      if (this.draftItem.dateTo !== this.draftBaseDate || hour !== this.draftItem.dateFrom.getHours() || min !== this.draftItem.dateFrom.getMinutes()) {
        this.draftItem = Object.assign({}, this.draftItem);
        this.draftItem.dateFrom = new Date(this.year, this.month, this.day, hour, min, 0, 0);
        this.draftItem.dateTo = this.draftBaseDate;
      }
    }
  }
  getSelectedHourAndMinuteFromMousePosition(mousePos) {
    var hour = Math.floor((mousePos.y / this.baseGridControl.offsetHeight) * this.getAmountOfHours() + this.startHour);
    var min = (Math.round(((mousePos.y / this.baseGridControl.offsetHeight) * this.getAmountOfHours() + this.startHour - hour) * 4) / 4) * 60;
    if (min === 60) {
      min = 0;
      hour++;
    }
    return { hour, min };
  }
  onMouseUp() {
    if (this.readonly && this.creatingDraft) {
      this.creatingDraft = false;
      this.mouseMovedWhileCreating = false;
      this.draftItem = Object.assign({}, this.draftItem);
      this.newDraftCreated.emit({ item: this.draftItem });
    }
  }
  onItemClicked(item) {
    this.itemClicked.emit({ item: item });
  }
  onActionClicked(item, actionId) {
    this.actionClicked.emit({ item: item, action: actionId });
  }
  getMousePosition(event, div) {
    const rect = div.getBoundingClientRect();
    const scrollTop = div.scrollTop;
    const scrollLeft = div.scrollLeft;
    const clientX = event.clientX;
    const clientY = event.clientY;
    const x = clientX - rect.left + scrollLeft;
    const y = clientY - rect.top + scrollTop;
    return { x, y };
  }
  getDateTo() {
    return new Date(this.year, this.month, this.day, this.endHour, 0, 0, 0);
  }
  getDateFrom() {
    return new Date(this.year, this.month, this.day, this.startHour, 0, 0, 0);
  }
  getAmountOfHours() {
    return this.endHour - this.startHour;
  }
  getBoundedDate(d) {
    if (d > this.getDateTo())
      return this.getDateTo();
    if (d < this.getDateFrom())
      return this.getDateFrom();
    return d;
  }
  render() {
    var day = this.getDay();
    var dayOfWeek = DateUtils.getDayOfWeekShortString(day);
    var dayOfMonth = this.day.toString();
    var hours = [];
    for (var hour = this.startHour; hour < this.endHour; hour++) {
      hours.push(hour);
    }
    var allDayItems = this.getAllDayItems();
    var items = this.getNonAllDayItems();
    if (this.draftItem != null && !this.creatingDraft)
      items.push(this.draftItem);
    var itemRows = this.sortIntoRows(items);
    if (this.draftItem != null && this.mouseMovedWhileCreating) {
      // add the draft item without looking at rows while user is moving the mouse
      itemRows.push([this.draftItem]);
      items.push(this.draftItem);
    }
    var hourPositionStyle = {
      top: ((this.currentDate.getTime() - this.getDateFrom().getTime()) / 1000 / 60 / 60 / this.getAmountOfHours()) * 100 + '%',
      display: this.currentDate >= this.getDateFrom() && this.currentDate <= this.getDateTo() ? 'block' : 'none',
    };
    var itemsViewClasses = 'K2BT_CalendarDayItemsView';
    if (!this.showhours)
      itemsViewClasses += ' K2BT_CalendarDayItemsNoMargin';
    var dayContainerClasses = 'K2BT_CalendarDayViewContainer';
    if (this.currentDate.getFullYear() === this.year && this.currentDate.getMonth() === this.month && this.currentDate.getDate() === this.day)
      dayContainerClasses += ' K2BT_CalendarDayViewToday';
    if (this.draftItem)
      dayContainerClasses += ' K2BT_CalendarDayCreatingDraft';
    return (h("div", { class: dayContainerClasses }, h("div", { class: "K2BT_CalendarDayViewHeader" }, h("div", { class: "K2BT_CalendarDayViewDoW" }, dayOfWeek), h("div", { class: "K2BT_CalendarDayViewDoM" }, dayOfMonth)), h("div", { class: "K2BT_AlldayItems" }, allDayItems.map(item => {
      return this.getAllDayItemContents(item);
    }), this.alldayrows
      ? /* Add padding to all day items view */
        Array.from(Array(this.alldayrows - allDayItems.length).keys()).map(i => (h("div", { class: "K2BT_AlldayItem", style: { visibility: 'hidden' } }, 'Hidden: ' + i)))
      : ''), h("div", { class: "K2BT_CalendarDayContent" }, h("div", { class: "K2BT_CalendarDayViewContainerBaseGrid", ref: c => (this.baseGridControl = c), onMouseDown: this.onMouseDown.bind(this), onMouseMove: this.onMouseMove.bind(this), onMouseUp: this.onMouseUp.bind(this) }, hours.map(hr => (h("div", { class: "K2BT_CalendarDayHour" }, this.showhours ? h("div", { class: "K2BT_CalendarDayHourNumber" }, hr) : '')))), h("div", { class: itemsViewClasses }, items.map(item => {
      return this.getInDayItemContents(item, itemRows);
    })), h("div", { class: "K2BT_CalendarDayHourMarker", style: hourPositionStyle }))));
  }
  getAllDayItemContents(item) {
    var _a, _b;
    var classList = new Array('K2BT_AlldayItem');
    CalendarItem.addItemClasses(classList, item, this.currentDate);
    var calendar = (_a = this.calendars.filter(c => c.id === item.calendarId)[0]) !== null && _a !== void 0 ? _a : this.calendars[0];
    return (h("div", { class: classList.join(' '), onClick: () => this.onItemClicked(item) }, h("div", { class: "K2BT_AllDayItemDescription" }, item.description), h("k2bt-calendar-action-menu", { actions: item.actions, onActionClicked: e => {
        e.stopPropagation();
        this.onActionClicked(item, e.detail);
      } }), h("div", { class: (_b = 'K2BT_CalendarBackground ' + (calendar === null || calendar === void 0 ? void 0 : calendar.class)) !== null && _b !== void 0 ? _b : 'K2BT_Calendar0' })));
  }
  getInDayItemContents(item, itemRows) {
    var _a, _b;
    var offsetHours = (this.getBoundedDate(item.dateFrom).getTime() - this.getDateFrom().getTime()) / 1000 / 60 / 60;
    var lengthHours = (this.getBoundedDate(item.dateTo).getTime() - this.getBoundedDate(item.dateFrom).getTime()) / 1000 / 60 / 60;
    var row = itemRows.filter(r => r.indexOf(item) != -1)[0];
    var style = {
      top: (offsetHours / this.getAmountOfHours()) * 100 + '%',
      height: (lengthHours / this.getAmountOfHours()) * 100 + '%',
      width: null,
      left: null,
    };
    if (row.length > 1) {
      style.width = 'calc(' + 100 / row.length + '% - 2px)';
      style.left = row.indexOf(item) * (100 / row.length) + '%';
    }
    var classList = new Array('K2BT_CalendarDayItem');
    CalendarItem.addItemClasses(classList, item, this.currentDate);
    var calendar = (_a = this.calendars.filter(c => c.id === item.calendarId)[0]) !== null && _a !== void 0 ? _a : this.calendars[0];
    const periodString = DateUtils.formatItemDate(item.dateFrom) + '-' + DateUtils.formatItemDate(item.dateTo);
    return (h("div", { class: classList.join(' '), style: style, onClick: () => this.onItemClicked(item), title: item.description + '\n' + periodString }, h("div", { class: "K2BT_CalendarDayItemContent" }, h("div", { class: "K2BT_CalendarDayItemText" }, h("div", { class: "K2BT_CalendarDayItemHeader" }, h("div", { class: "K2BT_CalendarDayItemDescription" }, item.description)), h("div", { class: "K2BT_CalendarDayItemHourPeriod" }, periodString)), h("k2bt-calendar-action-menu", { actions: item.actions, onActionClicked: e => {
        e.stopPropagation();
        this.onActionClicked(item, e.detail);
      } })), h("div", { class: (_b = 'K2BT_CalendarBackground ' + (calendar === null || calendar === void 0 ? void 0 : calendar.class)) !== null && _b !== void 0 ? _b : 'K2BT_Calendar0' })));
  }
  getNonAllDayItems() {
    return this.calendars
      .map(c => c.items
      .filter(i => !i.allDay && CalendarItem.isInDay(i, this.getDateFrom(), this.getDateTo()))
      .map(i => {
      i.calendarId = c.id;
      return i;
    }))
      .reduce((i1, i2) => i1.concat(i2), []);
  }
  getAllDayItems() {
    return this.calendars
      .map(c => c.items
      .filter(i => i.allDay && CalendarItem.allDayItemIsInDay(i, this.getDay()))
      .map(i => {
      i.calendarId = c.id;
      return i;
    }))
      .reduce((i1, i2) => i1.concat(i2), []);
  }
  componentDidLoad() {
    if (this.readonly)
      document.addEventListener('mouseup', this.scopedfunction);
    this.intervalId = setInterval(function () {
      this.currentDate = new Date();
    }.bind(this), 1000 * 60 * 10); // Every 10 minutes
  }
  disconnectedCallback() {
    if (this.readonly)
      document.removeEventListener('mouseup', this.scopedfunction);
    clearInterval(this.intervalId);
  }
  sortIntoRows(items) {
    items = items.sort((a, b) => a.dateFrom.getTime() - b.dateFrom.getTime());
    const rows = [];
    for (const item of items) {
      // Check if it collides with one of the rows
      var overlaps = false;
      for (const row of rows) {
        for (const columnItem of row) {
          if (CalendarItem.calendarItemsCollide(item, columnItem)) {
            overlaps = true;
            break;
          }
        }
        if (overlaps) {
          row.push(item);
          break;
        }
      }
      if (!overlaps) {
        rows.push([item]);
      }
    }
    const result = [];
    for (let row of rows) {
      result.push([...row].sort((i1, _i2) => (i1.isDraft ? 1 : -1)));
    }
    return result;
  }
  getDay() {
    return new Date(this.year, this.month, this.day, 0, 0, 0, 0);
  }
};

const K2btCalendarFullView$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.newDraftCreated = createEvent(this, "newDraftCreated", 7);
    this.draftCancelled = createEvent(this, "draftCancelled", 7);
    this.itemClicked = createEvent(this, "itemClicked", 7);
    this.fetchAppointmentsForPeriod = createEvent(this, "fetchAppointmentsForPeriod", 7);
    this.showheader = true;
    this.showcalendarnavigation = true;
    this.showperiodnavigation = true;
    this.showviewselection = true;
    this.showcalendarselection = true;
    this.starthour = 8;
    this.endhour = 20;
    this.readonly = true;
    this.todaycaption = 'Today';
    this.daycaption = 'Day';
    this.weekcaption = 'Week';
    this.workweekcaption = 'Work Week';
    this.monthcaption = 'Month';
    this.yearcaption = 'Year';
    this.seemorecaption = 'See more';
    this.itemsperdayinmonthview = 5;
    this.dayviewenabled = true;
    this.weekviewenabled = true;
    this.workweekviewenabled = true;
    this.monthviewenabled = true;
    this.weekstartday = WeekStartDay.Sunday;
    this.calendars = [];
    this.selectedCalendars = null;
    this.advancedSelectorOpen = false;
  }
  setPeriodAppointments(parm) {
    if (this.selectedCalendars == null) {
      this.selectedCalendars = parm.calendars.map(c => c.id);
    }
    this.clearAppointmentsInRange(parm.dateFrom, parm.dateTo);
    this.pruneFarawayItems(parm.dateFrom, parm.dateTo);
    this.mergeCalendars(parm.calendars);
    this.assignCalendarMissingClasses(this.calendars);
    this.calendars = [].concat(this.calendars);
  }
  pruneFarawayItems(dateFrom, dateTo) {
    for (let calendar of this.calendars) {
      if (calendar.items.length > K2btCalendarFullView$1.MAX_STORED_ITEMS) {
        calendar.items.sort((i1, i2) => this.getDistance(i1, dateFrom, dateTo) - this.getDistance(i2, dateFrom, dateTo));
        calendar.items = calendar.items.slice(0, K2btCalendarFullView$1.MAX_STORED_ITEMS);
      }
    }
  }
  // Item is assumed to be outside range, only check relevant bounds
  getDistance(i, dateFrom, dateTo) {
    return Math.min(Math.abs(i.dateTo.getTime() - dateFrom.getTime()), Math.abs(i.dateFrom.getTime() - dateTo.getTime()));
  }
  assignCalendarMissingClasses(calendars) {
    for (var i = 0; i < calendars.length; i++) {
      if (!calendars[i].class)
        calendars[i].class = 'K2BT_Calendar' + i;
    }
  }
  mergeCalendars(appointments) {
    for (let calendar of appointments) {
      var filteredCalendars = this.calendars.filter(c => c.id == calendar.id);
      if (filteredCalendars.length > 0) {
        filteredCalendars[0].items = filteredCalendars[0].items.concat(calendar.items);
      }
      else {
        this.calendars.push(calendar);
      }
    }
  }
  // dateFrom and dateTo should be treated as local date only values (no time)
  clearAppointmentsInRange(dateFrom, dateTo) {
    var lowerBound = dateFrom;
    var upperBound = dateTo;
    this.calendars = this.calendars.map(c => {
      return {
        id: c.id,
        description: c.description,
        class: c.class,
        items: c.items.filter(i => !DateUtils.datePeriodInBound(lowerBound, upperBound, DateUtils.getDateAtMidnight(i.dateFrom), DateUtils.getDateAtMidnight(i.dateTo))),
      };
    });
  }
  componentWillLoad() {
    this.currentview = CalendarPeriodView.Week;
    this.goToToday();
  }
  render() {
    return (h("div", { class: "K2BT_CalendarContainer" }, this.getHeader(), h("div", { class: "K2BT_CalendarFullViewContent" }, this.getCalendarFullViewContent())));
  }
  getHeader() {
    return this.showheader ? (h("div", { class: "K2BT_CalendarFullViewHeader" }, this.getTodayButton(), this.getPreviousNextButtons(), this.getContextTitle(), this.getCalendarSelectionCombobox(), this.getPeriodSelectionCombobox())) : ('');
  }
  getCalendarFullViewContent() {
    switch (this.currentview) {
      case CalendarPeriodView.Day:
        return (h("k2bt-calendar-day-view", { year: this.dateFrom.getFullYear(), month: this.dateFrom.getMonth(), day: this.dateFrom.getDate(), startHour: this.starthour, endHour: this.endhour, readonly: this.readonly, calendars: this.filterSelectedCalendars(this.calendars), showhours: true, alldayrows: 0, ref: c => (this.dayView = c) }));
      case CalendarPeriodView.Week:
      case CalendarPeriodView.WorkWeek:
        return (h("k2bt-calendar-period-view", { dateFrom: this.dateFrom, dateTo: this.dateTo, startHour: this.starthour, endHour: this.endhour, readonly: this.readonly, calendars: this.filterSelectedCalendars(this.calendars), ref: c => (this.weekView = c) }));
      case CalendarPeriodView.Month:
        return (h("k2bt-calendar-month-view", { year: this.dateFrom.getFullYear(), month: this.dateFrom.getMonth() + 1, readonly: this.readonly, calendars: this.filterSelectedCalendars(this.calendars), seemorecaption: this.seemorecaption, itemsperday: this.itemsperdayinmonthview, weekstartday: this.weekstartday,
          //@ts-ignore
          onDayClicked: ev => this.dayClicked(ev.detail.date), ref: c => (this.monthView = c) }));
    }
  }
  filterSelectedCalendars(calendars) {
    return calendars.filter(c => this.selectedCalendars.indexOf(c.id) !== -1);
  }
  calendarSelectionChanged() {
    this.selectedCalendars = this.calendarCombo.value;
  }
  getCalendarSelectionCombobox() {
    if (this.showcalendarselection) {
      var values = this.getAvailableCalendars();
      if (values.length > 1) {
        return (h("div", { class: "K2BT_CalendarCalendarSelectionCombobox" }, h("k2bt-enhancedcombo", { onChange: () => this.calendarSelectionChanged(), includesearch: false, showselectionastags: false, enableadditem: false, includeemptyitem: false, headermaxvisibleitems: 1, values: values, value: this.selectedCalendars, maxSelectionSize: 0, ref: c => (this.calendarCombo = c) })));
      }
    }
    return '';
  }
  getAvailableCalendars() {
    return this.calendars.map(c => {
      return { value: c.id, description: c.description, badgeClass: c.class };
    });
  }
  getPeriodSelectionCombobox() {
    if (this.showviewselection) {
      var values = this.getAvailableContentViews();
      if (values.length > 1) {
        var value = [this.currentview];
        return (h("div", { class: "K2BT_CalendarPeriodSelectionCombobox" }, h("k2bt-enhancedcombo", { onChange: this.periodSelectionChanged.bind(this), includesearch: false, enableadditem: false, includeemptyitem: false, values: values, value: value, ref: c => (this.periodCombo = c) })));
      }
    }
    return '';
  }
  getAvailableContentViews() {
    var values = [];
    if (this.dayviewenabled)
      values.push({
        value: CalendarPeriodView.Day,
        description: this.daycaption,
      });
    if (this.weekviewenabled)
      values.push({
        value: CalendarPeriodView.Week,
        description: this.weekcaption,
      });
    if (this.workweekviewenabled)
      values.push({
        value: CalendarPeriodView.WorkWeek,
        description: this.workweekcaption,
      });
    if (this.monthviewenabled)
      values.push({
        value: CalendarPeriodView.Month,
        description: this.monthcaption,
      });
    return values;
  }
  getContextTitle() {
    return h("div", { class: "K2BT_CalendarHeaderTitle" }, DateUtils.getMonthName(this.dateFrom.getMonth()) + ' ' + this.dateFrom.getFullYear());
  }
  periodSelectionChanged() {
    this.switchView(CalendarPeriodView[EnumUtils.getEnumKeyByEnumValue(CalendarPeriodView, this.periodCombo.value[0])]);
  }
  switchView(view) {
    this.currentview = view;
    this.goToDate(this.dateFrom);
  }
  cancelDraft() {
    var _a;
    (_a = this.dayView) === null || _a === void 0 ? void 0 : _a.cancelDraft();
    if (this.weekView)
      this.weekView.cancelDraft();
  }
  getPreviousNextButtons() {
    return this.showperiodnavigation ? (h("div", { class: "K2BT_CalendarPeriodButtons" }, h("div", { class: "K2BT_CalendarPreviousPeriodButton", onClick: this.goToPreviousPeriod.bind(this) }), h("div", { class: "K2BT_CalendarNextPeriodButton", onClick: this.goToNextPeriod.bind(this) }))) : ('');
  }
  goToNextPeriod() {
    var d = new Date(this.dateFrom.getTime());
    switch (this.currentview) {
      case CalendarPeriodView.Day:
        d.setDate(d.getDate() + 1);
        break;
      case CalendarPeriodView.Week:
      case CalendarPeriodView.WorkWeek:
        d.setDate(d.getDate() + 7);
        break;
      case CalendarPeriodView.Month:
        d = new Date(d.getFullYear(), d.getMonth() + 1, 1);
        break;
    }
    this.goToDate(d);
  }
  goToPreviousPeriod() {
    var d = new Date(this.dateFrom.getTime());
    switch (this.currentview) {
      case CalendarPeriodView.Day:
        d.setDate(d.getDate() - 1);
        break;
      case CalendarPeriodView.Week:
      case CalendarPeriodView.WorkWeek:
        d.setDate(d.getDate() - 7);
        break;
      case CalendarPeriodView.Month:
        d = new Date(d.getFullYear(), d.getMonth() - 1, 1);
        break;
    }
    this.goToDate(d);
  }
  closeMenu(ev) {
    if (this.advancedSelectorOpen && this.quickDaySelectorContainer && !this.quickDaySelectorContainer.contains(ev.target))
      this.advancedSelectorOpen = false;
  }
  getTodayButton() {
    return this.showcalendarnavigation ? (h("div", { class: "K2B_CalendarQuickDaySelectorContainer", ref: c => (this.quickDaySelectorContainer = c) }, h("div", { class: "K2B_CalendarQuickDaySelector" }, h("div", { class: "K2BT_CalendarTodayButton", onClick: this.goToToday.bind(this) }, this.todaycaption), h("div", { class: "K2BT_CalendarAdvancedSelectorToggle", onClick: () => (this.advancedSelectorOpen = !this.advancedSelectorOpen) })), this.getAdvancedSelectorContents())) : ('');
  }
  getAdvancedSelectorContents() {
    return this.advancedSelectorOpen ? (h("div", { class: "K2BT_CalendarAdvancedSelectorContents" }, h("k2bt-calendar-day-in-month-picker", { selectedDate: this.dateFrom, weekstartday: this.weekstartday, onDayClicked: e => {
        this.advancedSelectorOpen = false;
        this.goToDate(e.detail);
      } }))) : ('');
  }
  dayClicked(d) {
    if (this.dayviewenabled) {
      this.switchView(CalendarPeriodView.Day);
      this.goToDate(d);
    }
  }
  goToToday() {
    this.goToDate(new Date());
  }
  goToDate(date) {
    var originalDateFrom = this.dateFrom ? new Date(this.dateFrom.getTime()) : null;
    var originalDateTo = this.dateTo ? new Date(this.dateTo.getTime()) : null;
    switch (this.currentview) {
      case CalendarPeriodView.Day:
        this.dateFrom = date;
        this.dateTo = date;
        break;
      case CalendarPeriodView.Week:
        var d = new Date(date.getTime());
        if (this.weekstartday === WeekStartDay.Sunday) {
          d.setDate(d.getDate() - d.getDay());
        }
        else {
          d.setDate(d.getDate() - d.getDay() + 1);
        }
        this.dateFrom = d;
        d = new Date(date.getTime());
        if (this.weekstartday === WeekStartDay.Sunday) {
          d.setDate(d.getDate() - d.getDay() + 7);
        }
        else {
          d.setDate(d.getDate() - d.getDay() + 8);
        }
        this.dateTo = d;
        break;
      case CalendarPeriodView.WorkWeek:
        d = new Date(date.getTime());
        d.setDate(d.getDate() - d.getDay() + 1);
        this.dateFrom = d;
        d = new Date(date.getTime());
        d.setDate(d.getDate() - d.getDay() + 6);
        this.dateTo = d;
        break;
      case CalendarPeriodView.Month:
        this.dateFrom = new Date(date.getFullYear(), date.getMonth(), 1);
        this.dateTo = new Date(date.getFullYear(), date.getMonth() + 1, 0);
        break;
    }
    if (originalDateFrom != this.dateFrom || originalDateTo != this.dateTo) {
      this.fetchAppointmentsForPeriod.emit({
        dateFrom: this.dateFrom,
        dateTo: this.dateTo,
      });
    }
  }
};
K2btCalendarFullView$1.MAX_STORED_ITEMS = 300;

const K2btCalendarMonthView$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.dayClicked = createEvent(this, "dayClicked", 7);
    this.itemClicked = createEvent(this, "itemClicked", 7);
    this.actionClicked = createEvent(this, "actionClicked", 7);
    this.readonly = true;
    this.calendars = [];
    this.seemorecaption = 'See all';
    this.itemsperday = 5;
    this.weekstartday = WeekStartDay.Sunday;
    this.currentDate = new Date();
    this.intervalId = null;
  }
  onItemClicked(item) {
    this.itemClicked.emit({ item: item });
  }
  onActionClicked(item, actionId) {
    this.actionClicked.emit({ item: item, action: actionId });
  }
  render() {
    var weeks = DateUtils.getWeeksOfMonth(this.year, this.month, this.weekstartday);
    const firstDayOfMonth = new Date(this.year, this.month - 1, 1);
    const lastDayOfMonth = new Date(this.year, this.month, 0);
    return (h("div", { class: "K2BT_CalendarMonthView" }, h("div", { class: "K2BT_CalendarMonthDayNames" }, weeks[0]
      .map(d => d.getDay())
      .map(d => (h("div", { class: "K2BT_CalendarMonthViewDayName" }, DateUtils.getDayOfWeekShortStringFromDayNumber(d))))), weeks.map(w => {
      return (h("div", { class: "K2BT_CalendarMonthViewWeek" }, w.map(d => {
        var year = d.getFullYear();
        var month = d.getMonth();
        var day = d.getDate();
        var filteredAppointments = this.getFilteredAppointments(year, month, day);
        var visibleAppointments = this.itemsperday > 0 ? filteredAppointments.slice(0, this.itemsperday) : filteredAppointments;
        var dayClasses = this.getClassesForDayContainer(d, firstDayOfMonth, lastDayOfMonth);
        var dayNumberContainerClasses = this.getClassesForNumberContainer(year, month, day);
        visibleAppointments.sort((a, b) => a.dateFrom.getTime() - b.dateFrom.getTime());
        return (h("div", { class: dayClasses, onClick: () => this.onDayClicked(d) }, h("div", { class: "K2BT_CalendarMonthDayComponent" }, h("div", { class: dayNumberContainerClasses }, d.getDate()), h("div", { class: "K2BT_CalendarMonthViewDayAppointmentList" }, visibleAppointments.map(item => {
          var _a;
          var calendar = (_a = this.calendars.filter(c => c.id === item.calendarId)[0]) !== null && _a !== void 0 ? _a : this.calendars[0];
          var classList = new Array('K2BT_CalendarMonthViewDayAppointmentBadge');
          classList.push(calendar.class);
          CalendarItem.addItemClasses(classList, item, this.currentDate);
          const periodString = item.allDay ? '' : DateUtils.formatItemDate(item.dateFrom) + '-' + DateUtils.formatItemDate(item.dateTo);
          return (h("div", { class: "K2BT_CalendarMonthViewDayAppointment", onClick: e => {
              e.stopPropagation();
              this.onItemClicked(item);
            }, title: item.description + '\n' + periodString }, h("div", { class: classList.join(' ') }), h("div", { class: "K2BT_CalendarMonthViewDayAppointmentPeriod" }, periodString), h("div", { class: "K2BT_CalendarMonthViewDayAppointmentDescription" }, item.description), h("k2bt-calendar-action-menu", { actions: item.actions, onActionClicked: e => this.onActionClicked(item, e.detail) })));
        }), visibleAppointments.length < filteredAppointments.length ? h("div", { class: "K2BT_CalendarMonthViewDaySeeMore" }, this.seemorecaption) : ''))));
      })));
    })));
  }
  getClassesForNumberContainer(year, month, day) {
    var dayNumberContainerClasses = 'K2BT_CalendarMonthViewDayNumber';
    if (this.currentDate.getFullYear() === year && this.currentDate.getMonth() === month && this.currentDate.getDate() === day)
      dayNumberContainerClasses += ' K2BT_CalendarDayViewToday';
    return dayNumberContainerClasses;
  }
  getClassesForDayContainer(d, firstDayOfMonth, lastDayOfMonth) {
    var dayClasses = 'K2BT_CalendarMonthViewDay';
    if (!(d >= firstDayOfMonth && d <= lastDayOfMonth)) {
      dayClasses += ' K2BT_CalendarMonthViewDayOtherMonth';
    }
    if (d < new Date(this.currentDate.getFullYear(), this.currentDate.getMonth(), this.currentDate.getDate())) {
      dayClasses += ' K2BT_CalendarMonthViewDayPast';
    }
    return dayClasses;
  }
  getFilteredAppointments(year, month, day) {
    return this.calendars
      .map(c => c.items
      .filter(i => (i.dateFrom.getFullYear() == year && i.dateFrom.getMonth() == month && i.dateFrom.getDate() == day) ||
      (i.dateTo.getFullYear() == year && i.dateTo.getMonth() == month && i.dateTo.getDate() == day))
      .map(i => {
      i.calendarId = c.id;
      return i;
    }))
      .reduce((i1, i2) => i1.concat(i2), []);
  }
  componentDidLoad() {
    this.intervalId = setInterval(function () {
      this.currentDate = new Date();
    }.bind(this), 1000 * 60 * 10); // Every 10 minutes
  }
  disconnectedCallback() {
    clearInterval(this.intervalId);
  }
  onDayClicked(day) {
    this.dayClicked.emit({ date: day });
  }
};

const K2btCalendarPeriodView$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.newDraftCreated = createEvent(this, "newDraftCreated", 7);
    this.draftCanceled = createEvent(this, "draftCanceled", 7);
    this.itemClicked = createEvent(this, "itemClicked", 7);
    this.calendars = [];
    this.readonly = true;
    this.draftItemDescription = '(no title)';
    this.draftItem = null;
    this.dayComponents = [];
  }
  async onCreatingNewDraft(event) {
    for (let d of this.dayComponents) {
      if (event.target != d)
        await d.cancelDraft();
    }
  }
  async cancelDraft() {
    for (let d of this.dayComponents) {
      await d.cancelDraft();
    }
  }
  render() {
    var days = [];
    var d = new Date(this.dateFrom.getTime());
    while (d < this.dateTo) {
      days.push({ year: d.getFullYear(), month: d.getMonth(), day: d.getDate() });
      d.setDate(d.getDate() + 1);
    }
    this.dayComponents = [];
    var maxDayItemsInDate = Math.max(...days
      .map(day => new Date(day.year, day.month, day.day, 0, 0, 0, 0))
      .map(date => this.calendars.map(c => c.items.filter(i => i.allDay && CalendarItem.allDayItemIsInDay(i, date))).reduce((i1, i2) => i1.concat(i2), []).length));
    var first = true;
    return (h("div", { class: "K2BT_CalendarPeriod" }, days.map(d => {
      var showhours = first;
      first = false;
      return (h("div", { class: "K2BT_CalendarPeriodDayContainer" }, h("k2bt-calendar-day-view", { year: d.year, month: d.month, day: d.day, startHour: this.startHour, endHour: this.endHour, readonly: this.readonly, calendars: this.calendars, showhours: showhours, ref: c => this.dayComponents.push(c), alldayrows: maxDayItemsInDate, onCreatingNewDraft: this.onCreatingNewDraft.bind(this) })));
    })));
  }
};

const k2btEditCollectionCss = ":host{display:block}.K2BT_EditCollectionItemDelete::after{cursor:pointer}";

const K2btEditCollection$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.inputEvent = createEvent(this, "input", 7);
    this.changeEvent = createEvent(this, "change", 7);
    this.value = [];
    this.maxSelectionSize = 0;
    this.datatype = K2btEditCollection$1.DATATYPE_CHARACTER;
    // used only for character
    this.length = 10;
    // used only for numerics
    this.integers = 5;
    // used only for numerics
    this.decimals = 2;
    this.enabled = true;
    this.inputclass = '';
    this.readonlyclass = '';
    this.invitemessage = '';
    this.itemInputs = [];
    this._triggerChangeDebouncer = null;
  }
  removeItem(index) {
    this.value = this.value.filter((_item, i) => i != index);
    this.triggerChangeEvent();
  }
  onInput(index, _ev) {
    if (index == this.value.length && this.itemInputs[index].value !== '') {
      this.value = this.value.concat(['']);
    }
    this.value[index] = this.itemInputs[index].value;
    _ev.stopPropagation();
    this.inputEvent.emit(null);
  }
  triggerChangeEvent() {
    if (this._triggerChangeDebouncer != null)
      clearTimeout(this._triggerChangeDebouncer);
    this._triggerChangeDebouncer = setTimeout(() => {
      this.changeEvent.emit(null);
    }, 200);
  }
  render() {
    this.itemInputs = [];
    var styleObj = {};
    var maxLength = 0;
    if (this.datatype == K2btEditCollection$1.DATATYPE_NUMERIC) {
      //@ts-ignore
      styleObj.textAlign = 'right';
      maxLength = this.integers + this.decimals + (this.decimals > 0 ? 1 : 0);
    }
    else {
      maxLength = this.length;
    }
    //@ts-ignore
    styleObj.width = maxLength + 'ch';
    const displayedvalues = this.maxSelectionSize == 0 || this.value.length < this.maxSelectionSize ? this.value.concat(['']) : this.value;
    return (h(Host, null, h("div", { class: "K2BT_EditCollection", style: !this.enabled ? {} : { display: 'none' } }, !this.enabled
      ? this.value.map(v => (h("div", { class: "K2BT_EditCollectionItem" }, h("p", { class: "form-control-static" }, h("span", { class: !this.enabled ? this.readonlyclass : this.readonlyclass + ' gx-invisible', "data-gx-readonly": "" }, v)))))
      : ''), h("div", { class: "K2BT_EditCollection", style: this.enabled ? {} : { display: 'none' } }, displayedvalues.map((v, index) => (h("div", { class: "K2BT_EditCollectionItem" }, h("input", { class: 'form-control K2BT_EditCollectionItemInput ' + this.inputclass, type: "text", value: v, style: styleObj, maxLength: maxLength, onInput: ev => this.onInput(index, ev), ref: c => (this.itemInputs[index] = c), onChange: () => this.triggerChangeEvent(), placeholder: this.invitemessage }), h("span", { class: "K2BT_EditCollectionItemDelete", onClick: () => this.removeItem(index) })))))));
  }
  static get style() { return k2btEditCollectionCss; }
};
K2btEditCollection$1.DATATYPE_CHARACTER = 'CHAR';
K2btEditCollection$1.DATATYPE_NUMERIC = 'NUMERIC';

const k2btEnhancedcomboCss = ":host{display:block}k2bt-enhancedcombo:focus-visible{outline:none}.K2BTEnhancedComboContentsOpen{display:flex;flex-direction:column;position:absolute}.K2BTEnhancedComboItems{max-height:300px;overflow:auto;scrollbar-width:thin;scrollbar-color:#d7d7d7 transparent}.K2BTEnhancedComboItems::-webkit-scrollbar{width:6px}.K2BTEnhancedComboItems::-webkit-scrollbar-thumb{background-color:#d7d7d7;border:1px solid transparent;cursor:pointer}.K2BTEnhancedComboContentsClosed{display:none}.K2BTEnhancedComboItem{display:flex;flex-direction:row;align-items:center}.K2BTEnhancedComboItem:hover{cursor:pointer}.K2BTEnhancedComboItem_TextContainer{display:flex;flex-grow:1}.K2BTEnhancedComboSubtitle{font-size:12px}.K2BTEnhancedComboIcon{width:30px;height:30px;object-fit:cover;margin-right:4px}.K2BTEnhancedComboIconInvisible{visibility:hidden}.K2BTEnhancedComboNoItemsFound{width:100%;text-align:center;margin-top:40px;margin-bottom:40px}.K2BTEnhancedComboTrailingText{flex-grow:0}.K2BT_EnhancedComboCheckbox{margin-right:8px;margin-top:6px;pointer-events:none}.K2BTEnhancedComboDisabled{cursor:normal;opacity:0.8}";

const K2btEnhancedcombo$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.selectionErrorEvent = createEvent(this, "selectionError", 7);
    this.focusEvent = createEvent(this, "focus", 7);
    this.inputEvent = createEvent(this, "input", 7);
    this.changeEvent = createEvent(this, "change", 7);
    this.searchChangedEvent = createEvent(this, "searchChanged", 7);
    this.newRecordClickedEvent = createEvent(this, "newRecordClicked", 7);
    this.value = null;
    this.includesearch = true;
    this.includeemptyitem = true;
    this.emptyitemvalue = '';
    this.emptyitemtext = '(none)';
    this.noresultsfoundtext = 'No results found';
    this.enableadditem = true;
    this.additemcaption = 'New record';
    this.clearselectioncaption = 'Clear selection';
    this.selectallcaption = 'Clear selection';
    this.searchfieldplaceholder = 'Search';
    this.searchvalue = '';
    this.enabled = true;
    this.maxSelectionSize = 1;
    this.headermaxvisibleitems = 3;
    this.readonlyclass = '';
    this.showselectionastags = false;
    this.showiconsintags = true;
    this.displayaslist = false;
    this.enableclearselectionaction = false;
    this.enableselectallaction = false;
    this.activeElementValue = null;
    this.showemptyiteminlist = true;
    this.optionswidth = null;
    this.container = null;
    this.selectedElement = null;
    this.searchfield = null;
    this.listContainer = null;
    this._keyboardSearchPrefix = '';
    this._keyboardSearchLastKeystroke = null;
  }
  isCollection() {
    return this.maxSelectionSize != 1;
  }
  processKeydown(event) {
    if (event.key == 'Escape') {
      this.resetPrefixSearch();
      this.open = false;
      event.stopPropagation();
    }
    else if (event.key == 'Enter') {
      this.resetPrefixSearch();
      this.open = !this.open;
      event.stopPropagation();
    }
    else if (this.isCollection()) {
      this.processKeydownForCollection(event);
    }
    else {
      this.processKeydownSingleSelection(event);
    }
  }
  processKeydownForCollection(event) {
    var pos;
    if (this.open) {
      if (event.key == 'ArrowDown') {
        this.processArrowDownForCollection(event, pos);
      }
      else if (event.key == 'ArrowUp') {
        this.processArrowUpForCollection(event, pos);
      }
      else if (event.key === ' ' || event.key === 'Spacebar') {
        this.processSpacebarForCollection(event);
      }
      else {
        this.processCharacterForCollection(event);
      }
    }
  }
  processCharacterForCollection(event) {
    var key = event.key;
    // Check if the key pressed maps to a single character
    if (key.length == 1) {
      if (document.activeElement != this.searchfield) {
        var currentDate = Date.now();
        if (this._keyboardSearchLastKeystroke == null || currentDate - this._keyboardSearchLastKeystroke > 3000) {
          this._keyboardSearchPrefix = '';
        }
        this._keyboardSearchPrefix = this._keyboardSearchPrefix + key;
        var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
        var selectedValue = currentValues.find(v => v.description.toLowerCase().indexOf(this._keyboardSearchPrefix.toLowerCase()) == 0);
        if (selectedValue != null) {
          this.activeElementValue = selectedValue.value.trim();
        }
        this._keyboardSearchLastKeystroke = currentDate;
      }
    }
  }
  processSpacebarForCollection(event) {
    var _a;
    if (((_a = this.activeElementValue) !== null && _a !== void 0 ? _a : '') !== '') {
      this.setValueWithoutClosing(this.activeElementValue, false);
      event.stopPropagation();
      event.preventDefault();
    }
  }
  processArrowUpForCollection(event, pos) {
    this.resetPrefixSearch();
    event.stopPropagation();
    event.preventDefault();
    //up key
    var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
    if (currentValues.length > 0) {
      pos = currentValues.findIndex(v => v.value.trim() == this.activeElementValue);
      if (pos == -1) {
        pos = currentValues.length - 1;
      }
      else {
        pos = (((pos - 1) % currentValues.length) + currentValues.length) % currentValues.length;
      }
      this.activeElementValue = currentValues[pos].value.trim();
    }
  }
  processArrowDownForCollection(event, pos) {
    this.resetPrefixSearch();
    event.stopPropagation();
    event.preventDefault();
    //down key
    var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
    if (currentValues.length > 0) {
      pos = currentValues.findIndex(v => v.value.trim() == this.activeElementValue);
      if (pos == -1) {
        pos = 0;
      }
      else {
        pos = (((pos + 1) % currentValues.length) + currentValues.length) % currentValues.length;
      }
      this.activeElementValue = currentValues[pos].value.trim();
    }
  }
  processKeydownSingleSelection(event) {
    if (event.key == 'ArrowDown') {
      this.processArrowDownForSingleSelection(event);
    }
    else if (event.key == 'ArrowUp') {
      this.processArrowUpForSingleSelection(event);
    }
    else {
      this.processCharacterForSingleSelection(event);
    }
  }
  processCharacterForSingleSelection(event) {
    var key = event.key;
    // Check if the key pressed maps to a single character
    if (key.length == 1) {
      if (document.activeElement != this.searchfield) {
        var currentDate = Date.now();
        if (this._keyboardSearchLastKeystroke == null || currentDate - this._keyboardSearchLastKeystroke > 3000) {
          this._keyboardSearchPrefix = '';
        }
        this._keyboardSearchPrefix = this._keyboardSearchPrefix + key;
        var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
        var selectedValue = currentValues.find(v => v.description.toLowerCase().indexOf(this._keyboardSearchPrefix.toLowerCase()) == 0);
        if (selectedValue != null) {
          this.setValueWithoutClosing(selectedValue.value.trim(), false);
        }
        this._keyboardSearchLastKeystroke = currentDate;
      }
    }
  }
  processArrowUpForSingleSelection(event) {
    this.resetPrefixSearch();
    event.stopPropagation();
    event.preventDefault();
    //up key
    var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
    if (currentValues.length > 0) {
      var pos = currentValues.findIndex(v => v.value.trim() == this.value[0].trim());
      if (pos == -1) {
        this.setValueWithoutClosing(currentValues[currentValues.length - 1].value.trim(), false);
      }
      else {
        pos = (((pos - 1) % currentValues.length) + currentValues.length) % currentValues.length;
        this.setValueWithoutClosing(currentValues[pos].value.trim(), false);
      }
    }
  }
  processArrowDownForSingleSelection(event) {
    this.resetPrefixSearch();
    event.stopPropagation();
    event.preventDefault();
    //down key
    var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
    if (currentValues.length > 0) {
      var pos = currentValues.findIndex(v => v.value.trim() == this.value[0].trim());
      if (pos == -1) {
        this.setValueWithoutClosing(currentValues[0].value.trim(), false);
      }
      else {
        pos = (((pos + 1) % currentValues.length) + currentValues.length) % currentValues.length;
        this.setValueWithoutClosing(currentValues[pos].value.trim(), false);
      }
    }
  }
  onFocusIn() {
    this.resetPrefixSearch();
    this.focusEvent.emit();
  }
  resetPrefixSearch() {
    this._keyboardSearchLastKeystroke = null;
  }
  closeMenu(ev) {
    this.resetPrefixSearch();
    if (this.open && this.container && !this.container.contains(ev.target))
      this.open = false;
  }
  getHeaderContent() {
    if (this.value == null)
      return this.emptyitemtext;
    else {
      var vals = this.getAtomicValues().filter(value => this.valueIsSelected(value));
      if (vals.length > 0) {
        if (this.isCollection() && this.showselectionastags) {
          return this.getHeaderTagsContent(vals);
        }
        else {
          if (vals.length > this.headermaxvisibleitems)
            return [
              vals
                .slice(0, this.headermaxvisibleitems)
                .map(v => this.getSelectedItemSpan(v))
                .reduce((prev, curr) => [prev, h("span", null, ",\u00A0"), curr]),
              h("span", null, "\u00A0(+", vals.length - this.headermaxvisibleitems, ")"),
            ];
          else
            return vals.map(v => this.getSelectedItemSpan(v)).reduce((prev, curr) => [prev, h("span", null, ",\u00A0"), curr]);
        }
      }
      else
        return this.emptyitemtext;
    }
  }
  getSelectedItemSpan(v) {
    return h("span", { class: "K2BT_EnhancedControlInfoSelectedValue" }, v.description);
  }
  getReadonlyContent() {
    if (this.value == null)
      return this.emptyitemtext;
    else {
      var vals = this.getAtomicValues().filter(value => this.valueIsSelected(value));
      if (vals.length > 0)
        return vals.map(v => v.description).join(', ');
      else
        return this.emptyitemtext;
    }
  }
  getHeaderTagsContent(vals) {
    var containsIcons = ControlInfoValue.containsIcons(this.getAtomicValues());
    return vals.map(v => {
      return (h("div", { class: "K2BT_EnhancedComboHeaderTag" }, this.showiconsintags && containsIcons && v.imageUrl !== '' ? h("img", { class: "K2BT_EnhancedComboTagIcon", src: v.imageUrl }) : '', h("span", { class: "K2BT_EnhancedComboHeaderTagDescription" }, v.description), h("span", { class: "K2BT_EnhancedComboHeaderTagDelete", onClick: ev => {
          ev.cancelBubble = true;
          this.setValueWithoutClosing(v.value, false);
        } })));
    });
  }
  valueIsSelected(value) {
    return this.value.filter(v => value.value.trim() == v.toString().trim()).length > 0;
  }
  toggleContentVisibilty() {
    this.resetPrefixSearch();
    this.open = !this.open;
  }
  getRawValues() {
    var result;
    if (this.values == null) {
      result = [];
    }
    else if (typeof this.values === 'string') {
      if (this.values == '')
        result = [];
      else
        result = JSON.parse(this.values);
    }
    else {
      result = [...this.values];
    }
    if (this.includeemptyitem && !this.isCollection()) {
      result.unshift({
        value: '',
        description: this.emptyitemtext,
        imageUrl: '',
        detail: '',
        trailingText: '',
        badgeClass: '',
        items: [],
        tooltip: ''
      });
    }
    return result;
  }
  getFilteredValues() {
    var result = this.getRawValues();
    if (this.includesearch && this.searchvalue != '')
      result = this.getFilteredValues_impl(result);
    return result;
  }
  getFilteredValues_impl(origin) {
    var result = [];
    if (origin != undefined) {
      for (let cv of origin) {
        if (cv.description.toLocaleLowerCase().indexOf(this.searchvalue.toLocaleLowerCase()) != -1 ||
          cv.detail.toLocaleLowerCase().indexOf(this.searchvalue.toLocaleLowerCase()) != -1 ||
          cv.trailingText.toLocaleLowerCase().indexOf(this.searchvalue.toLocaleLowerCase()) != -1) {
          result = result.concat(cv);
        }
        else {
          // Check if child items must be added
          var filteredChildren = this.getFilteredValues_impl(cv.items);
          if (filteredChildren.length > 0) {
            var cv2 = Object.assign({}, cv);
            cv2.items = filteredChildren;
            result = result.concat(cv2);
          }
        }
      }
    }
    return result;
  }
  setValue(value) {
    this.setValueWithoutClosing(value, true);
    this.open = false;
  }
  setValueWithoutClosing(value, triggerChangeEventImmediately) {
    if (this.isCollection()) {
      if (this.value.filter(v => v.toString().trim() === value.toString().trim()).length === 0) {
        if (!this.selectionIsFull())
          this.value = this.value.concat([value]);
        else {
          this.errorCode = K2btEnhancedcombo$1.ERROR_SELECTION_FULL;
          this.selectionErrorEvent.emit({});
        }
      }
      else
        this.value = this.value.filter(v => v.toString().trim() !== value.toString().trim());
    }
    else {
      this.value = [value];
    }
    if (this.setValueDebouncer) {
      clearTimeout(this.setValueDebouncer);
      this.setValueDebouncer = null;
    }
    if (triggerChangeEventImmediately) {
      this.emitChangedEvents(value);
    }
    else {
      this.setValueDebouncer = setTimeout((() => {
        this.setValueDebouncer = null;
        this.emitChangedEvents(value);
      }).bind(this), 200);
    }
  }
  emitChangedEvents(value) {
    this.inputEvent.emit();
    this.changeEvent.emit(value);
  }
  changeSearchValue(value) {
    this.searchvalue = value;
    this.searchChangedEvent.emit({ value });
  }
  onIncludeNewRecordClick() {
    this.open = false;
    this.newRecordClickedEvent.emit(null);
  }
  render() {
    var style = {};
    if (this.optionswidth != null) {
      //@ts-ignore
      style.width = this.optionswidth;
      //@ts-ignore
      style.minWidth = 'initial';
    }
    this.selectedElement = null;
    var listClass = '';
    if (!this.displayaslist)
      listClass = this.open ? 'K2BTEnhancedComboContentsOpen' : 'K2BTEnhancedComboContentsClosed';
    else
      listClass = 'K2BTEnhancedListContents';
    return (h("div", null, h("p", { class: "form-control-static", style: this.enabled ? { display: 'none' } : {} }, h("span", { class: !this.enabled ? this.readonlyclass : this.readonlyclass + ' gx-invisible', "data-gx-readonly": "" }, this.getReadonlyContent())), h("div", { tabindex: "0", class: "K2BTEnhancedCombo", ref: node => (this.container = node), style: !this.enabled ? { display: 'none' } : {} }, this.addHeaderIfNecessary(), h("div", { class: listClass, style: style, ref: c => (this.listContainer = c) }, this.includesearch ? (h("input", { type: "text", class: "form-control K2BTEnhancedComboSearchInput", ref: c => (this.searchfield = c), onInput: event => {
        this.changeSearchValue(event.target.value);
        event.stopPropagation();
      }, onChange: event => event.stopPropagation(), placeholder: this.searchfieldplaceholder })) : (''), this.getCollectionActionsHeader(), h("div", { class: "K2BTEnhancedComboItems" }, this.getComboContent()), this.enableadditem ? (h("span", { class: "K2BTEnhancedComboNewAction", onClick: () => this.onIncludeNewRecordClick() }, this.additemcaption)) : ('')))));
  }
  getCollectionActionsHeader() {
    return this.enableselectallaction || this.enableclearselectionaction ? (h("div", { class: "K2BT_EnhancedComboCollectionActions" }, this.getClearSelectionAction(), this.getSelectAllAction())) : ('');
  }
  getSelectAllAction() {
    return this.enableselectallaction ? (h("span", { class: "K2BT_EnhancedComboSelectAll", onClick: () => this.onSelectAllClick() }, this.selectallcaption)) : ('');
  }
  getClearSelectionAction() {
    return this.enableclearselectionaction ? (h("span", { class: "K2BT_EnhancedComboSelectNone", onClick: () => this.onClearSelectionClick() }, this.clearselectioncaption)) : ('');
  }
  onClearSelectionClick() {
    this.value = [];
    this.emitChangedEvents(this.value);
  }
  onSelectAllClick() {
    this.value = this.getAtomicValues().map(v => v.value);
    this.emitChangedEvents(this.value);
  }
  addHeaderIfNecessary() {
    return !this.displayaslist ? (h("div", { class: "K2BTEnhancedComboHeader", onClick: () => this.toggleContentVisibilty() }, h("div", { class: "K2BTHeaderContent" }, this.getHeaderContent()))) : ('');
  }
  getAtomicValues() {
    return ControlInfoValue.getAtomicValues_impl(this.getRawValues());
  }
  getComboContent() {
    const rawValues = ControlInfoValue.getAtomicValues_impl(this.getRawValues());
    var containsDetails = ControlInfoValue.containsDetails(rawValues);
    var containsIcons = ControlInfoValue.containsIcons(rawValues);
    var containsTrailingText = ControlInfoValue.containsTrailingText(rawValues);
    var containsBadges = ControlInfoValue.containsBadges(rawValues);
    var containsTooltip = ControlInfoValue.containsTooltip(rawValues);
    const filteredValues = !this.open && !this.displayaslist ? [] : this.getFilteredValues();
    if (filteredValues.length > 0)
      return filteredValues.map(item => this.getItemContent(item, containsDetails, containsIcons, containsTrailingText, containsBadges, containsTooltip));
    else
      return h("div", { class: "K2BTEnhancedComboNoItemsFound" }, this.noresultsfoundtext);
  }
  onImageError(e) {
    e.target.classList.add('K2BTEnhancedComboIconInvisible');
  }
  getItemContent(item, containsDetails, containsIcons, containsTrailingText, containsBadges, containsTooltip) {
    var _a;
    if (((_a = item.items) === null || _a === void 0 ? void 0 : _a.length) > 0) {
      return this.getCategoryContent(item, containsDetails, containsIcons, containsTrailingText, containsBadges, containsTooltip);
    }
    else {
      return this.getAtomicItemContent(item, containsIcons, containsTrailingText, containsDetails, containsBadges, containsTooltip);
    }
  }
  getAtomicItemContent(item, containsIcons, containsTrailingText, containsDetails, containsBadges, containsTooltip) {
    var itemClass = 'K2BTEnhancedComboItem';
    if (this.valueIsSelected(item))
      itemClass += ' K2BTEnhancedComboSelected';
    else if (this.selectionIsFull())
      itemClass += ' K2BTEnhancedComboDisabled';
    if (this.activeElementValue === item.value)
      itemClass += ' K2BTEnhancedComboActive';
    var style = item.value === '' && !this.showemptyiteminlist ? { display: 'none' } : {};
    return (h("div", { class: itemClass, style: style, ref: c => {
        if ((!this.isCollection() && this.valueIsSelected(item)) || (this.isCollection() && this.activeElementValue === item.value))
          this.selectedElement = c;
      }, onClick: () => {
        if (this.isCollection())
          this.setValueWithoutClosing(item.value.trim(), true);
        else
          this.setValue(item.value.trim());
      } }, this.addCheckboxIfNecessary(item), this.addIconIfNecessary(containsIcons, item), this.addBadgeIfNecessary(containsBadges, item), this.addMainItemContent(item, containsTrailingText, containsDetails), this.addTooltipIfNecessary(item, containsTooltip)));
  }
  addTooltipIfNecessary(item, containsTooltip) {
    if (!containsTooltip)
      return '';
    //The html bellow tend to the final html in the final solution of fields with tooltips using K2BTools.
    //else return <div class="K2BTEnhancedCombo_TooltipContainer">   <div class="K2BTEnhancedComboTooltipImage"/> <span class="K2BTEnhancedCombo_Tooltip">{item.tooltip}</span></div>
    //This is a temporal solution to complete faster a customer request.
    else
      return h("div", { class: "K2BImage_ContextHelp GX_Image_K2BContextHelp_Class", title: item.tooltip });
  }
  addMainItemContent(item, containsTrailingText, containsDetails) {
    return (h("div", { class: "K2BTEnhancedComboItem_TextColumn" }, h("div", { class: "K2BTEnhancedComboItem_TextContainer" }, h("span", { class: "K2BTEnhancedComboDescription" }, this.getHighlightedText(item.description)), containsTrailingText ? h("span", { class: "K2BTEnhancedComboTrailingText" }, this.getHighlightedText(item.trailingText)) : ''), h("div", { class: "K2BTEnhancedComboItem_TextContainer" }, containsDetails ? h("span", { class: "K2BTEnhancedComboSubtitle" }, this.getHighlightedText(item.detail)) : '')));
  }
  addBadgeIfNecessary(containsBadges, item) {
    const c = 'K2BTEnhancedComboBadge ' + item.badgeClass;
    if (!containsBadges)
      return '';
    else
      return h("div", { class: c });
  }
  addIconIfNecessary(containsIcons, item) {
    if (!containsIcons)
      return '';
    else
      return (h("div", { class: "K2BTEnhancedComboIconContainer" }, h("img", { class: item.imageUrl === '' ? 'K2BTEnhancedComboIcon K2BTEnhancedComboIconInvisible' : 'K2BTEnhancedComboIcon', src: item.imageUrl, onError: e => this.onImageError(e) })));
  }
  addCheckboxIfNecessary(item) {
    return this.isCollection() ? (h("div", { class: "K2BT_EnhancedComboCheckbox" }, h("input", { type: "checkbox", checked: this.valueIsSelected(item), tabIndex: -1, disabled: this.selectionIsFull() && !this.valueIsSelected(item) }), h("label", { htmlFor: "", "data-gx-sr-only": "" }, "\u00A0"))) : ('');
  }
  getCategoryContent(item, containsDetails, containsIcons, containsTrailingText, containsBadges, containsTooltip) {
    return (h("div", { class: "K2BTEnhancedComboCategory" }, h("span", { class: "K2BTEnhancedComboCategoryName" }, item.description), h("div", { class: "K2BTEnchancedComboCategoryContents" }, item.items.map(i => this.getItemContent(i, containsDetails, containsIcons, containsTrailingText, containsBadges, containsTooltip)))));
  }
  selectionIsFull() {
    return this.isCollection() && this.maxSelectionSize != 0 && this.maxSelectionSize <= this.value.length;
  }
  getHighlightedText(originalText) {
    var position = originalText.toLowerCase().indexOf(this.searchvalue.toLowerCase());
    if (this.searchvalue == undefined || this.searchvalue == '' || position == -1) {
      return h("span", null, originalText);
    }
    else {
      return (h("span", null, originalText.substring(0, position), h("span", { class: "K2BTEnhancedComboSearchHighlight" }, originalText.substring(position, position + this.searchvalue.length)), this.getHighlightedText(originalText.substring(position + this.searchvalue.length))));
    }
  }
  componentDidRender() {
    if (this.selectedElement != null) {
      this.selectedElement.scrollIntoView({ block: 'nearest' });
    }
    if (this.listContainer.getBoundingClientRect().right + this.listContainer.getBoundingClientRect().width > window.innerWidth) {
      this.listContainer.style.right = '0px';
    }
    else {
      this.listContainer.style.right = null;
    }
  }
  componentDidLoad() {
    var currentValues = this.getAtomicValues();
    if (!this.isCollection() && !this.includeemptyitem && currentValues.filter(v => this.valueIsSelected(v)).length == 0) {
      if (currentValues.length > 0) {
        this.setValueWithoutClosing(currentValues[0].value.trim(), true);
      }
    }
  }
  static get style() { return k2btEnhancedcomboCss; }
};
K2btEnhancedcombo$1.ERROR_SELECTION_FULL = 'SELECTION_FULL';

const k2btEnhancedsuggestCss = ":host{display:block}k2bt-enhancedsuggest:focus-visible{outline:none}.K2BTEnhancedComboContentsOpen{display:flex;flex-direction:column;position:absolute}.K2BTEnhancedComboItems{max-height:300px;overflow:auto;scrollbar-width:thin;scrollbar-color:#d7d7d7 transparent}.K2BTEnhancedComboItems::-webkit-scrollbar{width:6px}.K2BTEnhancedComboItems::-webkit-scrollbar-thumb{background-color:#d7d7d7;border:1px solid transparent;cursor:pointer}.K2BTEnhancedComboContentsClosed{display:none}.K2BTEnhancedComboItem{display:flex;flex-direction:row;align-items:center}.K2BTEnhancedComboItem:hover{cursor:pointer}.K2BTEnhancedComboItem_TextContainer{display:flex;flex-grow:1}.K2BTEnhancedComboSubtitle{font-size:12px}.K2BTEnhancedComboIcon{width:30px;height:30px;object-fit:cover;margin-right:4px}.K2BTEnhancedComboIconInvisible{visibility:hidden}.K2BTEnhancedComboNoItemsFound{width:100%;text-align:center;margin-top:40px;margin-bottom:40px}.K2BTEnhancedComboTrailingText{flex-grow:0}@keyframes spinner{to{transform:rotate(360deg)}}.K2BT_LoadingSpinner{-webkit-box-sizing:border-box;box-sizing:border-box;position:relative;left:50%;width:20px;height:19px;margin-top:10px;margin-bottom:10px;border-radius:50%;border:2px solid #ccc;border-top-color:#000;-webkit-animation:spinner 0.6s linear infinite;animation:spinner 0.6s linear infinite;margin-left:-10px}.K2BT_EnhancedComboCheckbox{margin-right:8px;margin-top:6px;pointer-events:none}.K2BTEnhancedComboDisabled{cursor:normal;opacity:0.8}.K2BTEnhancedSuggestTags{display:flex;flex-direction:row;flex-wrap:wrap}";

const K2btEnhancedsuggest$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.selectionErrorEvent = createEvent(this, "selectionError", 7);
    this.focusEvent = createEvent(this, "focus", 7);
    this.suggestValueChanged = createEvent(this, "suggestValueChanged", 7);
    this.newRecordClickedEvent = createEvent(this, "newRecordClicked", 7);
    this.inputValueChangedEvent = createEvent(this, "inputValueChanged", 7);
    this.value = [];
    this.noresultsfoundtext = 'No results found';
    this.enableadditem = true;
    this.additemcaption = 'New record';
    this.enabled = true;
    this.readonlyclass = '';
    this.searchvalue = '';
    this.placeholder = '';
    this.selectedValueDescription = '';
    this.maxSelectionSize = 1;
    this.suggestmaxitems = 5;
    this.activeElementValue = null;
    this.showiconsintags = true;
    this.emptyitemtext = '(none)';
    this.clientSideFiltering = K2btEnhancedsuggest$1.FILTERING_ENABLED;
    this.inputSize = 0;
    this.container = null;
    this.selectedElement = null;
    this.searchfield = null;
    // See problem with transactions in display mode in 0022649: Soportar datasource dataprovider en extended suggest
    this.seekSuggestValuesForMissingValueExecuted = false;
    this.seekSuggestValuesDebouncer = null;
  }
  isCollection() {
    return this.maxSelectionSize != 1;
  }
  // @ts-ignore
  selectionIsFull() {
    return this.isCollection() && this.maxSelectionSize != 0 && this.maxSelectionSize <= this.value.length;
  }
  itemIsSelected(item) {
    var _a, _b;
    return ((_b = (_a = this.value) === null || _a === void 0 ? void 0 : _a.filter(selectedItem => selectedItem.toString().trim() == item.value.toString().trim()).length) !== null && _b !== void 0 ? _b : 0) > 0;
  }
  async setFocusToSearch() {
    if (this.searchfield != null)
      this.searchfield.focus();
  }
  async updateDescription() {
    if (!this.isCollection()) {
      const atomicValues = this.getAtomicValues();
      const selectedItems = atomicValues.filter(value => this.itemIsSelected(value));
      const selectedItemCount = selectedItems.length;
      if (selectedItemCount > 0) {
        this.searchvalue = this.getValueDescription();
        this.seekSuggestValuesForMissingValueExecuted = false;
      }
      else {
        if (!this.seekSuggestValuesForMissingValueExecuted) {
          this.inputValueChangedEvent.emit(null);
          this.seekSuggestValuesForMissingValueExecuted = true;
        }
        this.searchvalue = '';
      }
    }
  }
  processInput(event) {
    if (event.target == this.searchfield) {
      this.changeSearchValue(this.searchfield.value);
    }
    event.stopPropagation();
    return false;
  }
  processKeydown(event) {
    if (event.key === 'Escape') {
      this.open = false;
      event.stopPropagation();
    }
    if (event.key === 'Tab')
      this.open = false;
    if (!this.isCollection()) {
      this.processKeydownSingleSelection(event);
    }
    else {
      this.processKeydownCollection(event);
    }
  }
  processKeydownCollection(event) {
    if (event.key == 'Enter') {
      this.processEnterKeyWhenCollection(event);
    }
    else if (this.open) {
      if (event.key == 'ArrowDown') {
        this.processArrowDownKeyWhenCollection(event);
      }
      else if (event.key == 'ArrowUp') {
        this.processArrowUpKeyWhenCollection(event);
      }
      else if (event.key === ' ' || event.key === 'Spacebar') {
        this.processSpacebarKeyWhenCollection(event);
      }
    }
  }
  processEnterKeyWhenCollection(event) {
    if (this.open) {
      this.setValueWithoutClosing(this.activeElementValue);
      this.emitChangedEvent();
    }
    this.open = !this.open;
    event.stopPropagation();
  }
  processSpacebarKeyWhenCollection(event) {
    if (this.activeElementValue !== '' && this.activeElementValue !== null) {
      this.setValueWithoutClosing(this.activeElementValue);
      this.emitChangedEvent();
      event.stopPropagation();
      event.preventDefault();
    }
  }
  processArrowUpKeyWhenCollection(event) {
    event.stopPropagation();
    event.preventDefault();
    var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
    if (currentValues.length > 0) {
      var pos = currentValues.findIndex(v => v.value == this.activeElementValue);
      if (pos == -1) {
        pos = currentValues.length - 1;
      }
      else {
        if (pos === 0)
          pos = -1;
        else
          pos -= 1;
      }
      if (pos !== -1)
        this.activeElementValue = currentValues[pos].value;
      else
        this.activeElementValue = null;
    }
  }
  processArrowDownKeyWhenCollection(event) {
    event.stopPropagation();
    event.preventDefault();
    var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
    if (currentValues.length > 0) {
      var pos = currentValues.findIndex(v => v.value == this.activeElementValue);
      if (pos === -1) {
        pos = 0;
      }
      else {
        pos = (((pos + 1) % currentValues.length) + currentValues.length) % currentValues.length;
        if (pos === 0)
          pos = -1;
      }
      if (pos !== -1)
        this.activeElementValue = currentValues[pos].value;
      else
        this.activeElementValue = null;
    }
  }
  processKeydownSingleSelection(event) {
    if (event.key == 'Enter') {
      this.processEnterKeyWhenNotCollection(event);
    }
    else if (event.key == 'Tab') {
      this.processTabKeyWhenNotCollection(event);
    }
    else if (this.open) {
      if (event.key == 'ArrowDown') {
        this.processArrowDownKeyWhenNotCollection(event);
      }
      else if (event.key == 'ArrowUp') {
        this.processArrowUpKeyWhenNotCollection(event);
      }
    }
  }
  processArrowUpKeyWhenNotCollection(event) {
    event.stopPropagation();
    event.preventDefault();
    var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
    if (currentValues.length > 0) {
      var pos = currentValues.findIndex(v => v.value == this.value[0]);
      if (pos == -1) {
        this.setValueWithoutClosing(currentValues[currentValues.length - 1].value);
      }
      else {
        pos = (((pos - 1) % currentValues.length) + currentValues.length) % currentValues.length;
        this.setValueWithoutClosing(currentValues[pos].value);
      }
    }
  }
  processArrowDownKeyWhenNotCollection(event) {
    event.stopPropagation();
    event.preventDefault();
    var currentValues = ControlInfoValue.getAtomicValues_impl(this.getFilteredValues());
    if (currentValues.length > 0) {
      var pos = currentValues.findIndex(v => v.value == this.value[0]);
      if (pos == -1) {
        this.setValueWithoutClosing(currentValues[0].value);
      }
      else {
        pos = (((pos + 1) % currentValues.length) + currentValues.length) % currentValues.length;
        this.setValueWithoutClosing(currentValues[pos].value);
      }
    }
  }
  processEnterKeyWhenNotCollection(event) {
    this.open = !this.open;
    if (!this.open) {
      this.searchvalue = this.getValueDescription();
      this.emitChangedEvent();
    }
    event.stopPropagation();
  }
  processTabKeyWhenNotCollection(event) {
    if (!this.open) {
      if (this.value.length === 0) {
        var availableValues = this.getFilteredValuesWhenClosed();
        if (availableValues.length === 1)
          this.value = [availableValues[0].value];
      }
      this.searchvalue = this.getValueDescription();
      this.emitChangedEvent();
    }
    event.stopPropagation();
  }
  closeMenu(ev) {
    if (this.open && this.container && !this.container.contains(ev.target)) {
      this.open = false;
      if (!this.isCollection())
        this.setValue(this.value[0]);
    }
  }
  async setValue(value) {
    if (!this.isCollection())
      this.open = false;
    this.setValueWithoutClosing(value);
    if (!this.isCollection())
      this.searchvalue = this.getValueDescription();
    this.emitChangedEvent();
  }
  emitChangedEvent() {
    this.suggestValueChanged.emit(this.value);
  }
  setValueWithoutClosing(value) {
    if (!this.isCollection()) {
      if (value !== null && value !== undefined)
        this.value = [value];
      else
        this.value = [];
      this.selectedValueDescription = this.getValueDescription();
    }
    else {
      if (this.value.filter(v => v.toString().trim() === value.toString().trim()).length === 0) {
        if (!this.selectionIsFull())
          this.value = this.value.concat([value]);
        else {
          this.errorCode = K2btEnhancedsuggest$1.ERROR_SELECTION_FULL;
          this.selectionErrorEvent.emit({});
        }
      }
      else
        this.value = this.value.filter(v => v.toString().trim() !== value.toString().trim());
    }
  }
  getRawValues() {
    var result;
    if (this.values == null) {
      result = [];
    }
    else if (typeof this.values === 'string') {
      if (this.values == '')
        result = [];
      else
        result = JSON.parse(this.values);
    }
    else {
      result = [...this.values];
    }
    return result;
  }
  getFilteredValues() {
    if (!this.open)
      return [];
    // The suggest control should not consider categories
    return this.getFilteredValuesWhenClosed();
  }
  getFilteredValuesWhenClosed() {
    var result = this.getAtomicValues();
    if (this.searchvalue != '')
      result = this.getFilteredValues_impl(result);
    return result;
  }
  getFilteredValues_impl(origin) {
    var result = [];
    if (origin != undefined) {
      for (let cv of origin) {
        if (cv.description.toLocaleLowerCase().indexOf(this.searchvalue.toLocaleLowerCase()) != -1) {
          result = result.concat(cv);
        }
        else {
          // Check if child items must be added
          var filteredChildren = this.getFilteredValues_impl(cv.items);
          if (filteredChildren.length > 0) {
            var cv2 = Object.assign({}, cv);
            cv2.items = filteredChildren;
            result = result.concat(cv2);
          }
        }
      }
    }
    return result;
  }
  onIncludeNewRecordClick() {
    this.open = false;
    this.newRecordClickedEvent.emit(null);
  }
  getAtomicValues() {
    const atomicValues = ControlInfoValue.getAtomicValues_impl(this.getRawValues());
    return this.removeDuplicates(atomicValues);
  }
  removeDuplicates(atomicValues) {
    return atomicValues.filter((value, index, self) => index === self.findIndex(t => t.value === value.value));
  }
  getSuggestPopoverContent() {
    const rawValues = this.getRawValues();
    var containsDetails = ControlInfoValue.containsDetails(rawValues);
    var containsIcons = ControlInfoValue.containsIcons(rawValues);
    var containsTrailingText = ControlInfoValue.containsTrailingText(rawValues);
    if (this.clientSideFiltering == K2btEnhancedsuggest$1.FILTERING_ENABLED) {
      const filteredValues = this.getFilteredValues().slice(0, this.suggestmaxitems);
      if (filteredValues.length > 0)
        return filteredValues.map(item => this.getItemContent(item, containsDetails, containsIcons, containsTrailingText));
      else if (this.waitingInformationFromServer)
        return h("div", { class: "K2BT_LoadingSpinner" });
      else
        return h("div", { class: "K2BTEnhancedComboNoItemsFound" }, this.noresultsfoundtext);
    }
    else {
      const values = this.getAtomicValues().slice(0, this.suggestmaxitems);
      if (this.waitingInformationFromServer)
        return h("div", { class: "K2BT_LoadingSpinner" });
      else if (values.length > 0)
        return values.map(item => this.getItemContent(item, containsDetails, containsIcons, containsTrailingText));
      else
        return h("div", { class: "K2BTEnhancedComboNoItemsFound" }, this.noresultsfoundtext);
    }
  }
  onImageError(e) {
    e.target.classList.add('K2BTEnhancedComboIconInvisible');
  }
  getItemContent(item, containsDetails, containsIcons, containsTrailingText) {
    var _a;
    if (((_a = item.items) === null || _a === void 0 ? void 0 : _a.length) > 0) {
      // Item is a category
      return this.getCategoryContent(item, containsDetails, containsIcons, containsTrailingText);
    }
    else {
      return this.getAtomicItemContent(item, containsIcons, containsTrailingText, containsDetails);
    }
  }
  getAtomicItemContent(item, containsIcons, containsTrailingText, containsDetails) {
    var itemClass = this.getItemClass(item);
    return (h("div", { class: itemClass, ref: c => {
        if (this.itemIsSelected(item) || this.activeElementValue === item.value)
          this.selectedElement = c;
      }, onClick: () => {
        this.setValue(item.value);
      } }, this.addCheckboxIfNecessary(item), this.addIconIfNecessary(containsIcons, item), this.addMainItemContent(item, containsTrailingText, containsDetails)));
  }
  addMainItemContent(item, containsTrailingText, containsDetails) {
    return (h("div", { class: "K2BTEnhancedComboItem_TextColumn" }, h("div", { class: "K2BTEnhancedComboItem_TextContainer" }, h("span", { class: "K2BTEnhancedComboDescription" }, this.getHighlightedText(item.description)), containsTrailingText ? h("span", { class: "K2BTEnhancedComboTrailingText" }, this.getHighlightedText(item.trailingText)) : ''), h("div", { class: "K2BTEnhancedComboItem_TextContainer" }, containsDetails ? h("span", { class: "K2BTEnhancedComboSubtitle" }, this.getHighlightedText(item.detail)) : '')));
  }
  addIconIfNecessary(containsIcons, item) {
    return containsIcons ? (h("div", { class: "K2BTEnhancedComboIconContainer" }, h("img", { class: "K2BTEnhancedComboIcon", src: item.imageUrl, onError: e => this.onImageError(e) }))) : ('');
  }
  addCheckboxIfNecessary(item) {
    return this.isCollection() ? (h("div", { class: "K2BT_EnhancedComboCheckbox" }, h("input", { type: "checkbox", checked: this.itemIsSelected(item), tabIndex: -1, disabled: this.selectionIsFull() && !this.itemIsSelected(item) }), h("label", { htmlFor: "", "data-gx-sr-only": "" }, "\u00A0"))) : ('');
  }
  getItemClass(item) {
    var itemClass = 'K2BTEnhancedComboItem';
    if (this.itemIsSelected(item))
      itemClass += ' K2BTEnhancedComboSelected';
    else if (this.selectionIsFull())
      itemClass += ' K2BTEnhancedComboDisabled';
    if (this.activeElementValue === item.value)
      itemClass += ' K2BTEnhancedComboActive';
    return itemClass;
  }
  getCategoryContent(item, containsDetails, containsIcons, containsTrailingText) {
    return (h("div", { class: "K2BTEnhancedComboCategory" }, h("span", { class: "K2BTEnhancedComboCategoryName" }, item.description), h("div", { class: "K2BTEnchancedComboCategoryContents" }, item.items.map(i => this.getItemContent(i, containsDetails, containsIcons, containsTrailingText)))));
  }
  getHighlightedText(originalText) {
    var position = originalText.toLowerCase().indexOf(this.searchvalue.toLowerCase());
    if (this.clientSideFiltering == K2btEnhancedsuggest$1.FILTERING_DISABLED || this.searchvalue == undefined || this.searchvalue == '' || position == -1) {
      return h("span", null, originalText);
    }
    else {
      return (h("span", null, originalText.substring(0, position), h("span", { class: "K2BTEnhancedComboSearchHighlight" }, originalText.substring(position, position + this.searchvalue.length)), this.getHighlightedText(originalText.substring(position + this.searchvalue.length))));
    }
  }
  onValuesChanged(_newValue, _oldValue) {
    this.waitingInformationFromServer = false;
  }
  changeSearchValue(value) {
    this.searchvalue = value;
    this.open = true;
    this.waitingInformationFromServer = true;
    if (!this.isCollection()) {
      var valueSelected = this.getAtomicValues()
        .map(v => v.value)
        .indexOf(this.value[0]) != -1;
      this.value = [];
      if (valueSelected)
        this.emitChangedEvent();
    }
    if (this.seekSuggestValuesDebouncer) {
      clearTimeout(this.seekSuggestValuesDebouncer);
      this.seekSuggestValuesDebouncer = null;
    }
    this.seekSuggestValuesDebouncer = setTimeout((() => {
      this.seekSuggestValuesDebouncer = null;
      this.inputValueChangedEvent.emit(null);
    }).bind(this), 200);
  }
  getValueDescription() {
    if (this.value.length === 1) {
      var values = this.getAtomicValues();
      var v = values.find(val => val.value == this.value[0]);
      if (v != undefined) {
        return v.description;
      }
      return this.value[0];
    }
    return '';
  }
  getReadonlyValue(headerValues) {
    if (headerValues.length == 0)
      return this.emptyitemtext;
    else {
      return headerValues.map(v => v.description).join(', ');
    }
  }
  componentWillRender() {
    if (this.value == null || this.value == undefined || this.value.length === 0) {
      const valuesMatchingDescription = this.getAtomicValues().filter(item => { var _a; return item.description == ((_a = this.searchfield) === null || _a === void 0 ? void 0 : _a.value); });
      if (valuesMatchingDescription.length > 0) {
        this.setValueWithoutClosing(valuesMatchingDescription[0].value);
      }
    }
  }
  getHeaderTagsContent(vals) {
    var containsIcons = ControlInfoValue.containsIcons(this.getAtomicValues());
    if (this.isCollection())
      return vals.map(v => {
        return (h("div", { class: "K2BT_EnhancedComboHeaderTag" }, this.showiconsintags && containsIcons && v.imageUrl !== '' ? h("img", { class: "K2BT_EnhancedComboTagIcon", src: v.imageUrl }) : '', h("span", { class: "K2BT_EnhancedComboHeaderTagDescription" }, v.description), h("span", { class: "K2BT_EnhancedComboHeaderTagDelete", onClick: ev => {
            ev.cancelBubble = true;
            this.setValueWithoutClosing(v.value);
            this.emitChangedEvent();
          } })));
      });
  }
  render() {
    var itemsSectionStyle = {};
    if (this.inputSize > 0) {
      //@ts-ignore
      itemsSectionStyle.width = this.inputSize + 'ch !important;';
      //@ts-ignore
      itemsSectionStyle.minWidth = 'initial';
    }
    var atomicValues = this.getAtomicValues();
    const headerValues = this.value.map(sv => atomicValues.filter(v => sv.toString().trim() === v.value.trim())[0]).filter(f => f);
    this.selectedElement = null;
    return (h("div", null, h("p", { class: "form-control-static", style: this.enabled ? { display: 'none' } : {} }, h("span", { class: !this.enabled ? this.readonlyclass : this.readonlyclass + ' gx-invisible', "data-gx-readonly": "" }, this.getReadonlyValue(headerValues))), h("div", { class: "K2BTEnhancedSuggest", ref: node => (this.container = node), style: !this.enabled ? { display: 'none' } : {} }, h("div", { class: "K2BTEnhancedSuggestTags" }, this.getHeaderTagsContent(headerValues)), h("input", { type: "text", class: "form-control Attribute_Trn K2BTEnhancedSuggestInput", value: this.searchvalue, size: this.inputSize, onInput: event => {
        event.preventDefault();
        this.changeSearchValue(event.target.value);
        return false;
      }, placeholder: this.placeholder, onClick: () => {
        this.open = true;
        this.activeElementValue = null;
      }, onFocus: () => {
        this.open = true;
        this.focusEvent.emit();
      }, ref: control => (this.searchfield = control) }), h("div", { class: this.open ? 'K2BTEnhancedComboContentsOpen' : 'K2BTEnhancedComboContentsClosed', style: itemsSectionStyle }, h("div", { class: "K2BTEnhancedComboItems" }, this.getSuggestPopoverContent()), this.enableadditem ? (h("span", { class: "K2BTEnhancedComboNewAction", onClick: () => this.onIncludeNewRecordClick() }, this.additemcaption)) : ('')))));
  }
  static get watchers() { return {
    "values": ["onValuesChanged"]
  }; }
  static get style() { return k2btEnhancedsuggestCss; }
};
K2btEnhancedsuggest$1.ERROR_SELECTION_FULL = 'SELECTION_FULL';
K2btEnhancedsuggest$1.FILTERING_ENABLED = 'Enabled';
K2btEnhancedsuggest$1.FILTERING_DISABLED = 'Disabled';

const k2btImageRegionSelectorCss = ":host{display:block}.K2BT_LocationInBlueprintSelectorContainer{position:relative;background-repeat:no-repeat;background-size:100% 100%}.K2BT_LocationInBlueprintSelectorItem{position:absolute;background-repeat:no-repeat;background-size:100% 100%}.K2BT_LocationInBlueprintSelectorItemName{display:none}.K2BT_LocationInBlueprintSelectorContainer .tooltip-inner{white-space:pre-wrap}.K2BT_LocationInBlueprintSelectorItemUnavailable{cursor:initial;pointer-events:none}";

const K2btImageRegionSelector$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.selectionChangedEvent = createEvent(this, "selectionChanged", 7);
    this.selectionErrorEvent = createEvent(this, "selectionError", 7);
    this.maxSelectionSize = 1;
    this.enabled = true;
    this.elementsWithTooltips = [];
    this.elementDivs = [];
  }
  getRegions() {
    var _a, _b;
    return (_b = (_a = this.imageWithRegionsDefinition) === null || _a === void 0 ? void 0 : _a.Regions) !== null && _b !== void 0 ? _b : [];
  }
  itemClicked(id) {
    if (this.enabled) {
      var element = this.getRegions().filter(r => r.id == id)[0];
      if (element.status == ImageRegion.AVAILABLE) {
        if (this.selectedIds.includes(id)) {
          // It is always possible to remove items
          this.selectedIds = this.selectedIds.filter(i => i != id);
          this.selectionChangedEvent.emit({});
        }
        else {
          // Before selecting this item, check that the amount of items is OK
          if (this.maxSelectionSize == 1) {
            this.selectedIds = [id];
            this.selectionChangedEvent.emit({});
          }
          else if (this.maxSelectionSize == 0 || this.maxSelectionSize > this.selectedIds.length) {
            this.selectedIds = this.selectedIds.concat(id);
            this.selectionChangedEvent.emit({});
          }
          else {
            this.errorCode = K2btImageRegionSelector$1.ERROR_SELECTION_FULL;
            this.selectionErrorEvent.emit({});
          }
        }
      }
    }
  }
  componentDidRender() {
    // @ts-ignore
    $(this.elementsWithTooltips).tooltip();
  }
  render() {
    if (this.imageWithRegionsDefinition) {
      return this.renderExistingDefinition();
    }
    else {
      return this.renderNonExistingDefinition();
    }
  }
  renderNonExistingDefinition() {
    return h("div", null, "Image definition not set");
  }
  renderExistingDefinition() {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    this.elementsWithTooltips = [];
    this.elementDivs = [];
    return (h(Host, null, h("div", { class: (_c = 'K2BT_LocationInBlueprintSelectorContainer ' + ((_b = (_a = this.imageWithRegionsDefinition) === null || _a === void 0 ? void 0 : _a.Frame) === null || _b === void 0 ? void 0 : _b.class)) !== null && _c !== void 0 ? _c : '', style: {
        height: (_e = (_d = this.imageWithRegionsDefinition) === null || _d === void 0 ? void 0 : _d.Frame) === null || _e === void 0 ? void 0 : _e.height,
        width: (_g = (_f = this.imageWithRegionsDefinition) === null || _f === void 0 ? void 0 : _f.Frame) === null || _g === void 0 ? void 0 : _g.width,
        backgroundImage: ((_k = (_j = (_h = this.imageWithRegionsDefinition) === null || _h === void 0 ? void 0 : _h.Frame) === null || _j === void 0 ? void 0 : _j.backgroundImageURL) !== null && _k !== void 0 ? _k : '') != '' ? 'url(' + this.imageWithRegionsDefinition.Frame.backgroundImageURL + ')' : null,
      } }, this.getRegions().map(r => this.renderRegion(r)))));
  }
  renderRegion(r) {
    var imageUrl = this.getBestImageForItemStatus(r);
    var style = {
      backgroundImage: '',
      height: r.height,
      width: r.width,
      top: r.top,
      left: r.left,
    };
    return (h("div", { class: 'K2BT_LocationInBlueprintSelectorItem' +
        (this.selectedIds.map(v => v.toString().trim()).includes(r.id.toString().trim()) ? ' K2BT_LocationInBlueprintSelectorItemSelected' : '') +
        (r.status == ImageRegion.UNAVAILABLE || !this.enabled ? ' K2BT_LocationInBlueprintSelectorItemUnavailable' : '') +
        (r.class != '' ? ' ' + r.class : ''), ref: e => this.elementDivs.push({ id: r.id, element: e }), style: style }, this.getMapForItem(r), this.getImage(r, imageUrl), h("span", { class: "K2BT_LocationInBlueprintSelectorItemName" }, r.name)));
  }
  getBestImageForItemStatus(r) {
    if (this.selectedIds.map(v => v.toString().trim()).includes(r.id.toString().trim()) && r.selectedImageURL != '')
      return r.selectedImageURL;
    if ((!this.enabled || r.status == ImageRegion.UNAVAILABLE) && r.unavailableImageURL != '')
      return r.unavailableImageURL;
    return r.availableImageURL;
  }
  getImage(r, imageUrl) {
    return (h("img", { ref: e => {
        if (r.selectableRegionCoordinates == '')
          this.elementsWithTooltips.push(e);
      }, title: r.selectableRegionCoordinates == '' ? this.getTooltipLines(r) : '', src: imageUrl, style: { height: '100%', width: '100%', cursor: r.selectableRegionCoordinates == '' ? 'pointer' : null }, usemap: r.selectableRegionCoordinates != '' ? '#K2BT_Image' + r.id : null, onClick: () => {
        if (r.selectableRegionCoordinates == '')
          this.itemClicked(r.id);
      } }));
  }
  getTooltipLines(r) {
    if (r.tooltipLines == null || r.tooltipLines.length == 0)
      return null;
    else
      return r.tooltipLines.join('\n');
  }
  getMapForItem(r) {
    if (r.selectableRegionCoordinates != '') {
      return (h("map", { name: 'K2BT_Image' + r.id }, h("area", { shape: "poly", coords: r.selectableRegionCoordinates, onClick: () => this.itemClicked(r.id), href: "#", title: this.getTooltipLines(r), ref: c => this.elementsWithTooltips.push(c) }), h("area", { shape: "poly", coords: '0,0,' + r.width + ',0,' + r.width + ',' + r.height + ',0,' + r.height, onClick: e => this.clickBelow(e, r.id), onMouseOver: e => this.bringElementBelowToForeground(e, r.id) })));
    }
  }
  bringElementBelowToForeground(e, regionId) {
    var evt = e || window.event;
    var regionDiv = this.elementDivs.filter(d => d.id == regionId)[0].element;
    regionDiv.style.pointerEvents = 'none';
    // get element at point of click
    var starter = document.elementFromPoint(evt.clientX, evt.clientY);
    var elements = this.elementDivs.filter(div => div.element.contains(starter));
    if (elements.length > 0) {
      console.log('hovering: ' + elements[0].id);
      this.elementDivs.forEach(el => (el.element.style.zIndex = 'initial'));
      elements.forEach(el => (el.element.style.zIndex = '1'));
    }
    // bring back the cursor
    regionDiv.style.pointerEvents = 'auto';
  }
  clickBelow(e, regionId) {
    var evt = e || window.event;
    var regionDiv = this.elementDivs.filter(d => d.id == regionId)[0].element;
    regionDiv.style.pointerEvents = 'none';
    // get element at point of click
    var starter = document.elementFromPoint(evt.clientX, evt.clientY);
    var elements = this.elementDivs.filter(div => div.element.contains(starter));
    if (elements.length > 0)
      this.itemClicked(elements[0].id);
    // bring back the cursor
    regionDiv.style.pointerEvents = 'auto';
  }
  static get style() { return k2btImageRegionSelectorCss; }
};
K2btImageRegionSelector$1.ERROR_SELECTION_FULL = 'SELECTION_FULL';

var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

function getDefaultExportFromCjs (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, basedir, module) {
	return module = {
		path: basedir,
		exports: {},
		require: function (path, base) {
			return commonjsRequire();
		}
	}, fn(module, module.exports), module.exports;
}

function commonjsRequire () {
	throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
}

var inputmask = createCommonjsModule(function (module, exports) {
/*!
 * dist/inputmask
 * https://github.com/RobinHerbots/Inputmask
 * Copyright (c) 2010 - 2023 Robin Herbots
 * Licensed under the MIT license
 * Version: 5.0.8
 */
!function(e, t) {
    module.exports = t();
}("undefined" != typeof self ? self : commonjsGlobal, (function() {
    return function() {
        var e = {
            8741: function(e, t) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.default = void 0;
                var i = !("undefined" == typeof window || !window.document || !window.document.createElement);
                t.default = i;
            },
            3976: function(e, t, i) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.default = void 0;
                var n = i(2839), a = {
                    _maxTestPos: 500,
                    placeholder: "_",
                    optionalmarker: [ "[", "]" ],
                    quantifiermarker: [ "{", "}" ],
                    groupmarker: [ "(", ")" ],
                    alternatormarker: "|",
                    escapeChar: "\\",
                    mask: null,
                    regex: null,
                    oncomplete: function() {},
                    onincomplete: function() {},
                    oncleared: function() {},
                    repeat: 0,
                    greedy: !1,
                    autoUnmask: !1,
                    removeMaskOnSubmit: !1,
                    clearMaskOnLostFocus: !0,
                    insertMode: !0,
                    insertModeVisual: !0,
                    clearIncomplete: !1,
                    alias: null,
                    onKeyDown: function() {},
                    onBeforeMask: null,
                    onBeforePaste: function(e, t) {
                        return "function" == typeof t.onBeforeMask ? t.onBeforeMask.call(this, e, t) : e;
                    },
                    onBeforeWrite: null,
                    onUnMask: null,
                    showMaskOnFocus: !0,
                    showMaskOnHover: !0,
                    onKeyValidation: function() {},
                    skipOptionalPartCharacter: " ",
                    numericInput: !1,
                    rightAlign: !1,
                    undoOnEscape: !0,
                    radixPoint: "",
                    _radixDance: !1,
                    groupSeparator: "",
                    keepStatic: null,
                    positionCaretOnTab: !0,
                    tabThrough: !1,
                    supportsInputType: [ "text", "tel", "url", "password", "search" ],
                    ignorables: [ n.keys.Backspace, n.keys.Tab, n.keys.Pause, n.keys.Escape, n.keys.PageUp, n.keys.PageDown, n.keys.End, n.keys.Home, n.keys.ArrowLeft, n.keys.ArrowUp, n.keys.ArrowRight, n.keys.ArrowDown, n.keys.Insert, n.keys.Delete, n.keys.ContextMenu, n.keys.F1, n.keys.F2, n.keys.F3, n.keys.F4, n.keys.F5, n.keys.F6, n.keys.F7, n.keys.F8, n.keys.F9, n.keys.F10, n.keys.F11, n.keys.F12, n.keys.Process, n.keys.Unidentified, n.keys.Shift, n.keys.Control, n.keys.Alt, n.keys.Tab, n.keys.AltGraph, n.keys.CapsLock ],
                    isComplete: null,
                    preValidation: null,
                    postValidation: null,
                    staticDefinitionSymbol: void 0,
                    jitMasking: !1,
                    nullable: !0,
                    inputEventOnly: !1,
                    noValuePatching: !1,
                    positionCaretOnClick: "lvp",
                    casing: null,
                    inputmode: "text",
                    importDataAttributes: !0,
                    shiftPositions: !0,
                    usePrototypeDefinitions: !0,
                    validationEventTimeOut: 3e3,
                    substitutes: {}
                };
                t.default = a;
            },
            7392: function(e, t) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.default = void 0;
                t.default = {
                    9: {
                        validator: "[0-9\uff10-\uff19]",
                        definitionSymbol: "*"
                    },
                    a: {
                        validator: "[A-Za-z\u0410-\u044f\u0401\u0451\xc0-\xff\xb5]",
                        definitionSymbol: "*"
                    },
                    "*": {
                        validator: "[0-9\uff10-\uff19A-Za-z\u0410-\u044f\u0401\u0451\xc0-\xff\xb5]"
                    }
                };
            },
            253: function(e, t) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.default = function(e, t, i) {
                    if (void 0 === i) return e.__data ? e.__data[t] : null;
                    e.__data = e.__data || {}, e.__data[t] = i;
                };
            },
            3776: function(e, t, i) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.Event = void 0, t.off = function(e, t) {
                    var i, n;
                    f(this[0]) && e && (i = this[0].eventRegistry, n = this[0], e.split(" ").forEach((function(e) {
                        var a = l(e.split("."), 2);
                        (function(e, n) {
                            var a, r, o = [];
                            if (e.length > 0) if (void 0 === t) for (a = 0, r = i[e][n].length; a < r; a++) o.push({
                                ev: e,
                                namespace: n && n.length > 0 ? n : "global",
                                handler: i[e][n][a]
                            }); else o.push({
                                ev: e,
                                namespace: n && n.length > 0 ? n : "global",
                                handler: t
                            }); else if (n.length > 0) for (var s in i) for (var l in i[s]) if (l === n) if (void 0 === t) for (a = 0, 
                            r = i[s][l].length; a < r; a++) o.push({
                                ev: s,
                                namespace: l,
                                handler: i[s][l][a]
                            }); else o.push({
                                ev: s,
                                namespace: l,
                                handler: t
                            });
                            return o;
                        })(a[0], a[1]).forEach((function(e) {
                            var t = e.ev, a = e.handler;
                            !function(e, t, a) {
                                if (e in i == 1) if (n.removeEventListener ? n.removeEventListener(e, a, !1) : n.detachEvent && n.detachEvent("on".concat(e), a), 
                                "global" === t) for (var r in i[e]) i[e][r].splice(i[e][r].indexOf(a), 1); else i[e][t].splice(i[e][t].indexOf(a), 1);
                            }(t, e.namespace, a);
                        }));
                    })));
                    return this;
                }, t.on = function(e, t) {
                    if (f(this[0])) {
                        var i = this[0].eventRegistry, n = this[0];
                        e.split(" ").forEach((function(e) {
                            var a = l(e.split("."), 2), r = a[0], o = a[1];
                            !function(e, a) {
                                n.addEventListener ? n.addEventListener(e, t, !1) : n.attachEvent && n.attachEvent("on".concat(e), t), 
                                i[e] = i[e] || {}, i[e][a] = i[e][a] || [], i[e][a].push(t);
                            }(r, void 0 === o ? "global" : o);
                        }));
                    }
                    return this;
                }, t.trigger = function(e) {
                    var t = arguments;
                    if (f(this[0])) for (var i = this[0].eventRegistry, n = this[0], r = "string" == typeof e ? e.split(" ") : [ e.type ], s = 0; s < r.length; s++) {
                        var l = r[s].split("."), c = l[0], u = l[1] || "global";
                        if (void 0 !== document && "global" === u) {
                            var d, p = {
                                bubbles: !0,
                                cancelable: !0,
                                composed: !0,
                                detail: arguments[1]
                            };
                            if (document.createEvent) {
                                try {
                                    if ("input" === c) p.inputType = "insertText", d = new InputEvent(c, p); else d = new CustomEvent(c, p);
                                } catch (e) {
                                    (d = document.createEvent("CustomEvent")).initCustomEvent(c, p.bubbles, p.cancelable, p.detail);
                                }
                                e.type && (0, a.default)(d, e), n.dispatchEvent(d);
                            } else (d = document.createEventObject()).eventType = c, d.detail = arguments[1], 
                            e.type && (0, a.default)(d, e), n.fireEvent("on" + d.eventType, d);
                        } else if (void 0 !== i[c]) {
                            arguments[0] = arguments[0].type ? arguments[0] : o.default.Event(arguments[0]), 
                            arguments[0].detail = arguments.slice(1);
                            var h = i[c];
                            ("global" === u ? Object.values(h).flat() : h[u]).forEach((function(e) {
                                return e.apply(n, t);
                            }));
                        }
                    }
                    return this;
                };
                var n, a = u(i(600)), r = u(i(9380)), o = u(i(4963)), s = u(i(8741));
                function l(e, t) {
                    return function(e) {
                        if (Array.isArray(e)) return e;
                    }(e) || function(e, t) {
                        var i = null == e ? null : "undefined" != typeof Symbol && e[Symbol.iterator] || e["@@iterator"];
                        if (null != i) {
                            var n, a, r, o, s = [], l = !0, c = !1;
                            try {
                                if (r = (i = i.call(e)).next, 0 === t) {
                                    if (Object(i) !== i) return;
                                    l = !1;
                                } else for (;!(l = (n = r.call(i)).done) && (s.push(n.value), s.length !== t); l = !0) ;
                            } catch (e) {
                                c = !0, a = e;
                            } finally {
                                try {
                                    if (!l && null != i.return && (o = i.return(), Object(o) !== o)) return;
                                } finally {
                                    if (c) throw a;
                                }
                            }
                            return s;
                        }
                    }(e, t) || function(e, t) {
                        if (!e) return;
                        if ("string" == typeof e) return c(e, t);
                        var i = Object.prototype.toString.call(e).slice(8, -1);
                        "Object" === i && e.constructor && (i = e.constructor.name);
                        if ("Map" === i || "Set" === i) return Array.from(e);
                        if ("Arguments" === i || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(i)) return c(e, t);
                    }(e, t) || function() {
                        throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
                    }();
                }
                function c(e, t) {
                    (null == t || t > e.length) && (t = e.length);
                    for (var i = 0, n = new Array(t); i < t; i++) n[i] = e[i];
                    return n;
                }
                function u(e) {
                    return e && e.__esModule ? e : {
                        default: e
                    };
                }
                function f(e) {
                    return e instanceof Element;
                }
                t.Event = n, "function" == typeof r.default.CustomEvent ? t.Event = n = r.default.CustomEvent : s.default && (t.Event = n = function(e, t) {
                    t = t || {
                        bubbles: !1,
                        cancelable: !1,
                        composed: !0,
                        detail: void 0
                    };
                    var i = document.createEvent("CustomEvent");
                    return i.initCustomEvent(e, t.bubbles, t.cancelable, t.detail), i;
                }, n.prototype = r.default.Event.prototype);
            },
            600: function(e, t) {
                function i(e) {
                    return i = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function(e) {
                        return typeof e;
                    } : function(e) {
                        return e && "function" == typeof Symbol && e.constructor === Symbol && e !== Symbol.prototype ? "symbol" : typeof e;
                    }, i(e);
                }
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.default = function e() {
                    var t, n, a, r, o, s, l = arguments[0] || {}, c = 1, u = arguments.length, f = !1;
                    "boolean" == typeof l && (f = l, l = arguments[c] || {}, c++);
                    "object" !== i(l) && "function" != typeof l && (l = {});
                    for (;c < u; c++) if (null != (t = arguments[c])) for (n in t) a = l[n], l !== (r = t[n]) && (f && r && ("[object Object]" === Object.prototype.toString.call(r) || (o = Array.isArray(r))) ? (o ? (o = !1, 
                    s = a && Array.isArray(a) ? a : []) : s = a && "[object Object]" === Object.prototype.toString.call(a) ? a : {}, 
                    l[n] = e(f, s, r)) : void 0 !== r && (l[n] = r));
                    return l;
                };
            },
            4963: function(e, t, i) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.default = void 0;
                var n = s(i(600)), a = s(i(9380)), r = s(i(253)), o = i(3776);
                function s(e) {
                    return e && e.__esModule ? e : {
                        default: e
                    };
                }
                var l = a.default.document;
                function c(e) {
                    return e instanceof c ? e : this instanceof c ? void (null != e && e !== a.default && (this[0] = e.nodeName ? e : void 0 !== e[0] && e[0].nodeName ? e[0] : l.querySelector(e), 
                    void 0 !== this[0] && null !== this[0] && (this[0].eventRegistry = this[0].eventRegistry || {}))) : new c(e);
                }
                c.prototype = {
                    on: o.on,
                    off: o.off,
                    trigger: o.trigger
                }, c.extend = n.default, c.data = r.default, c.Event = o.Event;
                var u = c;
                t.default = u;
            },
            9845: function(e, t, i) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.mobile = t.iphone = t.ie = void 0;
                var n, a = (n = i(9380)) && n.__esModule ? n : {
                    default: n
                };
                var r = a.default.navigator && a.default.navigator.userAgent || "", o = r.indexOf("MSIE ") > 0 || r.indexOf("Trident/") > 0, s = navigator.userAgentData && navigator.userAgentData.mobile || a.default.navigator && a.default.navigator.maxTouchPoints || "ontouchstart" in a.default, l = /iphone/i.test(r);
                t.iphone = l, t.mobile = s, t.ie = o;
            },
            7184: function(e, t) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.default = function(e) {
                    return e.replace(i, "\\$1");
                };
                var i = new RegExp("(\\" + [ "/", ".", "*", "+", "?", "|", "(", ")", "[", "]", "{", "}", "\\", "$", "^" ].join("|\\") + ")", "gim");
            },
            6030: function(e, t, i) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.EventHandlers = void 0;
                var n = i(8711), a = i(2839), r = i(9845), o = i(7215), s = i(7760), l = i(4713);
                function c(e, t) {
                    var i = "undefined" != typeof Symbol && e[Symbol.iterator] || e["@@iterator"];
                    if (!i) {
                        if (Array.isArray(e) || (i = function(e, t) {
                            if (!e) return;
                            if ("string" == typeof e) return u(e, t);
                            var i = Object.prototype.toString.call(e).slice(8, -1);
                            "Object" === i && e.constructor && (i = e.constructor.name);
                            if ("Map" === i || "Set" === i) return Array.from(e);
                            if ("Arguments" === i || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(i)) return u(e, t);
                        }(e)) || t && e && "number" == typeof e.length) {
                            i && (e = i);
                            var n = 0, a = function() {};
                            return {
                                s: a,
                                n: function() {
                                    return n >= e.length ? {
                                        done: !0
                                    } : {
                                        done: !1,
                                        value: e[n++]
                                    };
                                },
                                e: function(e) {
                                    throw e;
                                },
                                f: a
                            };
                        }
                        throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
                    }
                    var r, o = !0, s = !1;
                    return {
                        s: function() {
                            i = i.call(e);
                        },
                        n: function() {
                            var e = i.next();
                            return o = e.done, e;
                        },
                        e: function(e) {
                            s = !0, r = e;
                        },
                        f: function() {
                            try {
                                o || null == i.return || i.return();
                            } finally {
                                if (s) throw r;
                            }
                        }
                    };
                }
                function u(e, t) {
                    (null == t || t > e.length) && (t = e.length);
                    for (var i = 0, n = new Array(t); i < t; i++) n[i] = e[i];
                    return n;
                }
                var f = {
                    keyEvent: function(e, t, i, c, u) {
                        var d = this.inputmask, p = d.opts, h = d.dependencyLib, v = d.maskset, m = this, g = h(m), y = e.key, k = n.caret.call(d, m), b = p.onKeyDown.call(this, e, n.getBuffer.call(d), k, p);
                        if (void 0 !== b) return b;
                        if (y === a.keys.Backspace || y === a.keys.Delete || r.iphone && y === a.keys.BACKSPACE_SAFARI || e.ctrlKey && y === a.keys.x && !("oncut" in m)) e.preventDefault(), 
                        o.handleRemove.call(d, m, y, k), (0, s.writeBuffer)(m, n.getBuffer.call(d, !0), v.p, e, m.inputmask._valueGet() !== n.getBuffer.call(d).join("")); else if (y === a.keys.End || y === a.keys.PageDown) {
                            e.preventDefault();
                            var x = n.seekNext.call(d, n.getLastValidPosition.call(d));
                            n.caret.call(d, m, e.shiftKey ? k.begin : x, x, !0);
                        } else y === a.keys.Home && !e.shiftKey || y === a.keys.PageUp ? (e.preventDefault(), 
                        n.caret.call(d, m, 0, e.shiftKey ? k.begin : 0, !0)) : p.undoOnEscape && y === a.keys.Escape && !0 !== e.altKey ? ((0, s.checkVal)(m, !0, !1, d.undoValue.split("")), g.trigger("click")) : y !== a.keys.Insert || e.shiftKey || e.ctrlKey || void 0 !== d.userOptions.insertMode ? !0 === p.tabThrough && y === a.keys.Tab ? !0 === e.shiftKey ? (k.end = n.seekPrevious.call(d, k.end, !0), 
                        !0 === l.getTest.call(d, k.end - 1).match.static && k.end--, k.begin = n.seekPrevious.call(d, k.end, !0), 
                        k.begin >= 0 && k.end > 0 && (e.preventDefault(), n.caret.call(d, m, k.begin, k.end))) : (k.begin = n.seekNext.call(d, k.begin, !0), 
                        k.end = n.seekNext.call(d, k.begin, !0), k.end < v.maskLength && k.end--, k.begin <= v.maskLength && (e.preventDefault(), 
                        n.caret.call(d, m, k.begin, k.end))) : e.shiftKey || p.insertModeVisual && !1 === p.insertMode && (y === a.keys.ArrowRight ? setTimeout((function() {
                            var e = n.caret.call(d, m);
                            n.caret.call(d, m, e.begin);
                        }), 0) : y === a.keys.ArrowLeft && setTimeout((function() {
                            var e = n.translatePosition.call(d, m.inputmask.caretPos.begin);
                            n.translatePosition.call(d, m.inputmask.caretPos.end);
                            d.isRTL ? n.caret.call(d, m, e + (e === v.maskLength ? 0 : 1)) : n.caret.call(d, m, e - (0 === e ? 0 : 1));
                        }), 0)) : o.isSelection.call(d, k) ? p.insertMode = !p.insertMode : (p.insertMode = !p.insertMode, 
                        n.caret.call(d, m, k.begin, k.begin));
                        return d.isComposing = y == a.keys.Process || y == a.keys.Unidentified, d.ignorable = p.ignorables.includes(y), 
                        f.keypressEvent.call(this, e, t, i, c, u);
                    },
                    keypressEvent: function(e, t, i, r, l) {
                        var c = this.inputmask || this, u = c.opts, f = c.dependencyLib, d = c.maskset, p = c.el, h = f(p), v = e.key;
                        if (!0 === t || e.ctrlKey && e.altKey || !(e.ctrlKey || e.metaKey || c.ignorable)) {
                            if (v) {
                                var m, g = t ? {
                                    begin: l,
                                    end: l
                                } : n.caret.call(c, p);
                                v = u.substitutes[v] || v, d.writeOutBuffer = !0;
                                var y = o.isValid.call(c, g, v, r, void 0, void 0, void 0, t);
                                if (!1 !== y && (n.resetMaskSet.call(c, !0), m = void 0 !== y.caret ? y.caret : n.seekNext.call(c, y.pos.begin ? y.pos.begin : y.pos), 
                                d.p = m), m = u.numericInput && void 0 === y.caret ? n.seekPrevious.call(c, m) : m, 
                                !1 !== i && (setTimeout((function() {
                                    u.onKeyValidation.call(p, v, y);
                                }), 0), d.writeOutBuffer && !1 !== y)) {
                                    var k = n.getBuffer.call(c);
                                    (0, s.writeBuffer)(p, k, m, e, !0 !== t);
                                }
                                if (e.preventDefault(), t) return !1 !== y && (y.forwardPosition = m), y;
                            }
                        } else v === a.keys.Enter && c.undoValue !== c._valueGet(!0) && (c.undoValue = c._valueGet(!0), 
                        setTimeout((function() {
                            h.trigger("change");
                        }), 0));
                    },
                    pasteEvent: function(e) {
                        var t, i = this.inputmask, a = i.opts, r = i._valueGet(!0), o = n.caret.call(i, this);
                        i.isRTL && (t = o.end, o.end = n.translatePosition.call(i, o.begin), o.begin = n.translatePosition.call(i, t));
                        var l = r.substr(0, o.begin), u = r.substr(o.end, r.length);
                        if (l == (i.isRTL ? n.getBufferTemplate.call(i).slice().reverse() : n.getBufferTemplate.call(i)).slice(0, o.begin).join("") && (l = ""), 
                        u == (i.isRTL ? n.getBufferTemplate.call(i).slice().reverse() : n.getBufferTemplate.call(i)).slice(o.end).join("") && (u = ""), 
                        window.clipboardData && window.clipboardData.getData) r = l + window.clipboardData.getData("Text") + u; else {
                            if (!e.clipboardData || !e.clipboardData.getData) return !0;
                            r = l + e.clipboardData.getData("text/plain") + u;
                        }
                        var f = r;
                        if (i.isRTL) {
                            f = f.split("");
                            var d, p = c(n.getBufferTemplate.call(i));
                            try {
                                for (p.s(); !(d = p.n()).done; ) {
                                    var h = d.value;
                                    f[0] === h && f.shift();
                                }
                            } catch (e) {
                                p.e(e);
                            } finally {
                                p.f();
                            }
                            f = f.join("");
                        }
                        if ("function" == typeof a.onBeforePaste) {
                            if (!1 === (f = a.onBeforePaste.call(i, f, a))) return !1;
                            f || (f = r);
                        }
                        (0, s.checkVal)(this, !0, !1, f.toString().split(""), e), e.preventDefault();
                    },
                    inputFallBackEvent: function(e) {
                        var t = this.inputmask, i = t.opts, o = t.dependencyLib;
                        var c, u = this, d = u.inputmask._valueGet(!0), p = (t.isRTL ? n.getBuffer.call(t).slice().reverse() : n.getBuffer.call(t)).join(""), h = n.caret.call(t, u, void 0, void 0, !0);
                        if (p !== d) {
                            if (c = function(e, a, r) {
                                for (var o, s, c, u = e.substr(0, r.begin).split(""), f = e.substr(r.begin).split(""), d = a.substr(0, r.begin).split(""), p = a.substr(r.begin).split(""), h = u.length >= d.length ? u.length : d.length, v = f.length >= p.length ? f.length : p.length, m = "", g = [], y = "~"; u.length < h; ) u.push(y);
                                for (;d.length < h; ) d.push(y);
                                for (;f.length < v; ) f.unshift(y);
                                for (;p.length < v; ) p.unshift(y);
                                var k = u.concat(f), b = d.concat(p);
                                for (s = 0, o = k.length; s < o; s++) switch (c = l.getPlaceholder.call(t, n.translatePosition.call(t, s)), 
                                m) {
                                  case "insertText":
                                    b[s - 1] === k[s] && r.begin == k.length - 1 && g.push(k[s]), s = o;
                                    break;

                                  case "insertReplacementText":
                                  case "deleteContentBackward":
                                    k[s] === y ? r.end++ : s = o;
                                    break;

                                  default:
                                    k[s] !== b[s] && (k[s + 1] !== y && k[s + 1] !== c && void 0 !== k[s + 1] || (b[s] !== c || b[s + 1] !== y) && b[s] !== y ? b[s + 1] === y && b[s] === k[s + 1] ? (m = "insertText", 
                                    g.push(k[s]), r.begin--, r.end--) : k[s] !== c && k[s] !== y && (k[s + 1] === y || b[s] !== k[s] && b[s + 1] === k[s + 1]) ? (m = "insertReplacementText", 
                                    g.push(k[s]), r.begin--) : k[s] === y ? (m = "deleteContentBackward", (n.isMask.call(t, n.translatePosition.call(t, s), !0) || b[s] === i.radixPoint) && r.end++) : s = o : (m = "insertText", 
                                    g.push(k[s]), r.begin--, r.end--));
                                }
                                return {
                                    action: m,
                                    data: g,
                                    caret: r
                                };
                            }(d, p, h), (u.inputmask.shadowRoot || u.ownerDocument).activeElement !== u && u.focus(), 
                            (0, s.writeBuffer)(u, n.getBuffer.call(t)), n.caret.call(t, u, h.begin, h.end, !0), 
                            !r.mobile && t.skipNextInsert && "insertText" === e.inputType && "insertText" === c.action && t.isComposing) return !1;
                            switch ("insertCompositionText" === e.inputType && "insertText" === c.action && t.isComposing ? t.skipNextInsert = !0 : t.skipNextInsert = !1, 
                            c.action) {
                              case "insertText":
                              case "insertReplacementText":
                                c.data.forEach((function(e, i) {
                                    var n = new o.Event("keypress");
                                    n.key = e, t.ignorable = !1, f.keypressEvent.call(u, n);
                                })), setTimeout((function() {
                                    t.$el.trigger("keyup");
                                }), 0);
                                break;

                              case "deleteContentBackward":
                                var v = new o.Event("keydown");
                                v.key = a.keys.Backspace, f.keyEvent.call(u, v);
                                break;

                              default:
                                (0, s.applyInputValue)(u, d), n.caret.call(t, u, h.begin, h.end, !0);
                            }
                            e.preventDefault();
                        }
                    },
                    setValueEvent: function(e) {
                        var t = this.inputmask, i = this, a = e && e.detail ? e.detail[0] : arguments[1];
                        void 0 === a && (a = i.inputmask._valueGet(!0)), (0, s.applyInputValue)(i, a), (e.detail && void 0 !== e.detail[1] || void 0 !== arguments[2]) && n.caret.call(t, i, e.detail ? e.detail[1] : arguments[2]);
                    },
                    focusEvent: function(e) {
                        var t = this.inputmask, i = t.opts, a = null == t ? void 0 : t._valueGet();
                        i.showMaskOnFocus && a !== n.getBuffer.call(t).join("") && (0, s.writeBuffer)(this, n.getBuffer.call(t), n.seekNext.call(t, n.getLastValidPosition.call(t))), 
                        !0 !== i.positionCaretOnTab || !1 !== t.mouseEnter || o.isComplete.call(t, n.getBuffer.call(t)) && -1 !== n.getLastValidPosition.call(t) || f.clickEvent.apply(this, [ e, !0 ]), 
                        t.undoValue = null == t ? void 0 : t._valueGet(!0);
                    },
                    invalidEvent: function(e) {
                        this.inputmask.validationEvent = !0;
                    },
                    mouseleaveEvent: function() {
                        var e = this.inputmask, t = e.opts, i = this;
                        e.mouseEnter = !1, t.clearMaskOnLostFocus && (i.inputmask.shadowRoot || i.ownerDocument).activeElement !== i && (0, s.HandleNativePlaceholder)(i, e.originalPlaceholder);
                    },
                    clickEvent: function(e, t) {
                        var i = this.inputmask;
                        i.clicked++;
                        var a = this;
                        if ((a.inputmask.shadowRoot || a.ownerDocument).activeElement === a) {
                            var r = n.determineNewCaretPosition.call(i, n.caret.call(i, a), t);
                            void 0 !== r && n.caret.call(i, a, r);
                        }
                    },
                    cutEvent: function(e) {
                        var t = this.inputmask, i = t.maskset, r = this, l = n.caret.call(t, r), c = t.isRTL ? n.getBuffer.call(t).slice(l.end, l.begin) : n.getBuffer.call(t).slice(l.begin, l.end), u = t.isRTL ? c.reverse().join("") : c.join("");
                        window.navigator.clipboard ? window.navigator.clipboard.writeText(u) : window.clipboardData && window.clipboardData.getData && window.clipboardData.setData("Text", u), 
                        o.handleRemove.call(t, r, a.keys.Delete, l), (0, s.writeBuffer)(r, n.getBuffer.call(t), i.p, e, t.undoValue !== t._valueGet(!0));
                    },
                    blurEvent: function(e) {
                        var t = this.inputmask, i = t.opts, a = t.dependencyLib;
                        t.clicked = 0;
                        var r = a(this), l = this;
                        if (l.inputmask) {
                            (0, s.HandleNativePlaceholder)(l, t.originalPlaceholder);
                            var c = l.inputmask._valueGet(), u = n.getBuffer.call(t).slice();
                            "" !== c && (i.clearMaskOnLostFocus && (-1 === n.getLastValidPosition.call(t) && c === n.getBufferTemplate.call(t).join("") ? u = [] : s.clearOptionalTail.call(t, u)), 
                            !1 === o.isComplete.call(t, u) && (setTimeout((function() {
                                r.trigger("incomplete");
                            }), 0), i.clearIncomplete && (n.resetMaskSet.call(t), u = i.clearMaskOnLostFocus ? [] : n.getBufferTemplate.call(t).slice())), 
                            (0, s.writeBuffer)(l, u, void 0, e)), t.undoValue !== t._valueGet(!0) && (t.undoValue = t._valueGet(!0), 
                            r.trigger("change"));
                        }
                    },
                    mouseenterEvent: function() {
                        var e = this.inputmask, t = e.opts.showMaskOnHover, i = this;
                        if (e.mouseEnter = !0, (i.inputmask.shadowRoot || i.ownerDocument).activeElement !== i) {
                            var a = (e.isRTL ? n.getBufferTemplate.call(e).slice().reverse() : n.getBufferTemplate.call(e)).join("");
                            t && (0, s.HandleNativePlaceholder)(i, a);
                        }
                    },
                    submitEvent: function() {
                        var e = this.inputmask, t = e.opts;
                        e.undoValue !== e._valueGet(!0) && e.$el.trigger("change"), -1 === n.getLastValidPosition.call(e) && e._valueGet && e._valueGet() === n.getBufferTemplate.call(e).join("") && e._valueSet(""), 
                        t.clearIncomplete && !1 === o.isComplete.call(e, n.getBuffer.call(e)) && e._valueSet(""), 
                        t.removeMaskOnSubmit && (e._valueSet(e.unmaskedvalue(), !0), setTimeout((function() {
                            (0, s.writeBuffer)(e.el, n.getBuffer.call(e));
                        }), 0));
                    },
                    resetEvent: function() {
                        var e = this.inputmask;
                        e.refreshValue = !0, setTimeout((function() {
                            (0, s.applyInputValue)(e.el, e._valueGet(!0));
                        }), 0);
                    }
                };
                t.EventHandlers = f;
            },
            9716: function(e, t, i) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.EventRuler = void 0;
                var n, a = (n = i(2394)) && n.__esModule ? n : {
                    default: n
                }, r = i(2839), o = i(8711), s = i(7760);
                var l = {
                    on: function(e, t, i) {
                        var n = e.inputmask.dependencyLib, l = function(t) {
                            t.originalEvent && (t = t.originalEvent || t, arguments[0] = t);
                            var l, c = this, u = c.inputmask, f = u ? u.opts : void 0;
                            if (void 0 === u && "FORM" !== this.nodeName) {
                                var d = n.data(c, "_inputmask_opts");
                                n(c).off(), d && new a.default(d).mask(c);
                            } else {
                                if ([ "submit", "reset", "setvalue" ].includes(t.type) || "FORM" === this.nodeName || !(c.disabled || c.readOnly && !("keydown" === t.type && t.ctrlKey && t.key === r.keys.c || !1 === f.tabThrough && t.key === r.keys.Tab))) {
                                    switch (t.type) {
                                      case "input":
                                        if (!0 === u.skipInputEvent) return u.skipInputEvent = !1, t.preventDefault();
                                        break;

                                      case "click":
                                      case "focus":
                                        return u.validationEvent ? (u.validationEvent = !1, e.blur(), (0, s.HandleNativePlaceholder)(e, (u.isRTL ? o.getBufferTemplate.call(u).slice().reverse() : o.getBufferTemplate.call(u)).join("")), 
                                        setTimeout((function() {
                                            e.focus();
                                        }), f.validationEventTimeOut), !1) : (l = arguments, void setTimeout((function() {
                                            e.inputmask && i.apply(c, l);
                                        }), 0));
                                    }
                                    var p = i.apply(c, arguments);
                                    return !1 === p && (t.preventDefault(), t.stopPropagation()), p;
                                }
                                t.preventDefault();
                            }
                        };
                        [ "submit", "reset" ].includes(t) ? (l = l.bind(e), null !== e.form && n(e.form).on(t, l)) : n(e).on(t, l), 
                        e.inputmask.events[t] = e.inputmask.events[t] || [], e.inputmask.events[t].push(l);
                    },
                    off: function(e, t) {
                        if (e.inputmask && e.inputmask.events) {
                            var i = e.inputmask.dependencyLib, n = e.inputmask.events;
                            for (var a in t && ((n = [])[t] = e.inputmask.events[t]), n) {
                                for (var r = n[a]; r.length > 0; ) {
                                    var o = r.pop();
                                    [ "submit", "reset" ].includes(a) ? null !== e.form && i(e.form).off(a, o) : i(e).off(a, o);
                                }
                                delete e.inputmask.events[a];
                            }
                        }
                    }
                };
                t.EventRuler = l;
            },
            219: function(e, t, i) {
                var n = d(i(2394)), a = i(2839), r = d(i(7184)), o = i(8711), s = i(4713);
                function l(e, t) {
                    return function(e) {
                        if (Array.isArray(e)) return e;
                    }(e) || function(e, t) {
                        var i = null == e ? null : "undefined" != typeof Symbol && e[Symbol.iterator] || e["@@iterator"];
                        if (null != i) {
                            var n, a, r, o, s = [], l = !0, c = !1;
                            try {
                                if (r = (i = i.call(e)).next, 0 === t) {
                                    if (Object(i) !== i) return;
                                    l = !1;
                                } else for (;!(l = (n = r.call(i)).done) && (s.push(n.value), s.length !== t); l = !0) ;
                            } catch (e) {
                                c = !0, a = e;
                            } finally {
                                try {
                                    if (!l && null != i.return && (o = i.return(), Object(o) !== o)) return;
                                } finally {
                                    if (c) throw a;
                                }
                            }
                            return s;
                        }
                    }(e, t) || function(e, t) {
                        if (!e) return;
                        if ("string" == typeof e) return c(e, t);
                        var i = Object.prototype.toString.call(e).slice(8, -1);
                        "Object" === i && e.constructor && (i = e.constructor.name);
                        if ("Map" === i || "Set" === i) return Array.from(e);
                        if ("Arguments" === i || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(i)) return c(e, t);
                    }(e, t) || function() {
                        throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
                    }();
                }
                function c(e, t) {
                    (null == t || t > e.length) && (t = e.length);
                    for (var i = 0, n = new Array(t); i < t; i++) n[i] = e[i];
                    return n;
                }
                function u(e) {
                    return u = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function(e) {
                        return typeof e;
                    } : function(e) {
                        return e && "function" == typeof Symbol && e.constructor === Symbol && e !== Symbol.prototype ? "symbol" : typeof e;
                    }, u(e);
                }
                function f(e, t) {
                    for (var i = 0; i < t.length; i++) {
                        var n = t[i];
                        n.enumerable = n.enumerable || !1, n.configurable = !0, "value" in n && (n.writable = !0), 
                        Object.defineProperty(e, (a = n.key, r = void 0, r = function(e, t) {
                            if ("object" !== u(e) || null === e) return e;
                            var i = e[Symbol.toPrimitive];
                            if (void 0 !== i) {
                                var n = i.call(e, t || "default");
                                if ("object" !== u(n)) return n;
                                throw new TypeError("@@toPrimitive must return a primitive value.");
                            }
                            return ("string" === t ? String : Number)(e);
                        }(a, "string"), "symbol" === u(r) ? r : String(r)), n);
                    }
                    var a, r;
                }
                function d(e) {
                    return e && e.__esModule ? e : {
                        default: e
                    };
                }
                var p = n.default.dependencyLib, h = function() {
                    function e(t, i, n) {
                        !function(e, t) {
                            if (!(e instanceof t)) throw new TypeError("Cannot call a class as a function");
                        }(this, e), this.mask = t, this.format = i, this.opts = n, this._date = new Date(1, 0, 1), 
                        this.initDateObject(t, this.opts);
                    }
                    var t, i;
                    return t = e, (i = [ {
                        key: "date",
                        get: function() {
                            return void 0 === this._date && (this._date = new Date(1, 0, 1), this.initDateObject(void 0, this.opts)), 
                            this._date;
                        }
                    }, {
                        key: "initDateObject",
                        value: function(e, t) {
                            var i;
                            for (P(t).lastIndex = 0; i = P(t).exec(this.format); ) {
                                var n = new RegExp("\\d+$").exec(i[0]), a = n ? i[0][0] + "x" : i[0], r = void 0;
                                if (void 0 !== e) {
                                    if (n) {
                                        var o = P(t).lastIndex, s = E(i.index, t);
                                        P(t).lastIndex = o, r = e.slice(0, e.indexOf(s.nextMatch[0]));
                                    } else r = e.slice(0, g[a] && g[a][4] || a.length);
                                    e = e.slice(r.length);
                                }
                                Object.prototype.hasOwnProperty.call(g, a) && this.setValue(this, r, a, g[a][2], g[a][1]);
                            }
                        }
                    }, {
                        key: "setValue",
                        value: function(e, t, i, n, a) {
                            if (void 0 !== t && (e[n] = "ampm" === n ? t : t.replace(/[^0-9]/g, "0"), e["raw" + n] = t.replace(/\s/g, "_")), 
                            void 0 !== a) {
                                var r = e[n];
                                ("day" === n && 29 === parseInt(r) || "month" === n && 2 === parseInt(r)) && (29 !== parseInt(e.day) || 2 !== parseInt(e.month) || "" !== e.year && void 0 !== e.year || e._date.setFullYear(2012, 1, 29)), 
                                "day" === n && (m = !0, 0 === parseInt(r) && (r = 1)), "month" === n && (m = !0), 
                                "year" === n && (m = !0, r.length < 4 && (r = M(r, 4, !0))), "" === r || isNaN(r) || a.call(e._date, r), 
                                "ampm" === n && a.call(e._date, r);
                            }
                        }
                    }, {
                        key: "reset",
                        value: function() {
                            this._date = new Date(1, 0, 1);
                        }
                    }, {
                        key: "reInit",
                        value: function() {
                            this._date = void 0, this.date;
                        }
                    } ]) && f(t.prototype, i), Object.defineProperty(t, "prototype", {
                        writable: !1
                    }), e;
                }(), v = (new Date).getFullYear(), m = !1, g = {
                    d: [ "[1-9]|[12][0-9]|3[01]", Date.prototype.setDate, "day", Date.prototype.getDate ],
                    dd: [ "0[1-9]|[12][0-9]|3[01]", Date.prototype.setDate, "day", function() {
                        return M(Date.prototype.getDate.call(this), 2);
                    } ],
                    ddd: [ "" ],
                    dddd: [ "" ],
                    m: [ "[1-9]|1[012]", function(e) {
                        var t = e ? parseInt(e) : 0;
                        return t > 0 && t--, Date.prototype.setMonth.call(this, t);
                    }, "month", function() {
                        return Date.prototype.getMonth.call(this) + 1;
                    } ],
                    mm: [ "0[1-9]|1[012]", function(e) {
                        var t = e ? parseInt(e) : 0;
                        return t > 0 && t--, Date.prototype.setMonth.call(this, t);
                    }, "month", function() {
                        return M(Date.prototype.getMonth.call(this) + 1, 2);
                    } ],
                    mmm: [ "" ],
                    mmmm: [ "" ],
                    yy: [ "[0-9]{2}", Date.prototype.setFullYear, "year", function() {
                        return M(Date.prototype.getFullYear.call(this), 2);
                    } ],
                    yyyy: [ "[0-9]{4}", Date.prototype.setFullYear, "year", function() {
                        return M(Date.prototype.getFullYear.call(this), 4);
                    } ],
                    h: [ "[1-9]|1[0-2]", Date.prototype.setHours, "hours", Date.prototype.getHours ],
                    hh: [ "0[1-9]|1[0-2]", Date.prototype.setHours, "hours", function() {
                        return M(Date.prototype.getHours.call(this), 2);
                    } ],
                    hx: [ function(e) {
                        return "[0-9]{".concat(e, "}");
                    }, Date.prototype.setHours, "hours", function(e) {
                        return Date.prototype.getHours;
                    } ],
                    H: [ "1?[0-9]|2[0-3]", Date.prototype.setHours, "hours", Date.prototype.getHours ],
                    HH: [ "0[0-9]|1[0-9]|2[0-3]", Date.prototype.setHours, "hours", function() {
                        return M(Date.prototype.getHours.call(this), 2);
                    } ],
                    Hx: [ function(e) {
                        return "[0-9]{".concat(e, "}");
                    }, Date.prototype.setHours, "hours", function(e) {
                        return function() {
                            return M(Date.prototype.getHours.call(this), e);
                        };
                    } ],
                    M: [ "[1-5]?[0-9]", Date.prototype.setMinutes, "minutes", Date.prototype.getMinutes ],
                    MM: [ "0[0-9]|1[0-9]|2[0-9]|3[0-9]|4[0-9]|5[0-9]", Date.prototype.setMinutes, "minutes", function() {
                        return M(Date.prototype.getMinutes.call(this), 2);
                    } ],
                    s: [ "[1-5]?[0-9]", Date.prototype.setSeconds, "seconds", Date.prototype.getSeconds ],
                    ss: [ "0[0-9]|1[0-9]|2[0-9]|3[0-9]|4[0-9]|5[0-9]", Date.prototype.setSeconds, "seconds", function() {
                        return M(Date.prototype.getSeconds.call(this), 2);
                    } ],
                    l: [ "[0-9]{3}", Date.prototype.setMilliseconds, "milliseconds", function() {
                        return M(Date.prototype.getMilliseconds.call(this), 3);
                    }, 3 ],
                    L: [ "[0-9]{2}", Date.prototype.setMilliseconds, "milliseconds", function() {
                        return M(Date.prototype.getMilliseconds.call(this), 2);
                    }, 2 ],
                    t: [ "[ap]", k, "ampm", b, 1 ],
                    tt: [ "[ap]m", k, "ampm", b, 2 ],
                    T: [ "[AP]", k, "ampm", b, 1 ],
                    TT: [ "[AP]M", k, "ampm", b, 2 ],
                    Z: [ ".*", void 0, "Z", function() {
                        var e = this.toString().match(/\((.+)\)/)[1];
                        e.includes(" ") && (e = (e = e.replace("-", " ").toUpperCase()).split(" ").map((function(e) {
                            return l(e, 1)[0];
                        })).join(""));
                        return e;
                    } ],
                    o: [ "" ],
                    S: [ "" ]
                }, y = {
                    isoDate: "yyyy-mm-dd",
                    isoTime: "HH:MM:ss",
                    isoDateTime: "yyyy-mm-dd'T'HH:MM:ss",
                    isoUtcDateTime: "UTC:yyyy-mm-dd'T'HH:MM:ss'Z'"
                };
                function k(e) {
                    var t = this.getHours();
                    e.toLowerCase().includes("p") ? this.setHours(t + 12) : e.toLowerCase().includes("a") && t >= 12 && this.setHours(t - 12);
                }
                function b() {
                    var e = this.getHours();
                    return (e = e || 12) >= 12 ? "PM" : "AM";
                }
                function x(e) {
                    var t = new RegExp("\\d+$").exec(e[0]);
                    if (t && void 0 !== t[0]) {
                        var i = g[e[0][0] + "x"].slice("");
                        return i[0] = i[0](t[0]), i[3] = i[3](t[0]), i;
                    }
                    if (g[e[0]]) return g[e[0]];
                }
                function P(e) {
                    if (!e.tokenizer) {
                        var t = [], i = [];
                        for (var n in g) if (/\.*x$/.test(n)) {
                            var a = n[0] + "\\d+";
                            -1 === i.indexOf(a) && i.push(a);
                        } else -1 === t.indexOf(n[0]) && t.push(n[0]);
                        e.tokenizer = "(" + (i.length > 0 ? i.join("|") + "|" : "") + t.join("+|") + ")+?|.", 
                        e.tokenizer = new RegExp(e.tokenizer, "g");
                    }
                    return e.tokenizer;
                }
                function w(e, t, i) {
                    if (!m) return !0;
                    if (void 0 === e.rawday || !isFinite(e.rawday) && new Date(e.date.getFullYear(), isFinite(e.rawmonth) ? e.month : e.date.getMonth() + 1, 0).getDate() >= e.day || "29" == e.day && (!isFinite(e.rawyear) || void 0 === e.rawyear || "" === e.rawyear) || new Date(e.date.getFullYear(), isFinite(e.rawmonth) ? e.month : e.date.getMonth() + 1, 0).getDate() >= e.day) return t;
                    if ("29" == e.day) {
                        var n = E(t.pos, i);
                        if ("yyyy" === n.targetMatch[0] && t.pos - n.targetMatchIndex == 2) return t.remove = t.pos + 1, 
                        t;
                    } else if ("02" == e.month && "30" == e.day && void 0 !== t.c) return e.day = "03", 
                    e.date.setDate(3), e.date.setMonth(1), t.insert = [ {
                        pos: t.pos,
                        c: "0"
                    }, {
                        pos: t.pos + 1,
                        c: t.c
                    } ], t.caret = o.seekNext.call(this, t.pos + 1), t;
                    return !1;
                }
                function S(e, t, i, n) {
                    var a, o, s = "";
                    for (P(i).lastIndex = 0; a = P(i).exec(e); ) {
                        if (void 0 === t) if (o = x(a)) s += "(" + o[0] + ")"; else switch (a[0]) {
                          case "[":
                            s += "(";
                            break;

                          case "]":
                            s += ")?";
                            break;

                          default:
                            s += (0, r.default)(a[0]);
                        } else if (o = x(a)) if (!0 !== n && o[3]) s += o[3].call(t.date); else o[2] ? s += t["raw" + o[2]] : s += a[0]; else s += a[0];
                    }
                    return s;
                }
                function M(e, t, i) {
                    for (e = String(e), t = t || 2; e.length < t; ) e = i ? e + "0" : "0" + e;
                    return e;
                }
                function _(e, t, i) {
                    return "string" == typeof e ? new h(e, t, i) : e && "object" === u(e) && Object.prototype.hasOwnProperty.call(e, "date") ? e : void 0;
                }
                function O(e, t) {
                    return S(t.inputFormat, {
                        date: e
                    }, t);
                }
                function E(e, t) {
                    var i, n, a = 0, r = 0;
                    for (P(t).lastIndex = 0; n = P(t).exec(t.inputFormat); ) {
                        var o = new RegExp("\\d+$").exec(n[0]);
                        if ((a += r = o ? parseInt(o[0]) : n[0].length) >= e + 1) {
                            i = n, n = P(t).exec(t.inputFormat);
                            break;
                        }
                    }
                    return {
                        targetMatchIndex: a - r,
                        nextMatch: n,
                        targetMatch: i
                    };
                }
                n.default.extendAliases({
                    datetime: {
                        mask: function(e) {
                            return e.numericInput = !1, g.S = e.i18n.ordinalSuffix.join("|"), e.inputFormat = y[e.inputFormat] || e.inputFormat, 
                            e.displayFormat = y[e.displayFormat] || e.displayFormat || e.inputFormat, e.outputFormat = y[e.outputFormat] || e.outputFormat || e.inputFormat, 
                            e.placeholder = "" !== e.placeholder ? e.placeholder : e.inputFormat.replace(/[[\]]/, ""), 
                            e.regex = S(e.inputFormat, void 0, e), e.min = _(e.min, e.inputFormat, e), e.max = _(e.max, e.inputFormat, e), 
                            null;
                        },
                        placeholder: "",
                        inputFormat: "isoDateTime",
                        displayFormat: null,
                        outputFormat: null,
                        min: null,
                        max: null,
                        skipOptionalPartCharacter: "",
                        i18n: {
                            dayNames: [ "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday" ],
                            monthNames: [ "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" ],
                            ordinalSuffix: [ "st", "nd", "rd", "th" ]
                        },
                        preValidation: function(e, t, i, n, a, r, o, s) {
                            if (s) return !0;
                            if (isNaN(i) && e[t] !== i) {
                                var l = E(t, a);
                                if (l.nextMatch && l.nextMatch[0] === i && l.targetMatch[0].length > 1) {
                                    var c = g[l.targetMatch[0]][0];
                                    if (new RegExp(c).test("0" + e[t - 1])) return e[t] = e[t - 1], e[t - 1] = "0", 
                                    {
                                        fuzzy: !0,
                                        buffer: e,
                                        refreshFromBuffer: {
                                            start: t - 1,
                                            end: t + 1
                                        },
                                        pos: t + 1
                                    };
                                }
                            }
                            return !0;
                        },
                        postValidation: function(e, t, i, n, a, r, o, l) {
                            var c, u;
                            if (o) return !0;
                            if (!1 === n && (((c = E(t + 1, a)).targetMatch && c.targetMatchIndex === t && c.targetMatch[0].length > 1 && void 0 !== g[c.targetMatch[0]] || (c = E(t + 2, a)).targetMatch && c.targetMatchIndex === t + 1 && c.targetMatch[0].length > 1 && void 0 !== g[c.targetMatch[0]]) && (u = g[c.targetMatch[0]][0]), 
                            void 0 !== u && (void 0 !== r.validPositions[t + 1] && new RegExp(u).test(i + "0") ? (e[t] = i, 
                            e[t + 1] = "0", n = {
                                pos: t + 2,
                                caret: t
                            }) : new RegExp(u).test("0" + i) && (e[t] = "0", e[t + 1] = i, n = {
                                pos: t + 2
                            })), !1 === n)) return n;
                            if (n.fuzzy && (e = n.buffer, t = n.pos), (c = E(t, a)).targetMatch && c.targetMatch[0] && void 0 !== g[c.targetMatch[0]]) {
                                var f = g[c.targetMatch[0]];
                                u = f[0];
                                var d = e.slice(c.targetMatchIndex, c.targetMatchIndex + c.targetMatch[0].length);
                                if (!1 === new RegExp(u).test(d.join("")) && 2 === c.targetMatch[0].length && r.validPositions[c.targetMatchIndex] && r.validPositions[c.targetMatchIndex + 1] && (r.validPositions[c.targetMatchIndex + 1].input = "0"), 
                                "year" == f[2]) for (var p = s.getMaskTemplate.call(this, !1, 1, void 0, !0), h = t + 1; h < e.length; h++) e[h] = p[h], 
                                delete r.validPositions[h];
                            }
                            var m = n, y = _(e.join(""), a.inputFormat, a);
                            return m && !isNaN(y.date.getTime()) && (a.prefillYear && (m = function(e, t, i) {
                                if (e.year !== e.rawyear) {
                                    var n = v.toString(), a = e.rawyear.replace(/[^0-9]/g, ""), r = n.slice(0, a.length), o = n.slice(a.length);
                                    if (2 === a.length && a === r) {
                                        var s = new Date(v, e.month - 1, e.day);
                                        e.day == s.getDate() && (!i.max || i.max.date.getTime() >= s.getTime()) && (e.date.setFullYear(v), 
                                        e.year = n, t.insert = [ {
                                            pos: t.pos + 1,
                                            c: o[0]
                                        }, {
                                            pos: t.pos + 2,
                                            c: o[1]
                                        } ]);
                                    }
                                }
                                return t;
                            }(y, m, a)), m = function(e, t, i, n, a) {
                                if (!t) return t;
                                if (t && i.min && !isNaN(i.min.date.getTime())) {
                                    var r;
                                    for (e.reset(), P(i).lastIndex = 0; r = P(i).exec(i.inputFormat); ) {
                                        var o;
                                        if ((o = x(r)) && o[3]) {
                                            for (var s = o[1], l = e[o[2]], c = i.min[o[2]], u = i.max ? i.max[o[2]] : c, f = [], d = !1, p = 0; p < c.length; p++) void 0 !== n.validPositions[p + r.index] || d ? (f[p] = l[p], 
                                            d = d || l[p] > c[p]) : (f[p] = c[p], "year" === o[2] && l.length - 1 == p && c != u && (f = (parseInt(f.join("")) + 1).toString().split("")), 
                                            "ampm" === o[2] && c != u && i.min.date.getTime() > e.date.getTime() && (f[p] = u[p]));
                                            s.call(e._date, f.join(""));
                                        }
                                    }
                                    t = i.min.date.getTime() <= e.date.getTime(), e.reInit();
                                }
                                return t && i.max && (isNaN(i.max.date.getTime()) || (t = i.max.date.getTime() >= e.date.getTime())), 
                                t;
                            }(y, m = w.call(this, y, m, a), a, r)), void 0 !== t && m && n.pos !== t ? {
                                buffer: S(a.inputFormat, y, a).split(""),
                                refreshFromBuffer: {
                                    start: t,
                                    end: n.pos
                                },
                                pos: n.caret || n.pos
                            } : m;
                        },
                        onKeyDown: function(e, t, i, n) {
                            e.ctrlKey && e.key === a.keys.ArrowRight && (this.inputmask._valueSet(O(new Date, n)), 
                            p(this).trigger("setvalue"));
                        },
                        onUnMask: function(e, t, i) {
                            return t ? S(i.outputFormat, _(e, i.inputFormat, i), i, !0) : t;
                        },
                        casing: function(e, t, i, n) {
                            return 0 == t.nativeDef.indexOf("[ap]") ? e.toLowerCase() : 0 == t.nativeDef.indexOf("[AP]") ? e.toUpperCase() : e;
                        },
                        onBeforeMask: function(e, t) {
                            return "[object Date]" === Object.prototype.toString.call(e) && (e = O(e, t)), e;
                        },
                        insertMode: !1,
                        insertModeVisual: !1,
                        shiftPositions: !1,
                        keepStatic: !1,
                        inputmode: "numeric",
                        prefillYear: !0
                    }
                });
            },
            3851: function(e, t, i) {
                var n, a = (n = i(2394)) && n.__esModule ? n : {
                    default: n
                }, r = i(8711), o = i(4713);
                a.default.extendDefinitions({
                    A: {
                        validator: "[A-Za-z\u0410-\u044f\u0401\u0451\xc0-\xff\xb5]",
                        casing: "upper"
                    },
                    "&": {
                        validator: "[0-9A-Za-z\u0410-\u044f\u0401\u0451\xc0-\xff\xb5]",
                        casing: "upper"
                    },
                    "#": {
                        validator: "[0-9A-Fa-f]",
                        casing: "upper"
                    }
                });
                var s = new RegExp("25[0-5]|2[0-4][0-9]|[01][0-9][0-9]");
                function l(e, t, i, n, a) {
                    return i - 1 > -1 && "." !== t.buffer[i - 1] ? (e = t.buffer[i - 1] + e, e = i - 2 > -1 && "." !== t.buffer[i - 2] ? t.buffer[i - 2] + e : "0" + e) : e = "00" + e, 
                    s.test(e);
                }
                a.default.extendAliases({
                    cssunit: {
                        regex: "[+-]?[0-9]+\\.?([0-9]+)?(px|em|rem|ex|%|in|cm|mm|pt|pc)"
                    },
                    url: {
                        regex: "(https?|ftp)://.*",
                        autoUnmask: !1,
                        keepStatic: !1,
                        tabThrough: !0
                    },
                    ip: {
                        mask: "i{1,3}.j{1,3}.k{1,3}.l{1,3}",
                        definitions: {
                            i: {
                                validator: l
                            },
                            j: {
                                validator: l
                            },
                            k: {
                                validator: l
                            },
                            l: {
                                validator: l
                            }
                        },
                        onUnMask: function(e, t, i) {
                            return e;
                        },
                        inputmode: "decimal",
                        substitutes: {
                            ",": "."
                        }
                    },
                    email: {
                        mask: function(e) {
                            var t = e.separator, i = e.quantifier, n = "*{1,64}[.*{1,64}][.*{1,64}][.*{1,63}]@-{1,63}.-{1,63}[.-{1,63}][.-{1,63}]", a = n;
                            if (t) for (var r = 0; r < i; r++) a += "[".concat(t).concat(n, "]");
                            return a;
                        },
                        greedy: !1,
                        casing: "lower",
                        separator: null,
                        quantifier: 5,
                        skipOptionalPartCharacter: "",
                        onBeforePaste: function(e, t) {
                            return (e = e.toLowerCase()).replace("mailto:", "");
                        },
                        definitions: {
                            "*": {
                                validator: "[0-9\uff11-\uff19A-Za-z\u0410-\u044f\u0401\u0451\xc0-\xff\xb5!#$%&'*+/=?^_`{|}~-]"
                            },
                            "-": {
                                validator: "[0-9A-Za-z-]"
                            }
                        },
                        onUnMask: function(e, t, i) {
                            return e;
                        },
                        inputmode: "email"
                    },
                    mac: {
                        mask: "##:##:##:##:##:##"
                    },
                    vin: {
                        mask: "V{13}9{4}",
                        definitions: {
                            V: {
                                validator: "[A-HJ-NPR-Za-hj-npr-z\\d]",
                                casing: "upper"
                            }
                        },
                        clearIncomplete: !0,
                        autoUnmask: !0
                    },
                    ssn: {
                        mask: "999-99-9999",
                        postValidation: function(e, t, i, n, a, s, l) {
                            var c = o.getMaskTemplate.call(this, !0, r.getLastValidPosition.call(this), !0, !0);
                            return /^(?!219-09-9999|078-05-1120)(?!666|000|9.{2}).{3}-(?!00).{2}-(?!0{4}).{4}$/.test(c.join(""));
                        }
                    }
                });
            },
            207: function(e, t, i) {
                var n = s(i(2394)), a = s(i(7184)), r = i(8711), o = i(2839);
                function s(e) {
                    return e && e.__esModule ? e : {
                        default: e
                    };
                }
                var l = n.default.dependencyLib;
                function c(e, t) {
                    for (var i = "", a = 0; a < e.length; a++) n.default.prototype.definitions[e.charAt(a)] || t.definitions[e.charAt(a)] || t.optionalmarker[0] === e.charAt(a) || t.optionalmarker[1] === e.charAt(a) || t.quantifiermarker[0] === e.charAt(a) || t.quantifiermarker[1] === e.charAt(a) || t.groupmarker[0] === e.charAt(a) || t.groupmarker[1] === e.charAt(a) || t.alternatormarker === e.charAt(a) ? i += "\\" + e.charAt(a) : i += e.charAt(a);
                    return i;
                }
                function u(e, t, i, n) {
                    if (e.length > 0 && t > 0 && (!i.digitsOptional || n)) {
                        var a = e.indexOf(i.radixPoint), r = !1;
                        i.negationSymbol.back === e[e.length - 1] && (r = !0, e.length--), -1 === a && (e.push(i.radixPoint), 
                        a = e.length - 1);
                        for (var o = 1; o <= t; o++) isFinite(e[a + o]) || (e[a + o] = "0");
                    }
                    return r && e.push(i.negationSymbol.back), e;
                }
                function f(e, t) {
                    var i = 0;
                    for (var n in "+" === e && (i = r.seekNext.call(this, t.validPositions.length - 1)), 
                    t.tests) if ((n = parseInt(n)) >= i) for (var a = 0, o = t.tests[n].length; a < o; a++) if ((void 0 === t.validPositions[n] || "-" === e) && t.tests[n][a].match.def === e) return n + (void 0 !== t.validPositions[n] && "-" !== e ? 1 : 0);
                    return i;
                }
                function d(e, t) {
                    for (var i = -1, n = 0, a = t.validPositions.length; n < a; n++) {
                        var r = t.validPositions[n];
                        if (r && r.match.def === e) {
                            i = n;
                            break;
                        }
                    }
                    return i;
                }
                function p(e, t, i, n, a) {
                    var r = t.buffer ? t.buffer.indexOf(a.radixPoint) : -1, o = (-1 !== r || n && a.jitMasking) && new RegExp(a.definitions[9].validator).test(e);
                    return a._radixDance && -1 !== r && o && null == t.validPositions[r] ? {
                        insert: {
                            pos: r === i ? r + 1 : r,
                            c: a.radixPoint
                        },
                        pos: i
                    } : o;
                }
                n.default.extendAliases({
                    numeric: {
                        mask: function(e) {
                            e.repeat = 0, e.groupSeparator === e.radixPoint && e.digits && "0" !== e.digits && ("." === e.radixPoint ? e.groupSeparator = "," : "," === e.radixPoint ? e.groupSeparator = "." : e.groupSeparator = ""), 
                            " " === e.groupSeparator && (e.skipOptionalPartCharacter = void 0), e.placeholder.length > 1 && (e.placeholder = e.placeholder.charAt(0)), 
                            "radixFocus" === e.positionCaretOnClick && "" === e.placeholder && (e.positionCaretOnClick = "lvp");
                            var t = "0", i = e.radixPoint;
                            !0 === e.numericInput && void 0 === e.__financeInput ? (t = "1", e.positionCaretOnClick = "radixFocus" === e.positionCaretOnClick ? "lvp" : e.positionCaretOnClick, 
                            e.digitsOptional = !1, isNaN(e.digits) && (e.digits = 2), e._radixDance = !1, i = "," === e.radixPoint ? "?" : "!", 
                            "" !== e.radixPoint && void 0 === e.definitions[i] && (e.definitions[i] = {}, e.definitions[i].validator = "[" + e.radixPoint + "]", 
                            e.definitions[i].placeholder = e.radixPoint, e.definitions[i].static = !0, e.definitions[i].generated = !0)) : (e.__financeInput = !1, 
                            e.numericInput = !0);
                            var n, r = "[+]";
                            if (r += c(e.prefix, e), "" !== e.groupSeparator ? (void 0 === e.definitions[e.groupSeparator] && (e.definitions[e.groupSeparator] = {}, 
                            e.definitions[e.groupSeparator].validator = "[" + e.groupSeparator + "]", e.definitions[e.groupSeparator].placeholder = e.groupSeparator, 
                            e.definitions[e.groupSeparator].static = !0, e.definitions[e.groupSeparator].generated = !0), 
                            r += e._mask(e)) : r += "9{+}", void 0 !== e.digits && 0 !== e.digits) {
                                var o = e.digits.toString().split(",");
                                isFinite(o[0]) && o[1] && isFinite(o[1]) ? r += i + t + "{" + e.digits + "}" : (isNaN(e.digits) || parseInt(e.digits) > 0) && (e.digitsOptional || e.jitMasking ? (n = r + i + t + "{0," + e.digits + "}", 
                                e.keepStatic = !0) : r += i + t + "{" + e.digits + "}");
                            } else e.inputmode = "numeric";
                            return r += c(e.suffix, e), r += "[-]", n && (r = [ n + c(e.suffix, e) + "[-]", r ]), 
                            e.greedy = !1, function(e) {
                                void 0 === e.parseMinMaxOptions && (null !== e.min && (e.min = e.min.toString().replace(new RegExp((0, a.default)(e.groupSeparator), "g"), ""), "," === e.radixPoint && (e.min = e.min.replace(e.radixPoint, ".")), 
                                e.min = isFinite(e.min) ? parseFloat(e.min) : NaN, isNaN(e.min) && (e.min = Number.MIN_VALUE)), 
                                null !== e.max && (e.max = e.max.toString().replace(new RegExp((0, a.default)(e.groupSeparator), "g"), ""), 
                                "," === e.radixPoint && (e.max = e.max.replace(e.radixPoint, ".")), e.max = isFinite(e.max) ? parseFloat(e.max) : NaN, 
                                isNaN(e.max) && (e.max = Number.MAX_VALUE)), e.parseMinMaxOptions = "done");
                            }(e), "" !== e.radixPoint && e.substituteRadixPoint && (e.substitutes["." == e.radixPoint ? "," : "."] = e.radixPoint), 
                            r;
                        },
                        _mask: function(e) {
                            return "(" + e.groupSeparator + "999){+|1}";
                        },
                        digits: "*",
                        digitsOptional: !0,
                        enforceDigitsOnBlur: !1,
                        radixPoint: ".",
                        positionCaretOnClick: "radixFocus",
                        _radixDance: !0,
                        groupSeparator: "",
                        allowMinus: !0,
                        negationSymbol: {
                            front: "-",
                            back: ""
                        },
                        prefix: "",
                        suffix: "",
                        min: null,
                        max: null,
                        SetMaxOnOverflow: !1,
                        step: 1,
                        inputType: "text",
                        unmaskAsNumber: !1,
                        roundingFN: Math.round,
                        inputmode: "decimal",
                        shortcuts: {
                            k: "1000",
                            m: "1000000"
                        },
                        placeholder: "0",
                        greedy: !1,
                        rightAlign: !0,
                        insertMode: !0,
                        autoUnmask: !1,
                        skipOptionalPartCharacter: "",
                        usePrototypeDefinitions: !1,
                        stripLeadingZeroes: !0,
                        substituteRadixPoint: !0,
                        definitions: {
                            0: {
                                validator: p
                            },
                            1: {
                                validator: p,
                                definitionSymbol: "9"
                            },
                            9: {
                                validator: "[0-9\uff10-\uff19\u0660-\u0669\u06f0-\u06f9]",
                                definitionSymbol: "*"
                            },
                            "+": {
                                validator: function(e, t, i, n, a) {
                                    return a.allowMinus && ("-" === e || e === a.negationSymbol.front);
                                }
                            },
                            "-": {
                                validator: function(e, t, i, n, a) {
                                    return a.allowMinus && e === a.negationSymbol.back;
                                }
                            }
                        },
                        preValidation: function(e, t, i, n, a, r, o, s) {
                            if (!1 !== a.__financeInput && i === a.radixPoint) return !1;
                            var l = e.indexOf(a.radixPoint), c = t;
                            if (t = function(e, t, i, n, a) {
                                return a._radixDance && a.numericInput && t !== a.negationSymbol.back && e <= i && (i > 0 || t == a.radixPoint) && (void 0 === n.validPositions[e - 1] || n.validPositions[e - 1].input !== a.negationSymbol.back) && (e -= 1), 
                                e;
                            }(t, i, l, r, a), "-" === i || i === a.negationSymbol.front) {
                                if (!0 !== a.allowMinus) return !1;
                                var u = !1, p = d("+", r), h = d("-", r);
                                return -1 !== p && (u = [ p, h ]), !1 !== u ? {
                                    remove: u,
                                    caret: c - a.negationSymbol.back.length
                                } : {
                                    insert: [ {
                                        pos: f.call(this, "+", r),
                                        c: a.negationSymbol.front,
                                        fromIsValid: !0
                                    }, {
                                        pos: f.call(this, "-", r),
                                        c: a.negationSymbol.back,
                                        fromIsValid: void 0
                                    } ],
                                    caret: c + a.negationSymbol.back.length
                                };
                            }
                            if (i === a.groupSeparator) return {
                                caret: c
                            };
                            if (s) return !0;
                            if (-1 !== l && !0 === a._radixDance && !1 === n && i === a.radixPoint && void 0 !== a.digits && (isNaN(a.digits) || parseInt(a.digits) > 0) && l !== t) return {
                                caret: a._radixDance && t === l - 1 ? l + 1 : l
                            };
                            if (!1 === a.__financeInput) if (n) {
                                if (a.digitsOptional) return {
                                    rewritePosition: o.end
                                };
                                if (!a.digitsOptional) {
                                    if (o.begin > l && o.end <= l) return i === a.radixPoint ? {
                                        insert: {
                                            pos: l + 1,
                                            c: "0",
                                            fromIsValid: !0
                                        },
                                        rewritePosition: l
                                    } : {
                                        rewritePosition: l + 1
                                    };
                                    if (o.begin < l) return {
                                        rewritePosition: o.begin - 1
                                    };
                                }
                            } else if (!a.showMaskOnHover && !a.showMaskOnFocus && !a.digitsOptional && a.digits > 0 && "" === this.__valueGet.call(this.el)) return {
                                rewritePosition: l
                            };
                            return {
                                rewritePosition: t
                            };
                        },
                        postValidation: function(e, t, i, n, a, r, o) {
                            if (!1 === n) return n;
                            if (o) return !0;
                            if (null !== a.min || null !== a.max) {
                                var s = a.onUnMask(e.slice().reverse().join(""), void 0, l.extend({}, a, {
                                    unmaskAsNumber: !0
                                }));
                                if (null !== a.min && s < a.min && (s.toString().length > a.min.toString().length || s < 0)) return !1;
                                if (null !== a.max && s > a.max) return !!a.SetMaxOnOverflow && {
                                    refreshFromBuffer: !0,
                                    buffer: u(a.max.toString().replace(".", a.radixPoint).split(""), a.digits, a).reverse()
                                };
                            }
                            return n;
                        },
                        onUnMask: function(e, t, i) {
                            if ("" === t && !0 === i.nullable) return t;
                            var n = e.replace(i.prefix, "");
                            return n = (n = n.replace(i.suffix, "")).replace(new RegExp((0, a.default)(i.groupSeparator), "g"), ""), 
                            "" !== i.placeholder.charAt(0) && (n = n.replace(new RegExp(i.placeholder.charAt(0), "g"), "0")), 
                            i.unmaskAsNumber ? ("" !== i.radixPoint && -1 !== n.indexOf(i.radixPoint) && (n = n.replace(a.default.call(this, i.radixPoint), ".")), 
                            n = (n = n.replace(new RegExp("^" + (0, a.default)(i.negationSymbol.front)), "-")).replace(new RegExp((0, a.default)(i.negationSymbol.back) + "$"), ""), Number(n)) : n;
                        },
                        isComplete: function(e, t) {
                            var i = (t.numericInput ? e.slice().reverse() : e).join("");
                            return i = (i = (i = (i = (i = i.replace(new RegExp("^" + (0, a.default)(t.negationSymbol.front)), "-")).replace(new RegExp((0, a.default)(t.negationSymbol.back) + "$"), "")).replace(t.prefix, "")).replace(t.suffix, "")).replace(new RegExp((0, a.default)(t.groupSeparator) + "([0-9]{3})", "g"), "$1"), "," === t.radixPoint && (i = i.replace((0, a.default)(t.radixPoint), ".")), isFinite(i);
                        },
                        onBeforeMask: function(e, t) {
                            var i = t.radixPoint || ",";
                            isFinite(t.digits) && (t.digits = parseInt(t.digits)), "number" != typeof e && "number" !== t.inputType || "" === i || (e = e.toString().replace(".", i));
                            var n = "-" === e.charAt(0) || e.charAt(0) === t.negationSymbol.front, r = e.split(i), o = r[0].replace(/[^\-0-9]/g, ""), s = r.length > 1 ? r[1].replace(/[^0-9]/g, "") : "", l = r.length > 1;
                            e = o + ("" !== s ? i + s : s);
                            var c = 0;
                            if ("" !== i && (c = t.digitsOptional ? t.digits < s.length ? t.digits : s.length : t.digits, 
                            "" !== s || !t.digitsOptional)) {
                                var f = Math.pow(10, c || 1);
                                e = e.replace((0, a.default)(i), "."), isNaN(parseFloat(e)) || (e = (t.roundingFN(parseFloat(e) * f) / f).toFixed(c)), 
                                e = e.toString().replace(".", i);
                            }
                            if (0 === t.digits && -1 !== e.indexOf(i) && (e = e.substring(0, e.indexOf(i))), 
                            null !== t.min || null !== t.max) {
                                var d = e.toString().replace(i, ".");
                                null !== t.min && d < t.min ? e = t.min.toString().replace(".", i) : null !== t.max && d > t.max && (e = t.max.toString().replace(".", i));
                            }
                            return n && "-" !== e.charAt(0) && (e = "-" + e), u(e.toString().split(""), c, t, l).join("");
                        },
                        onBeforeWrite: function(e, t, i, n) {
                            function r(e, t) {
                                if (!1 !== n.__financeInput || t) {
                                    var i = e.indexOf(n.radixPoint);
                                    -1 !== i && e.splice(i, 1);
                                }
                                if ("" !== n.groupSeparator) for (;-1 !== (i = e.indexOf(n.groupSeparator)); ) e.splice(i, 1);
                                return e;
                            }
                            var o, s;
                            if (n.stripLeadingZeroes && (s = function(e, t) {
                                var i = new RegExp("(^" + ("" !== t.negationSymbol.front ? (0, a.default)(t.negationSymbol.front) + "?" : "") + (0, a.default)(t.prefix) + ")(.*)(" + (0, a.default)(t.suffix) + ("" != t.negationSymbol.back ? (0, a.default)(t.negationSymbol.back) + "?" : "") + "$)").exec(e.slice().reverse().join("")), n = i ? i[2] : "", r = !1;
                                return n && (n = n.split(t.radixPoint.charAt(0))[0], r = new RegExp("^[0" + t.groupSeparator + "]*").exec(n)), 
                                !(!r || !(r[0].length > 1 || r[0].length > 0 && r[0].length < n.length)) && r;
                            }(t, n))) for (var c = t.join("").lastIndexOf(s[0].split("").reverse().join("")) - (s[0] == s.input ? 0 : 1), f = s[0] == s.input ? 1 : 0, d = s[0].length - f; d > 0; d--) delete this.maskset.validPositions[c + d], 
                            delete t[c + d];
                            if (e) switch (e.type) {
                              case "blur":
                              case "checkval":
                                if (null !== n.min) {
                                    var p = n.onUnMask(t.slice().reverse().join(""), void 0, l.extend({}, n, {
                                        unmaskAsNumber: !0
                                    }));
                                    if (null !== n.min && p < n.min) return {
                                        refreshFromBuffer: !0,
                                        buffer: u(n.min.toString().replace(".", n.radixPoint).split(""), n.digits, n).reverse()
                                    };
                                }
                                if (t[t.length - 1] === n.negationSymbol.front) {
                                    var h = new RegExp("(^" + ("" != n.negationSymbol.front ? (0, a.default)(n.negationSymbol.front) + "?" : "") + (0, a.default)(n.prefix) + ")(.*)(" + (0, a.default)(n.suffix) + ("" != n.negationSymbol.back ? (0, a.default)(n.negationSymbol.back) + "?" : "") + "$)").exec(r(t.slice(), !0).reverse().join(""));
                                    0 == (h ? h[2] : "") && (o = {
                                        refreshFromBuffer: !0,
                                        buffer: [ 0 ]
                                    });
                                } else if ("" !== n.radixPoint) {
                                    t.indexOf(n.radixPoint) === n.suffix.length && (o && o.buffer ? o.buffer.splice(0, 1 + n.suffix.length) : (t.splice(0, 1 + n.suffix.length), 
                                    o = {
                                        refreshFromBuffer: !0,
                                        buffer: r(t)
                                    }));
                                }
                                if (n.enforceDigitsOnBlur) {
                                    var v = (o = o || {}) && o.buffer || t.slice().reverse();
                                    o.refreshFromBuffer = !0, o.buffer = u(v, n.digits, n, !0).reverse();
                                }
                            }
                            return o;
                        },
                        onKeyDown: function(e, t, i, n) {
                            var a, r = l(this);
                            if (3 != e.location) {
                                var s, c = e.key;
                                if ((s = n.shortcuts && n.shortcuts[c]) && s.length > 1) return this.inputmask.__valueSet.call(this, parseFloat(this.inputmask.unmaskedvalue()) * parseInt(s)), 
                                r.trigger("setvalue"), !1;
                            }
                            if (e.ctrlKey) switch (e.key) {
                              case o.keys.ArrowUp:
                                return this.inputmask.__valueSet.call(this, parseFloat(this.inputmask.unmaskedvalue()) + parseInt(n.step)), 
                                r.trigger("setvalue"), !1;

                              case o.keys.ArrowDown:
                                return this.inputmask.__valueSet.call(this, parseFloat(this.inputmask.unmaskedvalue()) - parseInt(n.step)), 
                                r.trigger("setvalue"), !1;
                            }
                            if (!e.shiftKey && (e.key === o.keys.Delete || e.key === o.keys.Backspace || e.key === o.keys.BACKSPACE_SAFARI) && i.begin !== t.length) {
                                if (t[e.key === o.keys.Delete ? i.begin - 1 : i.end] === n.negationSymbol.front) return a = t.slice().reverse(), 
                                "" !== n.negationSymbol.front && a.shift(), "" !== n.negationSymbol.back && a.pop(), 
                                r.trigger("setvalue", [ a.join(""), i.begin ]), !1;
                                if (!0 === n._radixDance) {
                                    var f = t.indexOf(n.radixPoint);
                                    if (n.digitsOptional) {
                                        if (0 === f) return (a = t.slice().reverse()).pop(), r.trigger("setvalue", [ a.join(""), i.begin >= a.length ? a.length : i.begin ]), 
                                        !1;
                                    } else if (-1 !== f && (i.begin < f || i.end < f || e.key === o.keys.Delete && (i.begin === f || i.begin - 1 === f))) {
                                        var d = void 0;
                                        return i.begin === i.end && (e.key === o.keys.Backspace || e.key === o.keys.BACKSPACE_SAFARI ? i.begin++ : e.key === o.keys.Delete && i.begin - 1 === f && (d = l.extend({}, i), 
                                        i.begin--, i.end--)), (a = t.slice().reverse()).splice(a.length - i.begin, i.begin - i.end + 1), 
                                        a = u(a, n.digits, n).join(""), d && (i = d), r.trigger("setvalue", [ a, i.begin >= a.length ? f + 1 : i.begin ]), 
                                        !1;
                                    }
                                }
                            }
                        }
                    },
                    currency: {
                        prefix: "",
                        groupSeparator: ",",
                        alias: "numeric",
                        digits: 2,
                        digitsOptional: !1
                    },
                    decimal: {
                        alias: "numeric"
                    },
                    integer: {
                        alias: "numeric",
                        inputmode: "numeric",
                        digits: 0
                    },
                    percentage: {
                        alias: "numeric",
                        min: 0,
                        max: 100,
                        suffix: " %",
                        digits: 0,
                        allowMinus: !1
                    },
                    indianns: {
                        alias: "numeric",
                        _mask: function(e) {
                            return "(" + e.groupSeparator + "99){*|1}(" + e.groupSeparator + "999){1|1}";
                        },
                        groupSeparator: ",",
                        radixPoint: ".",
                        placeholder: "0",
                        digits: 2,
                        digitsOptional: !1
                    }
                });
            },
            9380: function(e, t, i) {
                var n;
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.default = void 0;
                var a = ((n = i(8741)) && n.__esModule ? n : {
                    default: n
                }).default ? window : {};
                t.default = a;
            },
            7760: function(e, t, i) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.HandleNativePlaceholder = function(e, t) {
                    var i = e ? e.inputmask : this;
                    if (s.ie) {
                        if (e.inputmask._valueGet() !== t && (e.placeholder !== t || "" === e.placeholder)) {
                            var n = r.getBuffer.call(i).slice(), a = e.inputmask._valueGet();
                            if (a !== t) {
                                var o = r.getLastValidPosition.call(i);
                                -1 === o && a === r.getBufferTemplate.call(i).join("") ? n = [] : -1 !== o && u.call(i, n), 
                                d(e, n);
                            }
                        }
                    } else e.placeholder !== t && (e.placeholder = t, "" === e.placeholder && e.removeAttribute("placeholder"));
                }, t.applyInputValue = c, t.checkVal = f, t.clearOptionalTail = u, t.unmaskedvalue = function(e) {
                    var t = e ? e.inputmask : this, i = t.opts, n = t.maskset;
                    if (e) {
                        if (void 0 === e.inputmask) return e.value;
                        e.inputmask && e.inputmask.refreshValue && c(e, e.inputmask._valueGet(!0));
                    }
                    for (var a = [], o = n.validPositions, s = 0, l = o.length; s < l; s++) o[s] && o[s].match && (1 != o[s].match.static || Array.isArray(n.metadata) && !0 !== o[s].generatedInput) && a.push(o[s].input);
                    var u = 0 === a.length ? "" : (t.isRTL ? a.reverse() : a).join("");
                    if ("function" == typeof i.onUnMask) {
                        var f = (t.isRTL ? r.getBuffer.call(t).slice().reverse() : r.getBuffer.call(t)).join("");
                        u = i.onUnMask.call(t, f, u, i);
                    }
                    return u;
                }, t.writeBuffer = d;
                var n = i(2839), a = i(4713), r = i(8711), o = i(7215), s = i(9845), l = i(6030);
                function c(e, t) {
                    var i = e ? e.inputmask : this, n = i.opts;
                    e.inputmask.refreshValue = !1, "function" == typeof n.onBeforeMask && (t = n.onBeforeMask.call(i, t, n) || t), 
                    f(e, !0, !1, t = (t || "").toString().split("")), i.undoValue = i._valueGet(!0), 
                    (n.clearMaskOnLostFocus || n.clearIncomplete) && e.inputmask._valueGet() === r.getBufferTemplate.call(i).join("") && -1 === r.getLastValidPosition.call(i) && e.inputmask._valueSet("");
                }
                function u(e) {
                    e.length = 0;
                    for (var t, i = a.getMaskTemplate.call(this, !0, 0, !0, void 0, !0); void 0 !== (t = i.shift()); ) e.push(t);
                    return e;
                }
                function f(e, t, i, n, s) {
                    var c = e ? e.inputmask : this, u = c.maskset, f = c.opts, p = c.dependencyLib, h = n.slice(), v = "", m = -1, g = void 0, y = f.skipOptionalPartCharacter;
                    f.skipOptionalPartCharacter = "", r.resetMaskSet.call(c), u.tests = {}, m = f.radixPoint ? r.determineNewCaretPosition.call(c, {
                        begin: 0,
                        end: 0
                    }, !1, !1 === f.__financeInput ? "radixFocus" : void 0).begin : 0, u.p = m, c.caretPos = {
                        begin: m
                    };
                    var k = [], b = c.caretPos;
                    if (h.forEach((function(e, t) {
                        if (void 0 !== e) {
                            var n = new p.Event("_checkval");
                            n.key = e, v += e;
                            var o = r.getLastValidPosition.call(c, void 0, !0);
                            !function(e, t) {
                                for (var i = a.getMaskTemplate.call(c, !0, 0).slice(e, r.seekNext.call(c, e, !1, !1)).join("").replace(/'/g, ""), n = i.indexOf(t); n > 0 && " " === i[n - 1]; ) n--;
                                var o = 0 === n && !r.isMask.call(c, e) && (a.getTest.call(c, e).match.nativeDef === t.charAt(0) || !0 === a.getTest.call(c, e).match.static && a.getTest.call(c, e).match.nativeDef === "'" + t.charAt(0) || " " === a.getTest.call(c, e).match.nativeDef && (a.getTest.call(c, e + 1).match.nativeDef === t.charAt(0) || !0 === a.getTest.call(c, e + 1).match.static && a.getTest.call(c, e + 1).match.nativeDef === "'" + t.charAt(0)));
                                if (!o && n > 0 && !r.isMask.call(c, e, !1, !0)) {
                                    var s = r.seekNext.call(c, e);
                                    c.caretPos.begin < s && (c.caretPos = {
                                        begin: s
                                    });
                                }
                                return o;
                            }(m, v) ? (g = l.EventHandlers.keypressEvent.call(c, n, !0, !1, i, c.caretPos.begin)) && (m = c.caretPos.begin + 1, 
                            v = "") : g = l.EventHandlers.keypressEvent.call(c, n, !0, !1, i, o + 1), g ? (void 0 !== g.pos && u.validPositions[g.pos] && !0 === u.validPositions[g.pos].match.static && void 0 === u.validPositions[g.pos].alternation && (k.push(g.pos), 
                            c.isRTL || (g.forwardPosition = g.pos + 1)), d.call(c, void 0, r.getBuffer.call(c), g.forwardPosition, n, !1), 
                            c.caretPos = {
                                begin: g.forwardPosition,
                                end: g.forwardPosition
                            }, b = c.caretPos) : void 0 === u.validPositions[t] && h[t] === a.getPlaceholder.call(c, t) && r.isMask.call(c, t, !0) ? c.caretPos.begin++ : c.caretPos = b;
                        }
                    })), k.length > 0) {
                        var x, P, w = r.seekNext.call(c, -1, void 0, !1);
                        if (!o.isComplete.call(c, r.getBuffer.call(c)) && k.length <= w || o.isComplete.call(c, r.getBuffer.call(c)) && k.length > 0 && k.length !== w && 0 === k[0]) for (var S = w; void 0 !== (x = k.shift()); ) {
                            var M = new p.Event("_checkval");
                            if ((P = u.validPositions[x]).generatedInput = !0, M.key = P.input, (g = l.EventHandlers.keypressEvent.call(c, M, !0, !1, i, S)) && void 0 !== g.pos && g.pos !== x && u.validPositions[g.pos] && !0 === u.validPositions[g.pos].match.static) k.push(g.pos); else if (!g) break;
                            S++;
                        }
                    }
                    t && d.call(c, e, r.getBuffer.call(c), g ? g.forwardPosition : c.caretPos.begin, s || new p.Event("checkval"), s && ("input" === s.type && c.undoValue !== r.getBuffer.call(c).join("") || "paste" === s.type)), 
                    f.skipOptionalPartCharacter = y;
                }
                function d(e, t, i, a, s) {
                    var l = e ? e.inputmask : this, c = l.opts, u = l.dependencyLib;
                    if (a && "function" == typeof c.onBeforeWrite) {
                        var f = c.onBeforeWrite.call(l, a, t, i, c);
                        if (f) {
                            if (f.refreshFromBuffer) {
                                var d = f.refreshFromBuffer;
                                o.refreshFromBuffer.call(l, !0 === d ? d : d.start, d.end, f.buffer || t), t = r.getBuffer.call(l, !0);
                            }
                            void 0 !== i && (i = void 0 !== f.caret ? f.caret : i);
                        }
                    }
                    if (void 0 !== e && (e.inputmask._valueSet(t.join("")), void 0 === i || void 0 !== a && "blur" === a.type || r.caret.call(l, e, i, void 0, void 0, void 0 !== a && "keydown" === a.type && (a.key === n.keys.Delete || a.key === n.keys.Backspace)), 
                    !0 === s)) {
                        var p = u(e), h = e.inputmask._valueGet();
                        e.inputmask.skipInputEvent = !0, p.trigger("input"), setTimeout((function() {
                            h === r.getBufferTemplate.call(l).join("") ? p.trigger("cleared") : !0 === o.isComplete.call(l, t) && p.trigger("complete");
                        }), 0);
                    }
                }
            },
            2394: function(e, t, i) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.default = void 0;
                var n = i(157), a = m(i(4963)), r = m(i(9380)), o = i(2391), s = i(4713), l = i(8711), c = i(7215), u = i(7760), f = i(9716), d = m(i(7392)), p = m(i(3976)), h = m(i(8741));
                function v(e) {
                    return v = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function(e) {
                        return typeof e;
                    } : function(e) {
                        return e && "function" == typeof Symbol && e.constructor === Symbol && e !== Symbol.prototype ? "symbol" : typeof e;
                    }, v(e);
                }
                function m(e) {
                    return e && e.__esModule ? e : {
                        default: e
                    };
                }
                var g = r.default.document, y = "_inputmask_opts";
                function k(e, t, i) {
                    if (h.default) {
                        if (!(this instanceof k)) return new k(e, t, i);
                        this.dependencyLib = a.default, this.el = void 0, this.events = {}, this.maskset = void 0, 
                        !0 !== i && ("[object Object]" === Object.prototype.toString.call(e) ? t = e : (t = t || {}, 
                        e && (t.alias = e)), this.opts = a.default.extend(!0, {}, this.defaults, t), this.noMasksCache = t && void 0 !== t.definitions, 
                        this.userOptions = t || {}, b(this.opts.alias, t, this.opts)), this.refreshValue = !1, 
                        this.undoValue = void 0, this.$el = void 0, this.skipInputEvent = !1, this.validationEvent = !1, 
                        this.ignorable = !1, this.mouseEnter = !1, this.clicked = 0, this.originalPlaceholder = void 0, 
                        this.isComposing = !1, this.hasAlternator = !1;
                    }
                }
                function b(e, t, i) {
                    var n = k.prototype.aliases[e];
                    return n ? (n.alias && b(n.alias, void 0, i), a.default.extend(!0, i, n), a.default.extend(!0, i, t), 
                    !0) : (null === i.mask && (i.mask = e), !1);
                }
                k.prototype = {
                    dataAttribute: "data-inputmask",
                    defaults: p.default,
                    definitions: d.default,
                    aliases: {},
                    masksCache: {},
                    get isRTL() {
                        return this.opts.isRTL || this.opts.numericInput;
                    },
                    mask: function(e) {
                        var t = this;
                        return "string" == typeof e && (e = g.getElementById(e) || g.querySelectorAll(e)), 
                        (e = e.nodeName ? [ e ] : Array.isArray(e) ? e : [].slice.call(e)).forEach((function(e, i) {
                            var s = a.default.extend(!0, {}, t.opts);
                            if (function(e, t, i, n) {
                                function o(t, a) {
                                    var o = "" === n ? t : n + "-" + t;
                                    null !== (a = void 0 !== a ? a : e.getAttribute(o)) && ("string" == typeof a && (0 === t.indexOf("on") ? a = r.default[a] : "false" === a ? a = !1 : "true" === a && (a = !0)), 
                                    i[t] = a);
                                }
                                if (!0 === t.importDataAttributes) {
                                    var s, l, c, u, f = e.getAttribute(n);
                                    if (f && "" !== f && (f = f.replace(/'/g, '"'), l = JSON.parse("{" + f + "}")), 
                                    l) for (u in c = void 0, l) if ("alias" === u.toLowerCase()) {
                                        c = l[u];
                                        break;
                                    }
                                    for (s in o("alias", c), i.alias && b(i.alias, i, t), t) {
                                        if (l) for (u in c = void 0, l) if (u.toLowerCase() === s.toLowerCase()) {
                                            c = l[u];
                                            break;
                                        }
                                        o(s, c);
                                    }
                                }
                                a.default.extend(!0, t, i), ("rtl" === e.dir || t.rightAlign) && (e.style.textAlign = "right");
                                ("rtl" === e.dir || t.numericInput) && (e.dir = "ltr", e.removeAttribute("dir"), 
                                t.isRTL = !0);
                                return Object.keys(i).length;
                            }(e, s, a.default.extend(!0, {}, t.userOptions), t.dataAttribute)) {
                                var l = (0, o.generateMaskSet)(s, t.noMasksCache);
                                void 0 !== l && (void 0 !== e.inputmask && (e.inputmask.opts.autoUnmask = !0, e.inputmask.remove()), 
                                e.inputmask = new k(void 0, void 0, !0), e.inputmask.opts = s, e.inputmask.noMasksCache = t.noMasksCache, 
                                e.inputmask.userOptions = a.default.extend(!0, {}, t.userOptions), e.inputmask.el = e, 
                                e.inputmask.$el = (0, a.default)(e), e.inputmask.maskset = l, a.default.data(e, y, t.userOptions), 
                                n.mask.call(e.inputmask));
                            }
                        })), e && e[0] && e[0].inputmask || this;
                    },
                    option: function(e, t) {
                        return "string" == typeof e ? this.opts[e] : "object" === v(e) ? (a.default.extend(this.userOptions, e), 
                        this.el && !0 !== t && this.mask(this.el), this) : void 0;
                    },
                    unmaskedvalue: function(e) {
                        if (this.maskset = this.maskset || (0, o.generateMaskSet)(this.opts, this.noMasksCache), 
                        void 0 === this.el || void 0 !== e) {
                            var t = ("function" == typeof this.opts.onBeforeMask && this.opts.onBeforeMask.call(this, e, this.opts) || e).split("");
                            u.checkVal.call(this, void 0, !1, !1, t), "function" == typeof this.opts.onBeforeWrite && this.opts.onBeforeWrite.call(this, void 0, l.getBuffer.call(this), 0, this.opts);
                        }
                        return u.unmaskedvalue.call(this, this.el);
                    },
                    remove: function() {
                        if (this.el) {
                            a.default.data(this.el, y, null);
                            var e = this.opts.autoUnmask ? (0, u.unmaskedvalue)(this.el) : this._valueGet(this.opts.autoUnmask);
                            e !== l.getBufferTemplate.call(this).join("") ? this._valueSet(e, this.opts.autoUnmask) : this._valueSet(""), 
                            f.EventRuler.off(this.el), Object.getOwnPropertyDescriptor && Object.getPrototypeOf ? Object.getOwnPropertyDescriptor(Object.getPrototypeOf(this.el), "value") && this.__valueGet && Object.defineProperty(this.el, "value", {
                                get: this.__valueGet,
                                set: this.__valueSet,
                                configurable: !0
                            }) : g.__lookupGetter__ && this.el.__lookupGetter__("value") && this.__valueGet && (this.el.__defineGetter__("value", this.__valueGet), 
                            this.el.__defineSetter__("value", this.__valueSet)), this.el.inputmask = void 0;
                        }
                        return this.el;
                    },
                    getemptymask: function() {
                        return this.maskset = this.maskset || (0, o.generateMaskSet)(this.opts, this.noMasksCache), 
                        (this.isRTL ? l.getBufferTemplate.call(this).reverse() : l.getBufferTemplate.call(this)).join("");
                    },
                    hasMaskedValue: function() {
                        return !this.opts.autoUnmask;
                    },
                    isComplete: function() {
                        return this.maskset = this.maskset || (0, o.generateMaskSet)(this.opts, this.noMasksCache), 
                        c.isComplete.call(this, l.getBuffer.call(this));
                    },
                    getmetadata: function() {
                        if (this.maskset = this.maskset || (0, o.generateMaskSet)(this.opts, this.noMasksCache), 
                        Array.isArray(this.maskset.metadata)) {
                            var e = s.getMaskTemplate.call(this, !0, 0, !1).join("");
                            return this.maskset.metadata.forEach((function(t) {
                                return t.mask !== e || (e = t, !1);
                            })), e;
                        }
                        return this.maskset.metadata;
                    },
                    isValid: function(e) {
                        if (this.maskset = this.maskset || (0, o.generateMaskSet)(this.opts, this.noMasksCache), 
                        e) {
                            var t = ("function" == typeof this.opts.onBeforeMask && this.opts.onBeforeMask.call(this, e, this.opts) || e).split("");
                            u.checkVal.call(this, void 0, !0, !1, t);
                        } else e = this.isRTL ? l.getBuffer.call(this).slice().reverse().join("") : l.getBuffer.call(this).join("");
                        for (var i = l.getBuffer.call(this), n = l.determineLastRequiredPosition.call(this), a = i.length - 1; a > n && !l.isMask.call(this, a); a--) ;
                        return i.splice(n, a + 1 - n), c.isComplete.call(this, i) && e === (this.isRTL ? l.getBuffer.call(this).slice().reverse().join("") : l.getBuffer.call(this).join(""));
                    },
                    format: function(e, t) {
                        this.maskset = this.maskset || (0, o.generateMaskSet)(this.opts, this.noMasksCache);
                        var i = ("function" == typeof this.opts.onBeforeMask && this.opts.onBeforeMask.call(this, e, this.opts) || e).split("");
                        u.checkVal.call(this, void 0, !0, !1, i);
                        var n = this.isRTL ? l.getBuffer.call(this).slice().reverse().join("") : l.getBuffer.call(this).join("");
                        return t ? {
                            value: n,
                            metadata: this.getmetadata()
                        } : n;
                    },
                    setValue: function(e) {
                        this.el && (0, a.default)(this.el).trigger("setvalue", [ e ]);
                    },
                    analyseMask: o.analyseMask
                }, k.extendDefaults = function(e) {
                    a.default.extend(!0, k.prototype.defaults, e);
                }, k.extendDefinitions = function(e) {
                    a.default.extend(!0, k.prototype.definitions, e);
                }, k.extendAliases = function(e) {
                    a.default.extend(!0, k.prototype.aliases, e);
                }, k.format = function(e, t, i) {
                    return k(t).format(e, i);
                }, k.unmask = function(e, t) {
                    return k(t).unmaskedvalue(e);
                }, k.isValid = function(e, t) {
                    return k(t).isValid(e);
                }, k.remove = function(e) {
                    "string" == typeof e && (e = g.getElementById(e) || g.querySelectorAll(e)), (e = e.nodeName ? [ e ] : e).forEach((function(e) {
                        e.inputmask && e.inputmask.remove();
                    }));
                }, k.setValue = function(e, t) {
                    "string" == typeof e && (e = g.getElementById(e) || g.querySelectorAll(e)), (e = e.nodeName ? [ e ] : e).forEach((function(e) {
                        e.inputmask ? e.inputmask.setValue(t) : (0, a.default)(e).trigger("setvalue", [ t ]);
                    }));
                }, k.dependencyLib = a.default, r.default.Inputmask = k;
                var x = k;
                t.default = x;
            },
            5296: function(e, t, i) {
                function n(e) {
                    return n = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function(e) {
                        return typeof e;
                    } : function(e) {
                        return e && "function" == typeof Symbol && e.constructor === Symbol && e !== Symbol.prototype ? "symbol" : typeof e;
                    }, n(e);
                }
                var a = h(i(9380)), r = h(i(2394)), o = h(i(8741));
                function l(e) {
                    var t = f();
                    return function() {
                        var i, a = p(e);
                        if (t) {
                            var r = p(this).constructor;
                            i = Reflect.construct(a, arguments, r);
                        } else i = a.apply(this, arguments);
                        return function(e, t) {
                            if (t && ("object" === n(t) || "function" == typeof t)) return t;
                            if (void 0 !== t) throw new TypeError("Derived constructors may only return object or undefined");
                            return function(e) {
                                if (void 0 === e) throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
                                return e;
                            }(e);
                        }(this, i);
                    };
                }
                function c(e) {
                    var t = "function" == typeof Map ? new Map : void 0;
                    return c = function(e) {
                        if (null === e || (i = e, -1 === Function.toString.call(i).indexOf("[native code]"))) return e;
                        var i;
                        if ("function" != typeof e) throw new TypeError("Super expression must either be null or a function");
                        if (void 0 !== t) {
                            if (t.has(e)) return t.get(e);
                            t.set(e, n);
                        }
                        function n() {
                            return u(e, arguments, p(this).constructor);
                        }
                        return n.prototype = Object.create(e.prototype, {
                            constructor: {
                                value: n,
                                enumerable: !1,
                                writable: !0,
                                configurable: !0
                            }
                        }), d(n, e);
                    }, c(e);
                }
                function u(e, t, i) {
                    return u = f() ? Reflect.construct.bind() : function(e, t, i) {
                        var n = [ null ];
                        n.push.apply(n, t);
                        var a = new (Function.bind.apply(e, n));
                        return i && d(a, i.prototype), a;
                    }, u.apply(null, arguments);
                }
                function f() {
                    if ("undefined" == typeof Reflect || !Reflect.construct) return !1;
                    if (Reflect.construct.sham) return !1;
                    if ("function" == typeof Proxy) return !0;
                    try {
                        return Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], (function() {}))), 
                        !0;
                    } catch (e) {
                        return !1;
                    }
                }
                function d(e, t) {
                    return d = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function(e, t) {
                        return e.__proto__ = t, e;
                    }, d(e, t);
                }
                function p(e) {
                    return p = Object.setPrototypeOf ? Object.getPrototypeOf.bind() : function(e) {
                        return e.__proto__ || Object.getPrototypeOf(e);
                    }, p(e);
                }
                function h(e) {
                    return e && e.__esModule ? e : {
                        default: e
                    };
                }
                var v = a.default.document;
                if (o.default && v && v.head && v.head.attachShadow && a.default.customElements && void 0 === a.default.customElements.get("input-mask")) {
                    var m = function(e) {
                        !function(e, t) {
                            if ("function" != typeof t && null !== t) throw new TypeError("Super expression must either be null or a function");
                            e.prototype = Object.create(t && t.prototype, {
                                constructor: {
                                    value: e,
                                    writable: !0,
                                    configurable: !0
                                }
                            }), Object.defineProperty(e, "prototype", {
                                writable: !1
                            }), t && d(e, t);
                        }(o, e);
                        var t, a = l(o);
                        function o() {
                            var e;
                            !function(e, t) {
                                if (!(e instanceof t)) throw new TypeError("Cannot call a class as a function");
                            }(this, o);
                            var t = (e = a.call(this)).getAttributeNames(), i = e.attachShadow({
                                mode: "closed"
                            }), n = v.createElement("input");
                            for (var s in n.type = "text", i.appendChild(n), t) Object.prototype.hasOwnProperty.call(t, s) && n.setAttribute(t[s], e.getAttribute(t[s]));
                            var l = new r.default;
                            return l.dataAttribute = "", l.mask(n), n.inputmask.shadowRoot = i, e;
                        }
                        return t = o, Object.defineProperty(t, "prototype", {
                            writable: !1
                        }), t;
                    }(c(HTMLElement));
                    a.default.customElements.define("input-mask", m);
                }
            },
            2839: function(e, t) {
                function i(e, t) {
                    return function(e) {
                        if (Array.isArray(e)) return e;
                    }(e) || function(e, t) {
                        var i = null == e ? null : "undefined" != typeof Symbol && e[Symbol.iterator] || e["@@iterator"];
                        if (null != i) {
                            var n, a, r, o, s = [], l = !0, c = !1;
                            try {
                                if (r = (i = i.call(e)).next, 0 === t) {
                                    if (Object(i) !== i) return;
                                    l = !1;
                                } else for (;!(l = (n = r.call(i)).done) && (s.push(n.value), s.length !== t); l = !0) ;
                            } catch (e) {
                                c = !0, a = e;
                            } finally {
                                try {
                                    if (!l && null != i.return && (o = i.return(), Object(o) !== o)) return;
                                } finally {
                                    if (c) throw a;
                                }
                            }
                            return s;
                        }
                    }(e, t) || function(e, t) {
                        if (!e) return;
                        if ("string" == typeof e) return n(e, t);
                        var i = Object.prototype.toString.call(e).slice(8, -1);
                        "Object" === i && e.constructor && (i = e.constructor.name);
                        if ("Map" === i || "Set" === i) return Array.from(e);
                        if ("Arguments" === i || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(i)) return n(e, t);
                    }(e, t) || function() {
                        throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
                    }();
                }
                function n(e, t) {
                    (null == t || t > e.length) && (t = e.length);
                    for (var i = 0, n = new Array(t); i < t; i++) n[i] = e[i];
                    return n;
                }
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.keys = t.keyCode = void 0, t.toKey = function(e, t) {
                    return r[e] || (t ? String.fromCharCode(e) : String.fromCharCode(e).toLowerCase());
                }, t.toKeyCode = function(e) {
                    return a[e];
                };
                var a = {
                    AltGraph: 18,
                    ArrowDown: 40,
                    ArrowLeft: 37,
                    ArrowRight: 39,
                    ArrowUp: 38,
                    Backspace: 8,
                    BACKSPACE_SAFARI: 127,
                    CapsLock: 20,
                    Delete: 46,
                    End: 35,
                    Enter: 13,
                    Escape: 27,
                    Home: 36,
                    Insert: 45,
                    PageDown: 34,
                    PageUp: 33,
                    Space: 32,
                    Tab: 9,
                    c: 67,
                    x: 88,
                    z: 90,
                    Shift: 16,
                    Control: 17,
                    Alt: 18,
                    Pause: 19,
                    Meta_LEFT: 91,
                    Meta_RIGHT: 92,
                    ContextMenu: 93,
                    Process: 229,
                    Unidentified: 229,
                    F1: 112,
                    F2: 113,
                    F3: 114,
                    F4: 115,
                    F5: 116,
                    F6: 117,
                    F7: 118,
                    F8: 119,
                    F9: 120,
                    F10: 121,
                    F11: 122,
                    F12: 123
                };
                t.keyCode = a;
                var r = Object.entries(a).reduce((function(e, t) {
                    var n = i(t, 2), a = n[0], r = n[1];
                    return e[r] = void 0 === e[r] ? a : e[r], e;
                }), {}), o = Object.entries(a).reduce((function(e, t) {
                    var n = i(t, 2), a = n[0];
                    return e[a] = "Space" === a ? " " : a, e;
                }), {});
                t.keys = o;
            },
            2391: function(e, t, i) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.analyseMask = function(e, t, i) {
                    var n, o, s, l, c, u, f = /(?:[?*+]|\{[0-9+*]+(?:,[0-9+*]*)?(?:\|[0-9+*]*)?\})|[^.?*+^${[]()|\\]+|./g, d = /\[\^?]?(?:[^\\\]]+|\\[\S\s]?)*]?|\\(?:0(?:[0-3][0-7]{0,2}|[4-7][0-7]?)?|[1-9][0-9]*|x[0-9A-Fa-f]{2}|u[0-9A-Fa-f]{4}|c[A-Za-z]|[\S\s]?)|\((?:\?[:=!]?)?|(?:[?*+]|\{[0-9]+(?:,[0-9]*)?\})\??|[^.?*+^${[()|\\]+|./g, p = !1, h = new a.default, v = [], m = [], g = !1;
                    function y(e, n, a) {
                        a = void 0 !== a ? a : e.matches.length;
                        var o = e.matches[a - 1];
                        if (t) {
                            if (0 === n.indexOf("[") || p && /\\d|\\s|\\w|\\p/i.test(n) || "." === n) {
                                var s = i.casing ? "i" : "";
                                /^\\p\{.*}$/i.test(n) && (s += "u"), e.matches.splice(a++, 0, {
                                    fn: new RegExp(n, s),
                                    static: !1,
                                    optionality: !1,
                                    newBlockMarker: void 0 === o ? "master" : o.def !== n,
                                    casing: null,
                                    def: n,
                                    placeholder: void 0,
                                    nativeDef: n
                                });
                            } else p && (n = n[n.length - 1]), n.split("").forEach((function(t, n) {
                                o = e.matches[a - 1], e.matches.splice(a++, 0, {
                                    fn: /[a-z]/i.test(i.staticDefinitionSymbol || t) ? new RegExp("[" + (i.staticDefinitionSymbol || t) + "]", i.casing ? "i" : "") : null,
                                    static: !0,
                                    optionality: !1,
                                    newBlockMarker: void 0 === o ? "master" : o.def !== t && !0 !== o.static,
                                    casing: null,
                                    def: i.staticDefinitionSymbol || t,
                                    placeholder: void 0 !== i.staticDefinitionSymbol ? t : void 0,
                                    nativeDef: (p ? "'" : "") + t
                                });
                            }));
                            p = !1;
                        } else {
                            var l = i.definitions && i.definitions[n] || i.usePrototypeDefinitions && r.default.prototype.definitions[n];
                            l && !p ? e.matches.splice(a++, 0, {
                                fn: l.validator ? "string" == typeof l.validator ? new RegExp(l.validator, i.casing ? "i" : "") : new function() {
                                    this.test = l.validator;
                                } : new RegExp("."),
                                static: l.static || !1,
                                optionality: l.optional || !1,
                                defOptionality: l.optional || !1,
                                newBlockMarker: void 0 === o || l.optional ? "master" : o.def !== (l.definitionSymbol || n),
                                casing: l.casing,
                                def: l.definitionSymbol || n,
                                placeholder: l.placeholder,
                                nativeDef: n,
                                generated: l.generated
                            }) : (e.matches.splice(a++, 0, {
                                fn: /[a-z]/i.test(i.staticDefinitionSymbol || n) ? new RegExp("[" + (i.staticDefinitionSymbol || n) + "]", i.casing ? "i" : "") : null,
                                static: !0,
                                optionality: !1,
                                newBlockMarker: void 0 === o ? "master" : o.def !== n && !0 !== o.static,
                                casing: null,
                                def: i.staticDefinitionSymbol || n,
                                placeholder: void 0 !== i.staticDefinitionSymbol ? n : void 0,
                                nativeDef: (p ? "'" : "") + n
                            }), p = !1);
                        }
                    }
                    function k() {
                        if (v.length > 0) {
                            if (y(l = v[v.length - 1], o), l.isAlternator) {
                                c = v.pop();
                                for (var e = 0; e < c.matches.length; e++) c.matches[e].isGroup && (c.matches[e].isGroup = !1);
                                v.length > 0 ? (l = v[v.length - 1]).matches.push(c) : h.matches.push(c);
                            }
                        } else y(h, o);
                    }
                    function b(e) {
                        var t = new a.default(!0);
                        return t.openGroup = !1, t.matches = e, t;
                    }
                    function x() {
                        if ((s = v.pop()).openGroup = !1, void 0 !== s) if (v.length > 0) {
                            if ((l = v[v.length - 1]).matches.push(s), l.isAlternator) {
                                for (var e = (c = v.pop()).matches[0].matches ? c.matches[0].matches.length : 1, t = 0; t < c.matches.length; t++) c.matches[t].isGroup = !1, 
                                c.matches[t].alternatorGroup = !1, null === i.keepStatic && e < (c.matches[t].matches ? c.matches[t].matches.length : 1) && (i.keepStatic = !0), 
                                e = c.matches[t].matches ? c.matches[t].matches.length : 1;
                                v.length > 0 ? (l = v[v.length - 1]).matches.push(c) : h.matches.push(c);
                            }
                        } else h.matches.push(s); else k();
                    }
                    function P(e) {
                        var t = e.pop();
                        return t.isQuantifier && (t = b([ e.pop(), t ])), t;
                    }
                    t && (i.optionalmarker[0] = void 0, i.optionalmarker[1] = void 0);
                    for (;n = t ? d.exec(e) : f.exec(e); ) {
                        if (o = n[0], t) {
                            switch (o.charAt(0)) {
                              case "?":
                                o = "{0,1}";
                                break;

                              case "+":
                              case "*":
                                o = "{" + o + "}";
                                break;

                              case "|":
                                if (0 === v.length) {
                                    var w = b(h.matches);
                                    w.openGroup = !0, v.push(w), h.matches = [], g = !0;
                                }
                            }
                            switch (o) {
                              case "\\d":
                                o = "[0-9]";
                                break;

                              case "\\p":
                                o += d.exec(e)[0], o += d.exec(e)[0];
                            }
                        }
                        if (p) k(); else switch (o.charAt(0)) {
                          case "$":
                          case "^":
                            t || k();
                            break;

                          case i.escapeChar:
                            p = !0, t && k();
                            break;

                          case i.optionalmarker[1]:
                          case i.groupmarker[1]:
                            x();
                            break;

                          case i.optionalmarker[0]:
                            v.push(new a.default(!1, !0));
                            break;

                          case i.groupmarker[0]:
                            v.push(new a.default(!0));
                            break;

                          case i.quantifiermarker[0]:
                            var S = new a.default(!1, !1, !0), M = (o = o.replace(/[{}?]/g, "")).split("|"), _ = M[0].split(","), O = isNaN(_[0]) ? _[0] : parseInt(_[0]), E = 1 === _.length ? O : isNaN(_[1]) ? _[1] : parseInt(_[1]), T = isNaN(M[1]) ? M[1] : parseInt(M[1]);
                            "*" !== O && "+" !== O || (O = "*" === E ? 0 : 1), S.quantifier = {
                                min: O,
                                max: E,
                                jit: T
                            };
                            var j = v.length > 0 ? v[v.length - 1].matches : h.matches;
                            (n = j.pop()).isGroup || (n = b([ n ])), j.push(n), j.push(S);
                            break;

                          case i.alternatormarker:
                            if (v.length > 0) {
                                var A = (l = v[v.length - 1]).matches[l.matches.length - 1];
                                u = l.openGroup && (void 0 === A.matches || !1 === A.isGroup && !1 === A.isAlternator) ? v.pop() : P(l.matches);
                            } else u = P(h.matches);
                            if (u.isAlternator) v.push(u); else if (u.alternatorGroup ? (c = v.pop(), u.alternatorGroup = !1) : c = new a.default(!1, !1, !1, !0), 
                            c.matches.push(u), v.push(c), u.openGroup) {
                                u.openGroup = !1;
                                var D = new a.default(!0);
                                D.alternatorGroup = !0, v.push(D);
                            }
                            break;

                          default:
                            k();
                        }
                    }
                    g && x();
                    for (;v.length > 0; ) s = v.pop(), h.matches.push(s);
                    h.matches.length > 0 && (!function e(n) {
                        n && n.matches && n.matches.forEach((function(a, r) {
                            var o = n.matches[r + 1];
                            (void 0 === o || void 0 === o.matches || !1 === o.isQuantifier) && a && a.isGroup && (a.isGroup = !1, 
                            t || (y(a, i.groupmarker[0], 0), !0 !== a.openGroup && y(a, i.groupmarker[1]))), 
                            e(a);
                        }));
                    }(h), m.push(h));
                    (i.numericInput || i.isRTL) && function e(t) {
                        for (var n in t.matches = t.matches.reverse(), t.matches) if (Object.prototype.hasOwnProperty.call(t.matches, n)) {
                            var a = parseInt(n);
                            if (t.matches[n].isQuantifier && t.matches[a + 1] && t.matches[a + 1].isGroup) {
                                var r = t.matches[n];
                                t.matches.splice(n, 1), t.matches.splice(a + 1, 0, r);
                            }
                            void 0 !== t.matches[n].matches ? t.matches[n] = e(t.matches[n]) : t.matches[n] = ((o = t.matches[n]) === i.optionalmarker[0] ? o = i.optionalmarker[1] : o === i.optionalmarker[1] ? o = i.optionalmarker[0] : o === i.groupmarker[0] ? o = i.groupmarker[1] : o === i.groupmarker[1] && (o = i.groupmarker[0]), 
                            o);
                        }
                        var o;
                        return t;
                    }(m[0]);
                    return m;
                }, t.generateMaskSet = function(e, t) {
                    var i;
                    function a(e, t) {
                        var i = t.repeat, n = t.groupmarker, a = t.quantifiermarker, r = t.keepStatic;
                        if (i > 0 || "*" === i || "+" === i) {
                            var l = "*" === i ? 0 : "+" === i ? 1 : i;
                            e = n[0] + e + n[1] + a[0] + l + "," + i + a[1];
                        }
                        if (!0 === r) {
                            var c = e.match(new RegExp("(.)\\[([^\\]]*)\\]", "g"));
                            c && c.forEach((function(t, i) {
                                var n = function(e, t) {
                                    return function(e) {
                                        if (Array.isArray(e)) return e;
                                    }(e) || function(e, t) {
                                        var i = null == e ? null : "undefined" != typeof Symbol && e[Symbol.iterator] || e["@@iterator"];
                                        if (null != i) {
                                            var n, a, r, o, s = [], l = !0, c = !1;
                                            try {
                                                if (r = (i = i.call(e)).next, 0 === t) {
                                                    if (Object(i) !== i) return;
                                                    l = !1;
                                                } else for (;!(l = (n = r.call(i)).done) && (s.push(n.value), s.length !== t); l = !0) ;
                                            } catch (e) {
                                                c = !0, a = e;
                                            } finally {
                                                try {
                                                    if (!l && null != i.return && (o = i.return(), Object(o) !== o)) return;
                                                } finally {
                                                    if (c) throw a;
                                                }
                                            }
                                            return s;
                                        }
                                    }(e, t) || function(e, t) {
                                        if (!e) return;
                                        if ("string" == typeof e) return s(e, t);
                                        var i = Object.prototype.toString.call(e).slice(8, -1);
                                        "Object" === i && e.constructor && (i = e.constructor.name);
                                        if ("Map" === i || "Set" === i) return Array.from(e);
                                        if ("Arguments" === i || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(i)) return s(e, t);
                                    }(e, t) || function() {
                                        throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
                                    }();
                                }(t.split("["), 2), a = n[0], r = n[1];
                                r = r.replace("]", ""), e = e.replace(new RegExp("".concat((0, o.default)(a), "\\[").concat((0, o.default)(r), "\\]")), a.charAt(0) === r.charAt(0) ? "(".concat(a, "|").concat(a).concat(r, ")") : "".concat(a, "[").concat(r, "]"));
                            }));
                        }
                        return e;
                    }
                    function l(e, i, o) {
                        var s, l, c = !1;
                        return null !== e && "" !== e || ((c = null !== o.regex) ? e = (e = o.regex).replace(/^(\^)(.*)(\$)$/, "$2") : (c = !0, 
                        e = ".*")), 1 === e.length && !1 === o.greedy && 0 !== o.repeat && (o.placeholder = ""), 
                        e = a(e, o), l = c ? "regex_" + o.regex : o.numericInput ? e.split("").reverse().join("") : e, 
                        null !== o.keepStatic && (l = "ks_" + o.keepStatic + l), void 0 === r.default.prototype.masksCache[l] || !0 === t ? (s = {
                            mask: e,
                            maskToken: r.default.prototype.analyseMask(e, c, o),
                            validPositions: [],
                            _buffer: void 0,
                            buffer: void 0,
                            tests: {},
                            excludes: {},
                            metadata: i,
                            maskLength: void 0,
                            jitOffset: {}
                        }, !0 !== t && (r.default.prototype.masksCache[l] = s, s = n.default.extend(!0, {}, r.default.prototype.masksCache[l]))) : s = n.default.extend(!0, {}, r.default.prototype.masksCache[l]), 
                        s;
                    }
                    "function" == typeof e.mask && (e.mask = e.mask(e));
                    if (Array.isArray(e.mask)) {
                        if (e.mask.length > 1) {
                            null === e.keepStatic && (e.keepStatic = !0);
                            var c = e.groupmarker[0];
                            return (e.isRTL ? e.mask.reverse() : e.mask).forEach((function(t) {
                                c.length > 1 && (c += e.alternatormarker), void 0 !== t.mask && "function" != typeof t.mask ? c += t.mask : c += t;
                            })), l(c += e.groupmarker[1], e.mask, e);
                        }
                        e.mask = e.mask.pop();
                    }
                    i = e.mask && void 0 !== e.mask.mask && "function" != typeof e.mask.mask ? l(e.mask.mask, e.mask, e) : l(e.mask, e.mask, e);
                    null === e.keepStatic && (e.keepStatic = !1);
                    return i;
                };
                var n = l(i(4963)), a = l(i(9695)), r = l(i(2394)), o = l(i(7184));
                function s(e, t) {
                    (null == t || t > e.length) && (t = e.length);
                    for (var i = 0, n = new Array(t); i < t; i++) n[i] = e[i];
                    return n;
                }
                function l(e) {
                    return e && e.__esModule ? e : {
                        default: e
                    };
                }
            },
            157: function(e, t, i) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.mask = function() {
                    var e = this, t = this.opts, i = this.el, u = this.dependencyLib;
                    o.EventRuler.off(i);
                    var f = function(t, i) {
                        "textarea" !== t.tagName.toLowerCase() && i.ignorables.push(n.keys.Enter);
                        var s = t.getAttribute("type"), l = "input" === t.tagName.toLowerCase() && i.supportsInputType.includes(s) || t.isContentEditable || "textarea" === t.tagName.toLowerCase();
                        if (!l) if ("input" === t.tagName.toLowerCase()) {
                            var c = document.createElement("input");
                            c.setAttribute("type", s), l = "text" === c.type, c = null;
                        } else l = "partial";
                        return !1 !== l ? function(t) {
                            var n, s;
                            function l() {
                                return this.inputmask ? this.inputmask.opts.autoUnmask ? this.inputmask.unmaskedvalue() : -1 !== a.getLastValidPosition.call(e) || !0 !== i.nullable ? (this.inputmask.shadowRoot || this.ownerDocument).activeElement === this && i.clearMaskOnLostFocus ? (e.isRTL ? r.clearOptionalTail.call(e, a.getBuffer.call(e).slice()).reverse() : r.clearOptionalTail.call(e, a.getBuffer.call(e).slice())).join("") : n.call(this) : "" : n.call(this);
                            }
                            function c(e) {
                                s.call(this, e), this.inputmask && (0, r.applyInputValue)(this, e);
                            }
                            if (!t.inputmask.__valueGet) {
                                if (!0 !== i.noValuePatching) {
                                    if (Object.getOwnPropertyDescriptor) {
                                        var f = Object.getPrototypeOf ? Object.getOwnPropertyDescriptor(Object.getPrototypeOf(t), "value") : void 0;
                                        f && f.get && f.set ? (n = f.get, s = f.set, Object.defineProperty(t, "value", {
                                            get: l,
                                            set: c,
                                            configurable: !0
                                        })) : "input" !== t.tagName.toLowerCase() && (n = function() {
                                            return this.textContent;
                                        }, s = function(e) {
                                            this.textContent = e;
                                        }, Object.defineProperty(t, "value", {
                                            get: l,
                                            set: c,
                                            configurable: !0
                                        }));
                                    } else document.__lookupGetter__ && t.__lookupGetter__("value") && (n = t.__lookupGetter__("value"), 
                                    s = t.__lookupSetter__("value"), t.__defineGetter__("value", l), t.__defineSetter__("value", c));
                                    t.inputmask.__valueGet = n, t.inputmask.__valueSet = s;
                                }
                                t.inputmask._valueGet = function(t) {
                                    return e.isRTL && !0 !== t ? n.call(this.el).split("").reverse().join("") : n.call(this.el);
                                }, t.inputmask._valueSet = function(t, i) {
                                    s.call(this.el, null == t ? "" : !0 !== i && e.isRTL ? t.split("").reverse().join("") : t);
                                }, void 0 === n && (n = function() {
                                    return this.value;
                                }, s = function(e) {
                                    this.value = e;
                                }, function(t) {
                                    if (u.valHooks && (void 0 === u.valHooks[t] || !0 !== u.valHooks[t].inputmaskpatch)) {
                                        var n = u.valHooks[t] && u.valHooks[t].get ? u.valHooks[t].get : function(e) {
                                            return e.value;
                                        }, o = u.valHooks[t] && u.valHooks[t].set ? u.valHooks[t].set : function(e, t) {
                                            return e.value = t, e;
                                        };
                                        u.valHooks[t] = {
                                            get: function(t) {
                                                if (t.inputmask) {
                                                    if (t.inputmask.opts.autoUnmask) return t.inputmask.unmaskedvalue();
                                                    var r = n(t);
                                                    return -1 !== a.getLastValidPosition.call(e, void 0, void 0, t.inputmask.maskset.validPositions) || !0 !== i.nullable ? r : "";
                                                }
                                                return n(t);
                                            },
                                            set: function(e, t) {
                                                var i = o(e, t);
                                                return e.inputmask && (0, r.applyInputValue)(e, t), i;
                                            },
                                            inputmaskpatch: !0
                                        };
                                    }
                                }(t.type), function(e) {
                                    o.EventRuler.on(e, "mouseenter", (function() {
                                        var e = this, t = e.inputmask._valueGet(!0);
                                        t != (e.inputmask.isRTL ? a.getBuffer.call(e.inputmask).slice().reverse() : a.getBuffer.call(e.inputmask)).join("") && (0, r.applyInputValue)(e, t);
                                    }));
                                }(t));
                            }
                        }(t) : t.inputmask = void 0, l;
                    }(i, t);
                    if (!1 !== f) {
                        e.originalPlaceholder = i.placeholder, e.maxLength = void 0 !== i ? i.maxLength : void 0, 
                        -1 === e.maxLength && (e.maxLength = void 0), "inputMode" in i && null === i.getAttribute("inputmode") && (i.inputMode = t.inputmode, 
                        i.setAttribute("inputmode", t.inputmode)), !0 === f && (t.showMaskOnFocus = t.showMaskOnFocus && -1 === [ "cc-number", "cc-exp" ].indexOf(i.autocomplete), 
                        s.iphone && (t.insertModeVisual = !1, i.setAttribute("autocorrect", "off")), o.EventRuler.on(i, "submit", c.EventHandlers.submitEvent), 
                        o.EventRuler.on(i, "reset", c.EventHandlers.resetEvent), o.EventRuler.on(i, "blur", c.EventHandlers.blurEvent), 
                        o.EventRuler.on(i, "focus", c.EventHandlers.focusEvent), o.EventRuler.on(i, "invalid", c.EventHandlers.invalidEvent), 
                        o.EventRuler.on(i, "click", c.EventHandlers.clickEvent), o.EventRuler.on(i, "mouseleave", c.EventHandlers.mouseleaveEvent), 
                        o.EventRuler.on(i, "mouseenter", c.EventHandlers.mouseenterEvent), o.EventRuler.on(i, "paste", c.EventHandlers.pasteEvent), 
                        o.EventRuler.on(i, "cut", c.EventHandlers.cutEvent), o.EventRuler.on(i, "complete", t.oncomplete), 
                        o.EventRuler.on(i, "incomplete", t.onincomplete), o.EventRuler.on(i, "cleared", t.oncleared), 
                        !0 !== t.inputEventOnly && o.EventRuler.on(i, "keydown", c.EventHandlers.keyEvent), 
                        (s.mobile || t.inputEventOnly) && i.removeAttribute("maxLength"), o.EventRuler.on(i, "input", c.EventHandlers.inputFallBackEvent)), 
                        o.EventRuler.on(i, "setvalue", c.EventHandlers.setValueEvent), a.getBufferTemplate.call(e).join(""), 
                        e.undoValue = e._valueGet(!0);
                        var d = (i.inputmask.shadowRoot || i.ownerDocument).activeElement;
                        if ("" !== i.inputmask._valueGet(!0) || !1 === t.clearMaskOnLostFocus || d === i) {
                            (0, r.applyInputValue)(i, i.inputmask._valueGet(!0), t);
                            var p = a.getBuffer.call(e).slice();
                            !1 === l.isComplete.call(e, p) && t.clearIncomplete && a.resetMaskSet.call(e), t.clearMaskOnLostFocus && d !== i && (-1 === a.getLastValidPosition.call(e) ? p = [] : r.clearOptionalTail.call(e, p)), 
                            (!1 === t.clearMaskOnLostFocus || t.showMaskOnFocus && d === i || "" !== i.inputmask._valueGet(!0)) && (0, r.writeBuffer)(i, p), d === i && a.caret.call(e, i, a.seekNext.call(e, a.getLastValidPosition.call(e)));
                        }
                    }
                };
                var n = i(2839), a = i(8711), r = i(7760), o = i(9716), s = i(9845), l = i(7215), c = i(6030);
            },
            9695: function(e, t) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.default = function(e, t, i, n) {
                    this.matches = [], this.openGroup = e || !1, this.alternatorGroup = !1, this.isGroup = e || !1, 
                    this.isOptional = t || !1, this.isQuantifier = i || !1, this.isAlternator = n || !1, 
                    this.quantifier = {
                        min: 1,
                        max: 1
                    };
                };
            },
            3194: function() {
                Array.prototype.includes || Object.defineProperty(Array.prototype, "includes", {
                    value: function(e, t) {
                        if (null == this) throw new TypeError('"this" is null or not defined');
                        var i = Object(this), n = i.length >>> 0;
                        if (0 === n) return !1;
                        for (var a = 0 | t, r = Math.max(a >= 0 ? a : n - Math.abs(a), 0); r < n; ) {
                            if (i[r] === e) return !0;
                            r++;
                        }
                        return !1;
                    }
                });
            },
            9302: function() {
                var e = Function.bind.call(Function.call, Array.prototype.reduce), t = Function.bind.call(Function.call, Object.prototype.propertyIsEnumerable), i = Function.bind.call(Function.call, Array.prototype.concat), n = Object.keys;
                Object.entries || (Object.entries = function(a) {
                    return e(n(a), (function(e, n) {
                        return i(e, "string" == typeof n && t(a, n) ? [ [ n, a[n] ] ] : []);
                    }), []);
                });
            },
            7149: function() {
                function e(t) {
                    return e = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function(e) {
                        return typeof e;
                    } : function(e) {
                        return e && "function" == typeof Symbol && e.constructor === Symbol && e !== Symbol.prototype ? "symbol" : typeof e;
                    }, e(t);
                }
                "function" != typeof Object.getPrototypeOf && (Object.getPrototypeOf = "object" === e("test".__proto__) ? function(e) {
                    return e.__proto__;
                } : function(e) {
                    return e.constructor.prototype;
                });
            },
            4013: function() {
                String.prototype.includes || (String.prototype.includes = function(e, t) {
                    return "number" != typeof t && (t = 0), !(t + e.length > this.length) && -1 !== this.indexOf(e, t);
                });
            },
            8711: function(e, t, i) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.caret = function(e, t, i, n, a) {
                    var r, o = this, s = this.opts;
                    if (void 0 === t) return "selectionStart" in e && "selectionEnd" in e ? (t = e.selectionStart, 
                    i = e.selectionEnd) : window.getSelection ? (r = window.getSelection().getRangeAt(0)).commonAncestorContainer.parentNode !== e && r.commonAncestorContainer !== e || (t = r.startOffset, 
                    i = r.endOffset) : document.selection && document.selection.createRange && (i = (t = 0 - (r = document.selection.createRange()).duplicate().moveStart("character", -e.inputmask._valueGet().length)) + r.text.length), 
                    {
                        begin: n ? t : c.call(o, t),
                        end: n ? i : c.call(o, i)
                    };
                    if (Array.isArray(t) && (i = o.isRTL ? t[0] : t[1], t = o.isRTL ? t[1] : t[0]), 
                    void 0 !== t.begin && (i = o.isRTL ? t.begin : t.end, t = o.isRTL ? t.end : t.begin), 
                    "number" == typeof t) {
                        t = n ? t : c.call(o, t), i = "number" == typeof (i = n ? i : c.call(o, i)) ? i : t;
                        var l = parseInt(((e.ownerDocument.defaultView || window).getComputedStyle ? (e.ownerDocument.defaultView || window).getComputedStyle(e, null) : e.currentStyle).fontSize) * i;
                        if (e.scrollLeft = l > e.scrollWidth ? l : 0, e.inputmask.caretPos = {
                            begin: t,
                            end: i
                        }, s.insertModeVisual && !1 === s.insertMode && t === i && (a || i++), e === (e.inputmask.shadowRoot || e.ownerDocument).activeElement) if ("setSelectionRange" in e) e.setSelectionRange(t, i); else if (window.getSelection) {
                            if (r = document.createRange(), void 0 === e.firstChild || null === e.firstChild) {
                                var u = document.createTextNode("");
                                e.appendChild(u);
                            }
                            r.setStart(e.firstChild, t < e.inputmask._valueGet().length ? t : e.inputmask._valueGet().length), 
                            r.setEnd(e.firstChild, i < e.inputmask._valueGet().length ? i : e.inputmask._valueGet().length), 
                            r.collapse(!0);
                            var f = window.getSelection();
                            f.removeAllRanges(), f.addRange(r);
                        } else e.createTextRange && ((r = e.createTextRange()).collapse(!0), r.moveEnd("character", i), 
                        r.moveStart("character", t), r.select());
                    }
                }, t.determineLastRequiredPosition = function(e) {
                    var t, i, r = this, s = r.maskset, l = r.dependencyLib, c = n.getMaskTemplate.call(r, !0, o.call(r), !0, !0), u = c.length, f = o.call(r), d = {}, p = s.validPositions[f], h = void 0 !== p ? p.locator.slice() : void 0;
                    for (t = f + 1; t < c.length; t++) h = (i = n.getTestTemplate.call(r, t, h, t - 1)).locator.slice(), 
                    d[t] = l.extend(!0, {}, i);
                    var v = p && void 0 !== p.alternation ? p.locator[p.alternation] : void 0;
                    for (t = u - 1; t > f && (((i = d[t]).match.optionality || i.match.optionalQuantifier && i.match.newBlockMarker || v && (v !== d[t].locator[p.alternation] && 1 != i.match.static || !0 === i.match.static && i.locator[p.alternation] && a.checkAlternationMatch.call(r, i.locator[p.alternation].toString().split(","), v.toString().split(",")) && "" !== n.getTests.call(r, t)[0].def)) && c[t] === n.getPlaceholder.call(r, t, i.match)); t--) u--;
                    return e ? {
                        l: u,
                        def: d[u] ? d[u].match : void 0
                    } : u;
                }, t.determineNewCaretPosition = function(e, t, i) {
                    var a = this, c = a.maskset, u = a.opts;
                    t && (a.isRTL ? e.end = e.begin : e.begin = e.end);
                    if (e.begin === e.end) {
                        switch (i = i || u.positionCaretOnClick) {
                          case "none":
                            break;

                          case "select":
                            e = {
                                begin: 0,
                                end: r.call(a).length
                            };
                            break;

                          case "ignore":
                            e.end = e.begin = l.call(a, o.call(a));
                            break;

                          case "radixFocus":
                            if (a.clicked > 1 && 0 == c.validPositions.length) break;
                            if (function(e) {
                                if ("" !== u.radixPoint && 0 !== u.digits) {
                                    var t = c.validPositions;
                                    if (void 0 === t[e] || t[e].input === n.getPlaceholder.call(a, e)) {
                                        if (e < l.call(a, -1)) return !0;
                                        var i = r.call(a).indexOf(u.radixPoint);
                                        if (-1 !== i) {
                                            for (var o = 0, s = t.length; o < s; o++) if (t[o] && i < o && t[o].input !== n.getPlaceholder.call(a, o)) return !1;
                                            return !0;
                                        }
                                    }
                                }
                                return !1;
                            }(e.begin)) {
                                var f = r.call(a).join("").indexOf(u.radixPoint);
                                e.end = e.begin = u.numericInput ? l.call(a, f) : f;
                                break;
                            }

                          default:
                            var d = e.begin, p = o.call(a, d, !0), h = l.call(a, -1 !== p || s.call(a, 0) ? p : -1);
                            if (d <= h) e.end = e.begin = s.call(a, d, !1, !0) ? d : l.call(a, d); else {
                                var v = c.validPositions[p], m = n.getTestTemplate.call(a, h, v ? v.match.locator : void 0, v), g = n.getPlaceholder.call(a, h, m.match);
                                if ("" !== g && r.call(a)[h] !== g && !0 !== m.match.optionalQuantifier && !0 !== m.match.newBlockMarker || !s.call(a, h, u.keepStatic, !0) && m.match.def === g) {
                                    var y = l.call(a, h);
                                    (d >= y || d === h) && (h = y);
                                }
                                e.end = e.begin = h;
                            }
                        }
                        return e;
                    }
                }, t.getBuffer = r, t.getBufferTemplate = function() {
                    var e = this.maskset;
                    void 0 === e._buffer && (e._buffer = n.getMaskTemplate.call(this, !1, 1), void 0 === e.buffer && (e.buffer = e._buffer.slice()));
                    return e._buffer;
                }, t.getLastValidPosition = o, t.isMask = s, t.resetMaskSet = function(e) {
                    var t = this.maskset;
                    t.buffer = void 0, !0 !== e && (t.validPositions = [], t.p = 0);
                }, t.seekNext = l, t.seekPrevious = function(e, t) {
                    var i = this, a = e - 1;
                    if (e <= 0) return 0;
                    for (;a > 0 && (!0 === t && (!0 !== n.getTest.call(i, a).match.newBlockMarker || !s.call(i, a, void 0, !0)) || !0 !== t && !s.call(i, a, void 0, !0)); ) a--;
                    return a;
                }, t.translatePosition = c;
                var n = i(4713), a = i(7215);
                function r(e) {
                    var t = this, i = t.maskset;
                    return void 0 !== i.buffer && !0 !== e || (i.buffer = n.getMaskTemplate.call(t, !0, o.call(t), !0), 
                    void 0 === i._buffer && (i._buffer = i.buffer.slice())), i.buffer;
                }
                function o(e, t, i) {
                    var n = this.maskset, a = -1, r = -1, o = i || n.validPositions;
                    void 0 === e && (e = -1);
                    for (var s = 0, l = o.length; s < l; s++) o[s] && (t || !0 !== o[s].generatedInput) && (s <= e && (a = s), 
                    s >= e && (r = s));
                    return -1 === a || a == e ? r : -1 == r || e - a < r - e ? a : r;
                }
                function s(e, t, i) {
                    var a = this, r = this.maskset, o = n.getTestTemplate.call(a, e).match;
                    if ("" === o.def && (o = n.getTest.call(a, e).match), !0 !== o.static) return o.fn;
                    if (!0 === i && void 0 !== r.validPositions[e] && !0 !== r.validPositions[e].generatedInput) return !0;
                    if (!0 !== t && e > -1) {
                        if (i) {
                            var s = n.getTests.call(a, e);
                            return s.length > 1 + ("" === s[s.length - 1].match.def ? 1 : 0);
                        }
                        var l = n.determineTestTemplate.call(a, e, n.getTests.call(a, e)), c = n.getPlaceholder.call(a, e, l.match);
                        return l.match.def !== c;
                    }
                    return !1;
                }
                function l(e, t, i) {
                    var a = this;
                    void 0 === i && (i = !0);
                    for (var r = e + 1; "" !== n.getTest.call(a, r).match.def && (!0 === t && (!0 !== n.getTest.call(a, r).match.newBlockMarker || !s.call(a, r, void 0, !0)) || !0 !== t && !s.call(a, r, void 0, i)); ) r++;
                    return r;
                }
                function c(e) {
                    var t = this.opts, i = this.el;
                    return !this.isRTL || "number" != typeof e || t.greedy && "" === t.placeholder || !i || (e = this._valueGet().length - e) < 0 && (e = 0), 
                    e;
                }
            },
            4713: function(e, t, i) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.determineTestTemplate = c, t.getDecisionTaker = o, t.getMaskTemplate = function(e, t, i, n, a) {
                    var r = this, o = this.opts, u = this.maskset, f = o.greedy;
                    a && o.greedy && (o.greedy = !1, r.maskset.tests = {});
                    t = t || 0;
                    var p, h, v, m, g = [], y = 0;
                    do {
                        if (!0 === e && u.validPositions[y]) h = (v = a && u.validPositions[y].match.optionality && void 0 === u.validPositions[y + 1] && (!0 === u.validPositions[y].generatedInput || u.validPositions[y].input == o.skipOptionalPartCharacter && y > 0) ? c.call(r, y, d.call(r, y, p, y - 1)) : u.validPositions[y]).match, 
                        p = v.locator.slice(), g.push(!0 === i ? v.input : !1 === i ? h.nativeDef : s.call(r, y, h)); else {
                            h = (v = l.call(r, y, p, y - 1)).match, p = v.locator.slice();
                            var k = !0 !== n && (!1 !== o.jitMasking ? o.jitMasking : h.jit);
                            (m = (m && h.static && h.def !== o.groupSeparator && null === h.fn || u.validPositions[y - 1] && h.static && h.def !== o.groupSeparator && null === h.fn) && u.tests[y]) || !1 === k || void 0 === k || "number" == typeof k && isFinite(k) && k > y ? g.push(!1 === i ? h.nativeDef : s.call(r, g.length, h)) : m = !1;
                        }
                        y++;
                    } while (!0 !== h.static || "" !== h.def || t > y);
                    "" === g[g.length - 1] && g.pop();
                    !1 === i && void 0 !== u.maskLength || (u.maskLength = y - 1);
                    return o.greedy = f, g;
                }, t.getPlaceholder = s, t.getTest = u, t.getTestTemplate = l, t.getTests = d, t.isSubsetOf = f;
                var n, a = (n = i(2394)) && n.__esModule ? n : {
                    default: n
                };
                function r(e, t) {
                    var i = (null != e.alternation ? e.mloc[o(e)] : e.locator).join("");
                    if ("" !== i) for (;i.length < t; ) i += "0";
                    return i;
                }
                function o(e) {
                    var t = e.locator[e.alternation];
                    return "string" == typeof t && t.length > 0 && (t = t.split(",")[0]), void 0 !== t ? t.toString() : "";
                }
                function s(e, t, i) {
                    var n = this.opts, a = this.maskset;
                    if (void 0 !== (t = t || u.call(this, e).match).placeholder || !0 === i) return "function" == typeof t.placeholder ? t.placeholder(n) : t.placeholder;
                    if (!0 === t.static) {
                        if (e > -1 && void 0 === a.validPositions[e]) {
                            var r, o = d.call(this, e), s = [];
                            if (o.length > 1 + ("" === o[o.length - 1].match.def ? 1 : 0)) for (var l = 0; l < o.length; l++) if ("" !== o[l].match.def && !0 !== o[l].match.optionality && !0 !== o[l].match.optionalQuantifier && (!0 === o[l].match.static || void 0 === r || !1 !== o[l].match.fn.test(r.match.def, a, e, !0, n)) && (s.push(o[l]), 
                            !0 === o[l].match.static && (r = o[l]), s.length > 1 && /[0-9a-bA-Z]/.test(s[0].match.def))) return n.placeholder.charAt(e % n.placeholder.length);
                        }
                        return t.def;
                    }
                    return n.placeholder.charAt(e % n.placeholder.length);
                }
                function l(e, t, i) {
                    return this.maskset.validPositions[e] || c.call(this, e, d.call(this, e, t ? t.slice() : t, i));
                }
                function c(e, t) {
                    var i = this.opts, n = 0, a = function(e, t) {
                        var i = 0, n = !1;
                        t.forEach((function(e) {
                            e.match.optionality && (0 !== i && i !== e.match.optionality && (n = !0), (0 === i || i > e.match.optionality) && (i = e.match.optionality));
                        })), i && (0 == e || 1 == t.length ? i = 0 : n || (i = 0));
                        return i;
                    }(e, t);
                    e = e > 0 ? e - 1 : 0;
                    var o, s, l, c = r(u.call(this, e));
                    i.greedy && t.length > 1 && "" === t[t.length - 1].match.def && (n = 1);
                    for (var f = 0; f < t.length - n; f++) {
                        var d = t[f];
                        o = r(d, c.length);
                        var p = Math.abs(o - c);
                        (void 0 === s || "" !== o && p < s || l && !i.greedy && l.match.optionality && l.match.optionality - a > 0 && "master" === l.match.newBlockMarker && (!d.match.optionality || d.match.optionality - a < 1 || !d.match.newBlockMarker) || l && !i.greedy && l.match.optionalQuantifier && !d.match.optionalQuantifier) && (s = p, 
                        l = d);
                    }
                    return l;
                }
                function u(e, t) {
                    var i = this.maskset;
                    return i.validPositions[e] ? i.validPositions[e] : (t || d.call(this, e))[0];
                }
                function f(e, t, i) {
                    function n(e) {
                        for (var t, i = [], n = -1, a = 0, r = e.length; a < r; a++) if ("-" === e.charAt(a)) for (t = e.charCodeAt(a + 1); ++n < t; ) i.push(String.fromCharCode(n)); else n = e.charCodeAt(a), 
                        i.push(e.charAt(a));
                        return i.join("");
                    }
                    return e.match.def === t.match.nativeDef || !(!(i.regex || e.match.fn instanceof RegExp && t.match.fn instanceof RegExp) || !0 === e.match.static || !0 === t.match.static) && -1 !== n(t.match.fn.toString().replace(/[[\]/]/g, "")).indexOf(n(e.match.fn.toString().replace(/[[\]/]/g, "")));
                }
                function d(e, t, i) {
                    var n, r, o = this, s = this.dependencyLib, l = this.maskset, u = this.opts, d = this.el, p = l.maskToken, h = t ? i : 0, v = t ? t.slice() : [ 0 ], m = [], g = !1, y = t ? t.join("") : "";
                    function k(t, i, r, s) {
                        function c(r, s, p) {
                            function v(e, t) {
                                var i = 0 === t.matches.indexOf(e);
                                return i || t.matches.every((function(n, a) {
                                    return !0 === n.isQuantifier ? i = v(e, t.matches[a - 1]) : Object.prototype.hasOwnProperty.call(n, "matches") && (i = v(e, n)), 
                                    !i;
                                })), i;
                            }
                            function x(e, t, i) {
                                var n, a;
                                if ((l.tests[e] || l.validPositions[e]) && (l.tests[e] || [ l.validPositions[e] ]).every((function(e, r) {
                                    if (e.mloc[t]) return n = e, !1;
                                    var o = void 0 !== i ? i : e.alternation, s = void 0 !== e.locator[o] ? e.locator[o].toString().indexOf(t) : -1;
                                    return (void 0 === a || s < a) && -1 !== s && (n = e, a = s), !0;
                                })), n) {
                                    var r = n.locator[n.alternation];
                                    return (n.mloc[t] || n.mloc[r] || n.locator).slice((void 0 !== i ? i : n.alternation) + 1);
                                }
                                return void 0 !== i ? x(e, t) : void 0;
                            }
                            function P(e, t) {
                                var i = e.alternation, n = void 0 === t || i === t.alternation && -1 === e.locator[i].toString().indexOf(t.locator[i]);
                                if (!n && i > t.alternation) for (var a = t.alternation; a < i; a++) if (e.locator[a] !== t.locator[a]) {
                                    i = a, n = !0;
                                    break;
                                }
                                if (n) {
                                    e.mloc = e.mloc || {};
                                    var r = e.locator[i];
                                    if (void 0 !== r) {
                                        if ("string" == typeof r && (r = r.split(",")[0]), void 0 === e.mloc[r] && (e.mloc[r] = e.locator.slice()), 
                                        void 0 !== t) {
                                            for (var o in t.mloc) "string" == typeof o && (o = o.split(",")[0]), void 0 === e.mloc[o] && (e.mloc[o] = t.mloc[o]);
                                            e.locator[i] = Object.keys(e.mloc).join(",");
                                        }
                                        return !0;
                                    }
                                    e.alternation = void 0;
                                }
                                return !1;
                            }
                            function w(e, t) {
                                if (e.locator.length !== t.locator.length) return !1;
                                for (var i = e.alternation + 1; i < e.locator.length; i++) if (e.locator[i] !== t.locator[i]) return !1;
                                return !0;
                            }
                            if (h > e + u._maxTestPos) throw "Inputmask: There is probably an error in your mask definition or in the code. Create an issue on github with an example of the mask you are using. " + l.mask;
                            if (h === e && void 0 === r.matches) {
                                if (m.push({
                                    match: r,
                                    locator: s.reverse(),
                                    cd: y,
                                    mloc: {}
                                }), !r.optionality || void 0 !== p || !(u.definitions && u.definitions[r.nativeDef] && u.definitions[r.nativeDef].optional || a.default.prototype.definitions[r.nativeDef] && a.default.prototype.definitions[r.nativeDef].optional)) return !0;
                                g = !0, h = e;
                            } else if (void 0 !== r.matches) {
                                if (r.isGroup && p !== r) return function() {
                                    if (r = c(t.matches[t.matches.indexOf(r) + 1], s, p)) return !0;
                                }();
                                if (r.isOptional) return function() {
                                    var t = r, a = m.length;
                                    if (r = k(r, i, s, p), m.length > 0) {
                                        if (m.forEach((function(e, t) {
                                            t >= a && (e.match.optionality = e.match.optionality ? e.match.optionality + 1 : 1);
                                        })), n = m[m.length - 1].match, void 0 !== p || !v(n, t)) return r;
                                        g = !0, h = e;
                                    }
                                }();
                                if (r.isAlternator) return function() {
                                    o.hasAlternator = !0;
                                    var n, a, v, y = r, k = [], b = m.slice(), S = s.length, M = !1, _ = i.length > 0 ? i.shift() : -1;
                                    if (-1 === _ || "string" == typeof _) {
                                        var O, E = h, T = i.slice(), j = [];
                                        if ("string" == typeof _) j = _.split(","); else for (O = 0; O < y.matches.length; O++) j.push(O.toString());
                                        if (void 0 !== l.excludes[e]) {
                                            for (var A = j.slice(), D = 0, B = l.excludes[e].length; D < B; D++) {
                                                var C = l.excludes[e][D].toString().split(":");
                                                s.length == C[1] && j.splice(j.indexOf(C[0]), 1);
                                            }
                                            0 === j.length && (delete l.excludes[e], j = A);
                                        }
                                        (!0 === u.keepStatic || isFinite(parseInt(u.keepStatic)) && E >= u.keepStatic) && (j = j.slice(0, 1));
                                        for (var R = 0; R < j.length; R++) {
                                            O = parseInt(j[R]), m = [], i = "string" == typeof _ && x(h, O, S) || T.slice();
                                            var L = y.matches[O];
                                            if (L && c(L, [ O ].concat(s), p)) r = !0; else if (0 === R && (M = !0), L && L.matches && L.matches.length > y.matches[0].matches.length) break;
                                            n = m.slice(), h = E, m = [];
                                            for (var F = 0; F < n.length; F++) {
                                                var I = n[F], N = !1;
                                                I.match.jit = I.match.jit || M, I.alternation = I.alternation || S, P(I);
                                                for (var V = 0; V < k.length; V++) {
                                                    var G = k[V];
                                                    if ("string" != typeof _ || void 0 !== I.alternation && j.includes(I.locator[I.alternation].toString())) {
                                                        if (I.match.nativeDef === G.match.nativeDef) {
                                                            N = !0, P(G, I);
                                                            break;
                                                        }
                                                        if (f(I, G, u)) {
                                                            P(I, G) && (N = !0, k.splice(k.indexOf(G), 0, I));
                                                            break;
                                                        }
                                                        if (f(G, I, u)) {
                                                            P(G, I);
                                                            break;
                                                        }
                                                        if (v = G, !0 === (a = I).match.static && !0 !== v.match.static && v.match.fn.test(a.match.def, l, e, !1, u, !1)) {
                                                            w(I, G) || void 0 !== d.inputmask.userOptions.keepStatic ? P(I, G) && (N = !0, k.splice(k.indexOf(G), 0, I)) : u.keepStatic = !0;
                                                            break;
                                                        }
                                                    }
                                                }
                                                N || k.push(I);
                                            }
                                        }
                                        m = b.concat(k), h = e, g = m.length > 0, r = k.length > 0, i = T.slice();
                                    } else r = c(y.matches[_] || t.matches[_], [ _ ].concat(s), p);
                                    if (r) return !0;
                                }();
                                if (r.isQuantifier && p !== t.matches[t.matches.indexOf(r) - 1]) return function() {
                                    for (var a = r, o = !1, f = i.length > 0 ? i.shift() : 0; f < (isNaN(a.quantifier.max) ? f + 1 : a.quantifier.max) && h <= e; f++) {
                                        var d = t.matches[t.matches.indexOf(a) - 1];
                                        if (r = c(d, [ f ].concat(s), d)) {
                                            if (m.forEach((function(t, i) {
                                                (n = b(d, t.match) ? t.match : m[m.length - 1].match).optionalQuantifier = f >= a.quantifier.min, 
                                                n.jit = (f + 1) * (d.matches.indexOf(n) + 1) > a.quantifier.jit, n.optionalQuantifier && v(n, d) && (g = !0, 
                                                h = e, u.greedy && null == l.validPositions[e - 1] && f > a.quantifier.min && -1 != [ "*", "+" ].indexOf(a.quantifier.max) && (m.pop(), 
                                                y = void 0), o = !0, r = !1), !o && n.jit && (l.jitOffset[e] = d.matches.length - d.matches.indexOf(n));
                                            })), o) break;
                                            return !0;
                                        }
                                    }
                                }();
                                if (r = k(r, i, s, p)) return !0;
                            } else h++;
                        }
                        for (var p = i.length > 0 ? i.shift() : 0; p < t.matches.length; p++) if (!0 !== t.matches[p].isQuantifier) {
                            var v = c(t.matches[p], [ p ].concat(r), s);
                            if (v && h === e) return v;
                            if (h > e) break;
                        }
                    }
                    function b(e, t) {
                        var i = -1 != e.matches.indexOf(t);
                        return i || e.matches.forEach((function(e, n) {
                            void 0 === e.matches || i || (i = b(e, t));
                        })), i;
                    }
                    if (e > -1) {
                        if (void 0 === t) {
                            for (var x, P = e - 1; void 0 === (x = l.validPositions[P] || l.tests[P]) && P > -1; ) P--;
                            void 0 !== x && P > -1 && (v = function(e, t) {
                                var i, n = [];
                                return Array.isArray(t) || (t = [ t ]), t.length > 0 && (void 0 === t[0].alternation || !0 === u.keepStatic ? 0 === (n = c.call(o, e, t.slice()).locator.slice()).length && (n = t[0].locator.slice()) : t.forEach((function(e) {
                                    "" !== e.def && (0 === n.length ? (i = e.alternation, n = e.locator.slice()) : e.locator[i] && -1 === n[i].toString().indexOf(e.locator[i]) && (n[i] += "," + e.locator[i]));
                                }))), n;
                            }(P, x), y = v.join(""), h = P);
                        }
                        if (l.tests[e] && l.tests[e][0].cd === y) return l.tests[e];
                        for (var w = v.shift(); w < p.length; w++) {
                            if (k(p[w], v, [ w ]) && h === e || h > e) break;
                        }
                    }
                    return (0 === m.length || g) && m.push({
                        match: {
                            fn: null,
                            static: !0,
                            optionality: !1,
                            casing: null,
                            def: "",
                            placeholder: ""
                        },
                        locator: [],
                        mloc: {},
                        cd: y
                    }), void 0 !== t && l.tests[e] ? r = s.extend(!0, [], m) : (l.tests[e] = s.extend(!0, [], m), 
                    r = l.tests[e]), m.forEach((function(e) {
                        e.match.optionality = e.match.defOptionality || !1;
                    })), r;
                }
            },
            7215: function(e, t, i) {
                Object.defineProperty(t, "__esModule", {
                    value: !0
                }), t.alternate = s, t.checkAlternationMatch = function(e, t, i) {
                    for (var n, a = this.opts.greedy ? t : t.slice(0, 1), r = !1, o = void 0 !== i ? i.split(",") : [], s = 0; s < o.length; s++) -1 !== (n = e.indexOf(o[s])) && e.splice(n, 1);
                    for (var l = 0; l < e.length; l++) if (a.includes(e[l])) {
                        r = !0;
                        break;
                    }
                    return r;
                }, t.handleRemove = function(e, t, i, o, l) {
                    var c = this, u = this.maskset, f = this.opts;
                    if ((f.numericInput || c.isRTL) && (t === a.keys.Backspace ? t = a.keys.Delete : t === a.keys.Delete && (t = a.keys.Backspace), 
                    c.isRTL)) {
                        var d = i.end;
                        i.end = i.begin, i.begin = d;
                    }
                    var p, h = r.getLastValidPosition.call(c, void 0, !0);
                    i.end >= r.getBuffer.call(c).length && h >= i.end && (i.end = h + 1);
                    t === a.keys.Backspace ? i.end - i.begin < 1 && (i.begin = r.seekPrevious.call(c, i.begin)) : t === a.keys.Delete && i.begin === i.end && (i.end = r.isMask.call(c, i.end, !0, !0) ? i.end + 1 : r.seekNext.call(c, i.end) + 1);
                    if (!1 !== (p = v.call(c, i))) {
                        if (!0 !== o && !1 !== f.keepStatic || null !== f.regex && -1 !== n.getTest.call(c, i.begin).match.def.indexOf("|")) {
                            var m = s.call(c, !0);
                            if (m) {
                                void 0 !== m.caret ? m.caret : m.pos ? r.seekNext.call(c, m.pos.begin ? m.pos.begin : m.pos) : r.getLastValidPosition.call(c, -1, !0);
                            }
                        }
                        !0 !== o && (u.p = t === a.keys.Delete ? i.begin + p : i.begin, u.p = r.determineNewCaretPosition.call(c, {
                            begin: u.p,
                            end: u.p
                        }, !1, !1 === f.insertMode && t === a.keys.Backspace ? "none" : void 0).begin);
                    }
                }, t.isComplete = c, t.isSelection = u, t.isValid = f, t.refreshFromBuffer = p, 
                t.revalidateMask = v;
                var n = i(4713), a = i(2839), r = i(8711), o = i(6030);
                function s(e, t, i, a, o, l) {
                    var c, u, d, p, h, v, m, g, y, k, b, x = this, P = this.dependencyLib, w = this.opts, S = x.maskset, M = P.extend(!0, [], S.validPositions), _ = P.extend(!0, {}, S.tests), O = !1, E = !1, T = void 0 !== o ? o : r.getLastValidPosition.call(x);
                    if (l && (k = l.begin, b = l.end, l.begin > l.end && (k = l.end, b = l.begin)), 
                    -1 === T && void 0 === o) c = 0, u = (p = n.getTest.call(x, c)).alternation; else for (;T >= 0; T--) if ((d = S.validPositions[T]) && void 0 !== d.alternation) {
                        if (T <= (e || 0) && p && p.locator[d.alternation] !== d.locator[d.alternation]) break;
                        c = T, u = S.validPositions[c].alternation, p = d;
                    }
                    if (void 0 !== u) {
                        m = parseInt(c), S.excludes[m] = S.excludes[m] || [], !0 !== e && S.excludes[m].push((0, n.getDecisionTaker)(p) + ":" + p.alternation);
                        var j = [], A = -1;
                        for (h = m; h < r.getLastValidPosition.call(x, void 0, !0) + 1; h++) -1 === A && e <= h && void 0 !== t && (j.push(t), 
                        A = j.length - 1), (v = S.validPositions[h]) && !0 !== v.generatedInput && (void 0 === l || h < k || h >= b) && j.push(v.input), 
                        delete S.validPositions[h];
                        for (-1 === A && void 0 !== t && (j.push(t), A = j.length - 1); void 0 !== S.excludes[m] && S.excludes[m].length < 10; ) {
                            for (S.tests = {}, r.resetMaskSet.call(x, !0), O = !0, h = 0; h < j.length && (g = O.caret || r.getLastValidPosition.call(x, void 0, !0) + 1, 
                            y = j[h], O = f.call(x, g, y, !1, a, !0)); h++) h === A && (E = O), 1 == e && O && (E = {
                                caretPos: h
                            });
                            if (O) break;
                            if (r.resetMaskSet.call(x), p = n.getTest.call(x, m), S.validPositions = P.extend(!0, [], M), 
                            S.tests = P.extend(!0, {}, _), !S.excludes[m]) {
                                E = s.call(x, e, t, i, a, m - 1, l);
                                break;
                            }
                            var D = (0, n.getDecisionTaker)(p);
                            if (-1 !== S.excludes[m].indexOf(D + ":" + p.alternation)) {
                                E = s.call(x, e, t, i, a, m - 1, l);
                                break;
                            }
                            for (S.excludes[m].push(D + ":" + p.alternation), h = m; h < r.getLastValidPosition.call(x, void 0, !0) + 1; h++) delete S.validPositions[h];
                        }
                    }
                    return E && !1 === w.keepStatic || delete S.excludes[m], E;
                }
                function l(e, t, i) {
                    var n = this.opts, r = this.maskset;
                    switch (n.casing || t.casing) {
                      case "upper":
                        e = e.toUpperCase();
                        break;

                      case "lower":
                        e = e.toLowerCase();
                        break;

                      case "title":
                        var o = r.validPositions[i - 1];
                        e = 0 === i || o && o.input === String.fromCharCode(a.keyCode.Space) ? e.toUpperCase() : e.toLowerCase();
                        break;

                      default:
                        if ("function" == typeof n.casing) {
                            var s = Array.prototype.slice.call(arguments);
                            s.push(r.validPositions), e = n.casing.apply(this, s);
                        }
                    }
                    return e;
                }
                function c(e) {
                    var t = this, i = this.opts, a = this.maskset;
                    if ("function" == typeof i.isComplete) return i.isComplete(e, i);
                    if ("*" !== i.repeat) {
                        var o = !1, s = r.determineLastRequiredPosition.call(t, !0), l = r.seekPrevious.call(t, s.l);
                        if (void 0 === s.def || s.def.newBlockMarker || s.def.optionality || s.def.optionalQuantifier) {
                            o = !0;
                            for (var c = 0; c <= l; c++) {
                                var u = n.getTestTemplate.call(t, c).match;
                                if (!0 !== u.static && void 0 === a.validPositions[c] && !0 !== u.optionality && !0 !== u.optionalQuantifier || !0 === u.static && e[c] !== n.getPlaceholder.call(t, c, u)) {
                                    o = !1;
                                    break;
                                }
                            }
                        }
                        return o;
                    }
                }
                function u(e) {
                    var t = this.opts.insertMode ? 0 : 1;
                    return this.isRTL ? e.begin - e.end > t : e.end - e.begin > t;
                }
                function f(e, t, i, a, o, d, m) {
                    var g = this, y = this.dependencyLib, k = this.opts, b = g.maskset;
                    i = !0 === i;
                    var x = e;
                    function P(e) {
                        if (void 0 !== e) {
                            if (void 0 !== e.remove && (Array.isArray(e.remove) || (e.remove = [ e.remove ]), 
                            e.remove.sort((function(e, t) {
                                return g.isRTL ? e.pos - t.pos : t.pos - e.pos;
                            })).forEach((function(e) {
                                v.call(g, {
                                    begin: e,
                                    end: e + 1
                                });
                            })), e.remove = void 0), void 0 !== e.insert && (Array.isArray(e.insert) || (e.insert = [ e.insert ]), 
                            e.insert.sort((function(e, t) {
                                return g.isRTL ? t.pos - e.pos : e.pos - t.pos;
                            })).forEach((function(e) {
                                "" !== e.c && f.call(g, e.pos, e.c, void 0 === e.strict || e.strict, void 0 !== e.fromIsValid ? e.fromIsValid : a);
                            })), e.insert = void 0), e.refreshFromBuffer && e.buffer) {
                                var t = e.refreshFromBuffer;
                                p.call(g, !0 === t ? t : t.start, t.end, e.buffer), e.refreshFromBuffer = void 0;
                            }
                            void 0 !== e.rewritePosition && (x = e.rewritePosition, e = !0);
                        }
                        return e;
                    }
                    function w(t, i, o) {
                        var s = !1;
                        return n.getTests.call(g, t).every((function(c, f) {
                            var d = c.match;
                            if (r.getBuffer.call(g, !0), !1 !== (s = (!d.jit || void 0 !== b.validPositions[r.seekPrevious.call(g, t)]) && (null != d.fn ? d.fn.test(i, b, t, o, k, u.call(g, e)) : (i === d.def || i === k.skipOptionalPartCharacter) && "" !== d.def && {
                                c: n.getPlaceholder.call(g, t, d, !0) || d.def,
                                pos: t
                            }))) {
                                var p = void 0 !== s.c ? s.c : i, h = t;
                                return p = p === k.skipOptionalPartCharacter && !0 === d.static ? n.getPlaceholder.call(g, t, d, !0) || d.def : p, 
                                !0 !== (s = P(s)) && void 0 !== s.pos && s.pos !== t && (h = s.pos), !0 !== s && void 0 === s.pos && void 0 === s.c ? !1 : (!1 === v.call(g, e, y.extend({}, c, {
                                    input: l.call(g, p, d, h)
                                }), a, h) && (s = !1), !1);
                            }
                            return !0;
                        })), s;
                    }
                    void 0 !== e.begin && (x = g.isRTL ? e.end : e.begin);
                    var S = !0, M = y.extend(!0, {}, b.validPositions);
                    if (!1 === k.keepStatic && void 0 !== b.excludes[x] && !0 !== o && !0 !== a) for (var _ = x; _ < (g.isRTL ? e.begin : e.end); _++) void 0 !== b.excludes[_] && (b.excludes[_] = void 0, 
                    delete b.tests[_]);
                    if ("function" == typeof k.preValidation && !0 !== a && !0 !== d && (S = P(S = k.preValidation.call(g, r.getBuffer.call(g), x, t, u.call(g, e), k, b, e, i || o))), 
                    !0 === S) {
                        if (S = w(x, t, i), (!i || !0 === a) && !1 === S && !0 !== d) {
                            var O = b.validPositions[x];
                            if (!O || !0 !== O.match.static || O.match.def !== t && t !== k.skipOptionalPartCharacter) {
                                if (k.insertMode || void 0 === b.validPositions[r.seekNext.call(g, x)] || e.end > x) {
                                    var E = !1;
                                    if (b.jitOffset[x] && void 0 === b.validPositions[r.seekNext.call(g, x)] && !1 !== (S = f.call(g, x + b.jitOffset[x], t, !0, !0)) && (!0 !== o && (S.caret = x), 
                                    E = !0), e.end > x && (b.validPositions[x] = void 0), !E && !r.isMask.call(g, x, k.keepStatic && 0 === x)) for (var T = x + 1, j = r.seekNext.call(g, x, !1, 0 !== x); T <= j; T++) if (!1 !== (S = w(T, t, i))) {
                                        S = h.call(g, x, void 0 !== S.pos ? S.pos : T) || S, x = T;
                                        break;
                                    }
                                }
                            } else S = {
                                caret: r.seekNext.call(g, x)
                            };
                        }
                        g.hasAlternator && !0 !== o && !i && (!1 === S && k.keepStatic && (c.call(g, r.getBuffer.call(g)) || 0 === x) ? S = s.call(g, x, t, i, a, void 0, e) : (u.call(g, e) && b.tests[x] && b.tests[x].length > 1 && k.keepStatic || 1 == S && !0 !== k.numericInput && b.tests[x] && b.tests[x].length > 1 && r.getLastValidPosition.call(g, void 0, !0) > x) && (S = s.call(g, !0))), 
                        !0 === S && (S = {
                            pos: x
                        });
                    }
                    if ("function" == typeof k.postValidation && !0 !== a && !0 !== d) {
                        var A = k.postValidation.call(g, r.getBuffer.call(g, !0), void 0 !== e.begin ? g.isRTL ? e.end : e.begin : e, t, S, k, b, i, m);
                        void 0 !== A && (S = !0 === A ? S : A);
                    }
                    S && void 0 === S.pos && (S.pos = x), !1 === S || !0 === d ? (r.resetMaskSet.call(g, !0), 
                    b.validPositions = y.extend(!0, [], M)) : h.call(g, void 0, x, !0);
                    var D = P(S);
                    void 0 !== g.maxLength && (r.getBuffer.call(g).length > g.maxLength && !a && (r.resetMaskSet.call(g, !0), 
                    b.validPositions = y.extend(!0, [], M), D = !1));
                    return D;
                }
                function d(e, t, i) {
                    for (var a = this.maskset, r = !1, o = n.getTests.call(this, e), s = 0; s < o.length; s++) {
                        if (o[s].match && (o[s].match.nativeDef === t.match[i.shiftPositions ? "def" : "nativeDef"] && (!i.shiftPositions || !t.match.static) || o[s].match.nativeDef === t.match.nativeDef || i.regex && !o[s].match.static && o[s].match.fn.test(t.input, a, e, !1, i))) {
                            r = !0;
                            break;
                        }
                        if (o[s].match && o[s].match.def === t.match.nativeDef) {
                            r = void 0;
                            break;
                        }
                    }
                    return !1 === r && void 0 !== a.jitOffset[e] && (r = d.call(this, e + a.jitOffset[e], t, i)), 
                    r;
                }
                function p(e, t, i) {
                    var n, a, s = this, l = this.maskset, c = this.opts, u = this.dependencyLib, f = c.skipOptionalPartCharacter, d = s.isRTL ? i.slice().reverse() : i;
                    if (c.skipOptionalPartCharacter = "", !0 === e) r.resetMaskSet.call(s), l.tests = {}, 
                    e = 0, t = i.length, a = r.determineNewCaretPosition.call(s, {
                        begin: 0,
                        end: 0
                    }, !1).begin; else {
                        for (n = e; n < t; n++) delete l.validPositions[n];
                        a = e;
                    }
                    var p = new u.Event("keypress");
                    for (n = e; n < t; n++) {
                        p.key = d[n].toString(), s.ignorable = !1;
                        var h = o.EventHandlers.keypressEvent.call(s, p, !0, !1, !1, a);
                        !1 !== h && void 0 !== h && (a = h.forwardPosition);
                    }
                    c.skipOptionalPartCharacter = f;
                }
                function h(e, t, i) {
                    var a = this, o = this.maskset, s = this.dependencyLib;
                    if (void 0 === e) for (e = t - 1; e > 0 && !o.validPositions[e]; e--) ;
                    for (var l = e; l < t; l++) {
                        if (void 0 === o.validPositions[l] && !r.isMask.call(a, l, !1)) if (0 == l ? n.getTest.call(a, l) : o.validPositions[l - 1]) {
                            var c = n.getTests.call(a, l).slice();
                            "" === c[c.length - 1].match.def && c.pop();
                            var u, d = n.determineTestTemplate.call(a, l, c);
                            if (d && (!0 !== d.match.jit || "master" === d.match.newBlockMarker && (u = o.validPositions[l + 1]) && !0 === u.match.optionalQuantifier) && ((d = s.extend({}, d, {
                                input: n.getPlaceholder.call(a, l, d.match, !0) || d.match.def
                            })).generatedInput = !0, v.call(a, l, d, !0), !0 !== i)) {
                                var p = o.validPositions[t].input;
                                return o.validPositions[t] = void 0, f.call(a, t, p, !0, !0);
                            }
                        }
                    }
                }
                function v(e, t, i, a) {
                    var o = this, s = this.maskset, l = this.opts, c = this.dependencyLib;
                    function u(e, t, i) {
                        var n = t[e];
                        if (void 0 !== n && !0 === n.match.static && !0 !== n.match.optionality && (void 0 === t[0] || void 0 === t[0].alternation)) {
                            var a = i.begin <= e - 1 ? t[e - 1] && !0 === t[e - 1].match.static && t[e - 1] : t[e - 1], r = i.end > e + 1 ? t[e + 1] && !0 === t[e + 1].match.static && t[e + 1] : t[e + 1];
                            return a && r;
                        }
                        return !1;
                    }
                    var p = 0, h = void 0 !== e.begin ? e.begin : e, v = void 0 !== e.end ? e.end : e, m = !0;
                    if (e.begin > e.end && (h = e.end, v = e.begin), a = void 0 !== a ? a : h, void 0 === i && (h !== v || l.insertMode && void 0 !== s.validPositions[a] || void 0 === t || t.match.optionalQuantifier || t.match.optionality)) {
                        var g, y = c.extend(!0, {}, s.validPositions), k = r.getLastValidPosition.call(o, void 0, !0);
                        for (s.p = h, g = k; g >= h; g--) delete s.validPositions[g], void 0 === t && delete s.tests[g + 1];
                        var b, x, P = a, w = P;
                        for (t && (s.validPositions[a] = c.extend(!0, {}, t), w++, P++), g = t ? v : v - 1; g <= k; g++) {
                            if (void 0 !== (b = y[g]) && !0 !== b.generatedInput && (g >= v || g >= h && u(g, y, {
                                begin: h,
                                end: v
                            }))) {
                                for (;"" !== n.getTest.call(o, w).match.def; ) {
                                    if (!1 !== (x = d.call(o, w, b, l)) || "+" === b.match.def) {
                                        "+" === b.match.def && r.getBuffer.call(o, !0);
                                        var S = f.call(o, w, b.input, "+" !== b.match.def, !0);
                                        if (m = !1 !== S, P = (S.pos || w) + 1, !m && x) break;
                                    } else m = !1;
                                    if (m) {
                                        void 0 === t && b.match.static && g === e.begin && p++;
                                        break;
                                    }
                                    if (!m && r.getBuffer.call(o), w > s.maskLength) break;
                                    w++;
                                }
                                "" == n.getTest.call(o, w).match.def && (m = !1), w = P;
                            }
                            if (!m) break;
                        }
                        if (!m) return s.validPositions = c.extend(!0, [], y), r.resetMaskSet.call(o, !0), 
                        !1;
                    } else t && n.getTest.call(o, a).match.cd === t.match.cd && (s.validPositions[a] = c.extend(!0, {}, t));
                    return r.resetMaskSet.call(o, !0), p;
                }
            }
        }, t = {};
        function i(n) {
            var a = t[n];
            if (void 0 !== a) return a.exports;
            var r = t[n] = {
                exports: {}
            };
            return e[n](r, r.exports, i), r.exports;
        }
        var n = {};
        return function() {
            var e, t = n;
            Object.defineProperty(t, "__esModule", {
                value: !0
            }), t.default = void 0, i(7149), i(3194), i(9302), i(4013), i(3851), i(219), i(207), 
            i(5296);
            var a = ((e = i(2394)) && e.__esModule ? e : {
                default: e
            }).default;
            t.default = a;
        }(), n;
    }();
}));
});

const Inputmask = /*@__PURE__*/getDefaultExportFromCjs(inputmask);

const k2btMaskedInputCss = ":host{display:block}";

const K2btMaskedInput$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.inputEvent = createEvent(this, "input", 7);
    this.changeEvent = createEvent(this, "change", 7);
    this.enabled = true;
    this.uppercase = false;
    this.nativeInput = null;
    this.SYMBOL_OPTIONAL_DIGIT = 'Z';
    this.SYMBOL_OPTIONAL_LETTER = 'B';
    this.SYMBOL_DIGIT = '9';
    this.SYMBOL_LETTER = 'A';
    this.SYMBOL_CHARACTER = 'X';
    this.SYMBOL_ESCAPE = '\\';
    this.SYMBOL_MASK_SEPARATOR = ';';
    this.LANGUAGE_SYMBOLS = [
      this.SYMBOL_OPTIONAL_DIGIT,
      this.SYMBOL_OPTIONAL_LETTER,
      this.SYMBOL_DIGIT,
      this.SYMBOL_LETTER,
      this.SYMBOL_CHARACTER,
      this.SYMBOL_ESCAPE,
      this.SYMBOL_MASK_SEPARATOR,
    ];
    this.INPUTMASK_SPECIAL_CHARS = 'a9X[]{}()|\\';
    this.changeTimeout = null;
  }
  async getUnformattedValue() {
    var _a, _b, _c;
    // @ts-ignore
    return (_c = (_b = (_a = this.nativeInput) === null || _a === void 0 ? void 0 : _a.inputmask) === null || _b === void 0 ? void 0 : _b.unmaskedvalue()) !== null && _c !== void 0 ? _c : '';
  }
  async getFormattedValue() {
    var _a, _b;
    // @ts-ignore
    return (_b = (_a = this.nativeInput) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : '';
  }
  componentDidLoad() {
    if (this.mask) {
      this.getInputMask().mask(this.nativeInput);
    }
  }
  getInputMask() {
    return Inputmask(this.getTransformedMask(this.mask), {
      numericInput: this.numeric,
      keepStatic: true,
      jitMasking: true,
      positionCaretOnTab: !this.numeric,
      casing: this.uppercase ? 'upper' : null,
      definitions: {
        X: {
          validator: '[A-Za-z0-9]',
        },
      },
    });
  }
  getTransformedMask(mask) {
    mask = this.reorderOptionalCharacters(mask);
    var pos = 0; // start processing at beggining
    var result = [];
    while (pos < mask.length) {
      var currentMaskBuffer = '';
      var startNewMask = false;
      // if input is numeric, check if trailing zeroes exist
      if (this.numeric && mask[pos] == this.SYMBOL_OPTIONAL_DIGIT) {
        var closing = ']';
        currentMaskBuffer += '[9';
        pos++;
        var exit_numeric_preamble = false;
        while (pos < mask.length && !exit_numeric_preamble) {
          switch (mask[pos]) {
            case this.SYMBOL_OPTIONAL_DIGIT:
              currentMaskBuffer += '[9';
              closing += ']';
              pos++;
              break;
            case this.SYMBOL_CHARACTER:
            case this.SYMBOL_DIGIT:
            case this.SYMBOL_LETTER:
            case this.SYMBOL_MASK_SEPARATOR:
            case this.SYMBOL_OPTIONAL_LETTER:
              exit_numeric_preamble = true;
              break;
            case this.SYMBOL_ESCAPE:
              pos++; // Move over to next character and treat it as a literal character (default case)
            default:
              currentMaskBuffer = '[' + currentMaskBuffer + closing + this.escape_char(mask[pos]) + ']';
              closing = '';
              pos++;
              break;
          }
        }
        currentMaskBuffer += closing;
      }
      var escaping = false;
      while (pos < mask.length && !startNewMask) {
        if (!escaping) {
          switch (mask[pos]) {
            case this.SYMBOL_MASK_SEPARATOR:
              startNewMask = true;
              break;
            case this.SYMBOL_DIGIT:
            case this.SYMBOL_CHARACTER:
              currentMaskBuffer += mask[pos];
              break;
            case this.SYMBOL_LETTER:
              currentMaskBuffer += 'a';
              break;
            case this.SYMBOL_OPTIONAL_DIGIT:
              currentMaskBuffer += '[9]';
              break;
            case this.SYMBOL_OPTIONAL_LETTER:
              currentMaskBuffer += '[a]';
              break;
            case this.SYMBOL_ESCAPE:
              escaping = true;
            default:
              currentMaskBuffer += this.escape_char(mask[pos]);
          }
        }
        else {
          escaping = false;
          currentMaskBuffer += this.escape_char(mask[pos]);
        }
        pos++;
      }
      result.push(currentMaskBuffer);
    }
    return result;
  }
  reorderOptionalCharacters(mask) {
    var maskArray = [...mask];
    for (var i = 1; i < mask.length; i++) {
      var j = i;
      while ((maskArray[j - 1] == this.SYMBOL_OPTIONAL_LETTER && maskArray[j] == this.SYMBOL_LETTER) ||
        (maskArray[j - 1] == this.SYMBOL_OPTIONAL_DIGIT && maskArray[j] == this.SYMBOL_DIGIT)) {
        var temp = maskArray[j];
        maskArray[j] = maskArray[j - 1];
        maskArray[j - 1] = temp;
        j--;
      }
    }
    return maskArray.join('');
  }
  escape_char(c) {
    if (this.INPUTMASK_SPECIAL_CHARS.indexOf(c) != -1)
      return '\\' + c;
    else
      return c;
  }
  onInput() {
    clearTimeout(this.changeTimeout);
    var component = this;
    this.changeTimeout = setTimeout(function () {
      component.inputEvent.emit({ value: component.nativeInput.value });
    }, 300);
  }
  render() {
    var styleObj = {};
    if (!this.enabled) {
      styleObj.display = 'none';
    }
    if (this.numeric) {
      styleObj.textAlign = 'right';
    }
    var classString = this.inputclass;
    var readonlyClassString = this.readonlyclass + ' ' + this.inputclass;
    return (h("div", null, h("p", { class: "form-control-static", style: this.enabled ? { display: 'none' } : {} }, h("span", { class: readonlyClassString, "data-gx-readonly": "" }, !this.enabled ? this.getInputMask().format(this.value) : '')), h("input", { type: "text", inputmode: "text", class: 'form-control ' + classString, value: this.value, ref: el => (this.nativeInput = el), onInput: () => this.onInput(), onChange: () => this.changeEvent.emit(), style: styleObj })));
  }
  static get style() { return k2btMaskedInputCss; }
};

class MultiLevelMenu {
}
MultiLevelMenu.HINT_GROUP = 'group';
MultiLevelMenu.HINT_COLLAPSIBLE = 'collapsible';
MultiLevelMenu.HINT_GROUPWITHOUTTITLE = 'groupwithouttitle';

const k2btMegaMenuCss = ":host{display:block}";

const K2btMegaMenu$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.items = [];
    this.openSublevels = [];
    this.timeout = null;
  }
  onAtomicItemClick(i) {
    console.log('Item clicked: ' + i.Code);
  }
  onSubLevelClick(i) {
    if (this.openSublevels.indexOf(i.Code) != -1)
      this.openSublevels = this.openSublevels.filter(c => c != i.Code);
    else
      this.openSublevels = [...this.openSublevels, i.Code];
  }
  render() {
    return (h("div", { class: "K2BT_MegaMenuContainer" }, h("div", { class: "K2BT_MegaMenuHeader" }, this.items.map(i => this.renderFirstLevelItem(i))), h("div", { class: "K2BT_MegaMenuContents" }, this.getMegaMenuContents())));
  }
  getMegaMenuContents() {
    return this.items
      .filter(i => { var _a; return ((_a = i.Items) === null || _a === void 0 ? void 0 : _a.length) > 0; })
      .map(i => {
      var classes = 'K2BT_MegaMenuFirstLevelSublevelContents';
      if (this.activeSublevelCode === i.Code)
        classes += ' K2BT_MegaMenuFirstLevelSublevelContentsOpen';
      var subLevels = i.Items.filter(it => { var _a; return ((_a = it.Items) === null || _a === void 0 ? void 0 : _a.length) > 0; });
      return (h("div", { class: classes, onMouseEnter: () => this.onFirstLevelMouseEnter(i.Code), onMouseLeave: () => this.onFirstLevelMouseLeave() }, subLevels.length > 0
        ? i.Items.map(it => this.renderSecondLevelItem(it))
        : //@ts-ignore
          this.renderSecondLevelSublevel({
            Items: i.Items,
          })));
    });
  }
  renderFirstLevelItem(i) {
    var _a;
    if (((_a = i.Items) === null || _a === void 0 ? void 0 : _a.length) > 0)
      return this.renderFirstLevelSublevel(i);
    else
      return this.renderFirstLevelAtomicItem(i);
  }
  renderFirstLevelAtomicItem(i) {
    return (h("div", { class: "K2BT_MegaMenuFirstLevelItem", onClick: () => this.onAtomicItemClick(i) }, h("a", { href: i.Link }, this.getItemIconAndTitle(i))));
  }
  renderFirstLevelSublevel(i) {
    var classes = 'K2BT_MegaMenuFirstLevelSublevel';
    if (this.activeSublevelCode === i.Code)
      classes += ' K2BT_MegaMenuFirstLevelSublevelOpen';
    return (h("div", { class: classes, onClick: () => this.onSubLevelClick(i), onMouseEnter: () => this.onFirstLevelMouseEnter(i.Code), onMouseLeave: () => this.onFirstLevelMouseLeave() }, h("div", { class: "K2BT_MegaMenuFirstLevelSublevelHeader" }, this.getItemIconAndTitle(i))));
  }
  getItemIconAndTitle(i) {
    return (h("div", null, i.ImageClass ? h("span", { class: 'K2BT_MegaMenuIcon ' + i.ImageClass }) : '', h("span", { class: "K2BT_MegaMenuTitle" }, i.Title)));
  }
  onFirstLevelMouseEnter(code) {
    if (this.activeSublevelCode != code) {
      this.activeSublevelCode = code;
    }
    if (this.timeout) {
      clearTimeout(this.timeout);
      this.timeout = null;
    }
  }
  onFirstLevelMouseLeave() {
    this.timeout = setTimeout((() => {
      this.activeSublevelCode = '';
    }).bind(this), 500);
  }
  renderSecondLevelItem(i) {
    var _a;
    if (((_a = i.Items) === null || _a === void 0 ? void 0 : _a.length) > 0)
      return this.renderSecondLevelSublevel(i);
    else
      return this.renderSecondLevelAtomicItem(i);
  }
  renderSecondLevelAtomicItem(i) {
    return (h("div", { class: "K2BT_MegaMenuSecondLevelItem", onClick: () => this.onAtomicItemClick(i) }, h("a", { href: i.Link }, this.getItemIconAndTitle(i))));
  }
  renderSecondLevelSublevel(i) {
    if (i.ShowAs != MultiLevelMenu.HINT_GROUPWITHOUTTITLE) {
      return (h("div", { class: "K2BT_MegaMenuSecondLevelSublevel", onClick: () => this.onSubLevelClick(i) }, i.Title ? h("div", { class: "K2BT_MegaMenuSecondLevelSublevelHeader" }, this.getItemIconAndTitle(i)) : '', h("div", { class: "K2BT_MegaMenuSecondLevelSublevelContents" }, i.Items.map(it => this.renderInnerLevelItem(it)))));
    }
    else {
      return (h("div", { class: "K2BT_MegaMenuSecondLevelSublevel K2BT_MegaMenuSecondLevelSublevelNoTitle", onClick: () => this.onSubLevelClick(i) }, h("div", { class: "K2BT_MegaMenuSecondLevelSublevelContents" }, i.Items.map(it => this.renderSecondLevelSublevel(it)))));
    }
  }
  renderInnerLevelItem(i) {
    var _a;
    if (((_a = i.Items) === null || _a === void 0 ? void 0 : _a.length) > 0)
      return this.renderInnerLevelSublevel(i);
    else
      return this.renderInnerLevelAtomicItem(i);
  }
  renderInnerLevelAtomicItem(i) {
    return (h("div", { class: "K2BT_MegaMenuInnerLevelItem", onClick: () => this.onAtomicItemClick(i) }, h("a", { href: i.Link }, this.getItemIconAndTitle(i))));
  }
  renderInnerLevelSublevel(i) {
    const isCollapsible = i.ShowAs === MultiLevelMenu.HINT_COLLAPSIBLE || i.ShowAs === '';
    var classes = ['K2BT_MegaMenuInnerLevelSublevel'];
    if (isCollapsible)
      classes.push('K2BT_MegaMenuInnerLevelSublevelCollapsible');
    if (this.openSublevels.indexOf(i.Code) !== -1)
      classes.push('K2BT_MegaMenuInnerLevelSublevelOpen');
    return (h("div", { class: classes.join(' '), onClick: () => {
        if (isCollapsible)
          this.onSubLevelClick(i);
      } }, h("div", { class: "K2BT_MegaMenuInnerLevelSublevelHeader" }, this.getItemIconAndTitle(i)), h("div", { class: "K2BT_MegaMenuInnerLevelSublevelContents" }, i.Items.map(it => this.renderInnerLevelItem(it)))));
  }
  static get style() { return k2btMegaMenuCss; }
};

const k2btNumericInputCss = ":host{display:block}";

const K2btNumericInput$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.inputEvent = createEvent(this, "input", 7);
    this.changeEvent = createEvent(this, "change", 7);
    this.decimals = 2;
    this.integerdigits = 10;
    this.includethousandseparator = true;
    this.includesign = false;
    this.usermustenterdecimalseparator = true;
    this.decimalseparator = '.';
    this.thousandseparator = ',';
    this.valueprefix = '';
    this.zeropadding = false;
    this.inputclass = '';
    this.readonlyclass = '';
    this.editorPosition = 0;
    this.value = '0';
    this.changeTimeout = null;
  }
  onInput() {
    clearTimeout(this.changeTimeout);
    var component = this;
    this.changeTimeout = setTimeout(function () {
      component.inputEvent.emit({ value: component.inputControl.value });
    }, 300);
  }
  /*
  This method must determine if this keypress should be accepted or not.

  In some cases it should move the cursor or alter how the keypress should be processed.
  */
  handleKeydown(event) {
    if (event.key != 'Tab' && event.key != 'Shift' && this.inputControl.selectionStart == 0 && this.inputControl.selectionEnd == this.inputControl.value.length) {
      this.inputControl.value = this.getFormattedValue_impl('0');
      if (!this.usermustenterdecimalseparator) {
        this.setCursorPosition(this.inputControl.value.length);
      }
    }
    var preventDefault = false;
    const currentPosition = this.getCursorPosition();
    if (event.key == 'Delete' || event.key == 'Backspace') {
      preventDefault = this.processDelete(event, currentPosition);
    }
    else if (event.key == this.decimalseparator) {
      preventDefault = this.processDecimalSeparator();
    }
    else if (event.key == '-') {
      preventDefault = this.processMinusSign(currentPosition);
    }
    else if (K2btNumericInput$1.digits.includes(event.key)) {
      preventDefault = this.processDigitInsertion(event, currentPosition);
    }
    else if (event.key.length == 1 && !event.ctrlKey) {
      // ignore all other keys
      preventDefault = true;
    }
    if (preventDefault) {
      event.preventDefault();
      return false;
    }
  }
  processMinusSign(currentPosition) {
    var preventDefault = true; // Generally this key should not be processed
    const firstDigitPosition = Math.min(...K2btNumericInput$1.digits.map(d => this.inputControl.value.indexOf(d)).filter(i => i != -1));
    if (this.includesign && currentPosition == firstDigitPosition && this.inputControl.value[firstDigitPosition - 1] != '-') {
      // If the number should support sign, the cursor is in the position is where the sign should be added, and no minus sign is there, this keypress should be processed
      preventDefault = false;
    }
    return preventDefault;
  }
  processDecimalSeparator() {
    var preventDefault = false;
    if (this.inputControl.value.charAt(this.getCursorPosition()) == this.decimalseparator) {
      this.setCursorPosition(this.getCursorPosition() + 1);
    }
    var selectionBegin = Math.min(this.inputControl.selectionStart, this.inputControl.selectionEnd);
    var selectionFinish = Math.max(this.inputControl.selectionStart, this.inputControl.selectionEnd);
    if (this.decimals > 0 && selectionBegin == 0 && selectionFinish == this.inputControl.value.length) {
      this.setCursorPosition(this.inputControl.value.indexOf(this.decimalseparator) + 1);
    }
    if (this.inputControl.value.indexOf(this.decimalseparator) != -1 || this.decimals == 0) {
      preventDefault = true;
    }
    return preventDefault;
  }
  processDigitInsertion(event, currentPosition) {
    var decimalPosition = this.inputControl.value.indexOf(this.decimalseparator);
    if (event.key == '0') {
      // ignore non-significant zeroes
      const firstSignificantDigitPosition = Math.min(...K2btNumericInput$1.digits
        .filter(d => d != '0')
        .map(d => this.inputControl.value.indexOf(d))
        .filter(i => i != -1));
      if (firstSignificantDigitPosition != -1 &&
        !((decimalPosition != -1 && currentPosition > decimalPosition) || (firstSignificantDigitPosition != -1 && currentPosition > firstSignificantDigitPosition))) {
        return true; // prevent default
      }
    }
    var preventDefault = false;
    if (!this.usermustenterdecimalseparator && currentPosition == this.inputControl.value.length) {
      // This digit is inserted at the end of the number, and the configuration is set for ATM-style.
      // Because of this, the current digits should be left shifted and the new digit added at the end.
      // The decimal separator should stay in place.
      // The digit should be ignored if the amount of digits matches the definition for integer digits + decimals
      var newdigits = (this.inputControl.value.replace(/\D/g, '') + event.key).replace(/^0+/, '');
      if (newdigits.length <= this.decimals + this.integerdigits) {
        if (newdigits.length <= this.decimals) {
          this.inputControl.value = this.getFormattedValue_impl('0.' + '0'.repeat(this.decimals - newdigits.length) + newdigits);
        }
        else {
          this.inputControl.value = this.getFormattedValue_impl(newdigits.substring(0, newdigits.length - this.decimals) + '.' + newdigits.substring(newdigits.length - this.decimals));
        }
        preventDefault = true;
        this.onInputChange();
      }
      preventDefault = true;
    }
    else {
      if (this.decimals > 0 && decimalPosition != -1 && decimalPosition < currentPosition) {
        // The digit is inserted after the decimal sign.
        // Check that the current position is not after the last accepted decimal.
        if (this.decimals != currentPosition - decimalPosition - 1) {
          // If it's not after the last decimal, the "next" digit should be overwritten and the cursor should be moved to the next position.
          this.inputControl.value = this.inputControl.value.substring(0, this.getCursorPosition()) + event.key + this.inputControl.value.substring(this.getCursorPosition() + 1);
          this.setCursorPosition(currentPosition + 1);
        }
        preventDefault = true;
      }
      else {
        // The digit is not in the decimal part.
        var integerPart = this.inputControl.value;
        if (this.decimals > 0 && decimalPosition != -1) {
          integerPart = integerPart.substring(0, integerPart.indexOf(this.decimalseparator));
        }
        // Input is in integer part
        var currentDigits = integerPart.replace(/\D/g, '').replace(/^0+/, '').length;
        if (currentDigits >= this.integerdigits) {
          // If integer part is complete, discard this keypress
          preventDefault = true;
        }
        else {
          if (currentDigits == 0) {
            if (this.inputControl.value.indexOf(this.decimalseparator) != -1) {
              this.inputControl.value = this.getFormattedValue_impl(event.key + this.value.substring(this.value.indexOf(this.decimalseparator)));
            }
            else {
              this.inputControl.value = this.getFormattedValue_impl(event.key);
              this.setCursorPosition(this.inputControl.value.length);
            }
            if (this.inputControl.value.indexOf(this.decimalseparator) != -1) {
              this.setCursorPosition(this.inputControl.value.indexOf(this.decimalseparator));
            }
            else {
              this.setCursorPosition(this.inputControl.value.length);
            }
            preventDefault = true;
            this.onInputChange();
          }
        }
      }
    }
    return preventDefault;
  }
  processDelete(event, currentPosition) {
    var preventDefault = false;
    var selectionBegin = Math.min(this.inputControl.selectionStart, this.inputControl.selectionEnd);
    var selectionFinish = Math.max(this.inputControl.selectionStart, this.inputControl.selectionEnd);
    if (selectionBegin == 0 && selectionFinish == this.inputControl.value.length) {
      this.inputControl.value = this.getFormattedValue_impl('0');
      if (this.usermustenterdecimalseparator) {
        // If user must enter decimal separator, place cursor in the unit's place
        this.setCursorPosition(this.inputControl.value.indexOf(this.decimalseparator) == -1 ? this.inputControl.value.length : this.inputControl.value.indexOf(this.decimalseparator));
      }
      else {
        // Else, place it at the end
        this.setCursorPosition(this.inputControl.value.length);
      }
      preventDefault = true;
      this.onInputChange();
    }
    else if (selectionBegin != selectionFinish) {
      // The user has selected a range delete it
      var newValue = this.inputControl.value.substring(0, selectionBegin) + this.inputControl.value.substring(selectionFinish);
      newValue = newValue.replace(/\D/g, '');
      this.inputControl.value = this.getFormattedValue_impl(newValue.substring(0, newValue.length - this.decimals) + '.' + newValue.substring(newValue.length - this.decimals));
      preventDefault = true;
      // Adjust cursor position
      if (event.key == 'Backspace')
        this.setCursorPosition(selectionBegin);
      else
        this.setCursorPosition(selectionFinish);
      this.onInputChange();
    }
    else if (event.key == 'Backspace') {
      if (!this.usermustenterdecimalseparator && currentPosition == this.inputControl.value.length) {
        var newdigits = this.inputControl.value.replace(/\D/g, '').replace(/^0+/, '');
        newdigits = newdigits.substring(0, newdigits.length - 1);
        var decimals = newdigits.substring(newdigits.length - this.decimals);
        decimals = '0'.repeat(this.decimals - decimals.length) + decimals;
        this.value = newdigits.substring(0, newdigits.length - this.decimals) + '.' + decimals;
        preventDefault = true;
      }
      else {
        if (this.inputControl.value.charAt(this.getCursorPosition() - 1) == this.thousandseparator ||
          this.inputControl.value.charAt(this.getCursorPosition() - 1) == this.decimalseparator) {
          this.setCursorPosition(this.getCursorPosition() - 1);
          preventDefault = true;
        }
        else if (selectionBegin == 0 && this.inputControl.value.charAt(selectionFinish) == this.decimalseparator) {
          const currentPosition = this.getCursorPosition();
          this.inputControl.value = this.inputControl.value.substring(0, this.getCursorPosition() - 1) + this.inputControl.value.substring(this.getCursorPosition());
          this.setCursorPosition(currentPosition - 1);
          preventDefault = true;
          this.onInputChange();
        }
      }
    }
    else if (event.key == 'Delete') {
      if (this.inputControl.value.charAt(this.getCursorPosition()) == this.thousandseparator || this.inputControl.value.charAt(this.getCursorPosition()) == this.decimalseparator) {
        this.setCursorPosition(this.getCursorPosition() + 1);
        preventDefault = true;
      }
      else if (selectionBegin == 0 && this.inputControl.value.charAt(selectionFinish + 1) == this.decimalseparator) {
        this.inputControl.value = this.inputControl.value.substring(0, this.getCursorPosition() - 1) + this.inputControl.value.substring(this.getCursorPosition() + 1);
        this.setCursorPosition(currentPosition);
        preventDefault = true;
        this.onInputChange();
      }
    }
    return preventDefault;
  }
  escapeRegExp(string) {
    return string.replace(/[.*+\-?^${}()|[\]\\]/g, '\\$&'); // $& means the whole matched string
  }
  replaceAll(str, find, replace) {
    return str.replace(new RegExp(this.escapeRegExp(find), 'g'), replace);
  }
  onInputChange() {
    // the contents were changed, try to parse the value
    var originalValue = this.inputControl.value;
    var previousValue = this.value;
    this.editorPosition = this.getCursorPosition();
    this.value = this.inputControl.value.replace(new RegExp('[^\\d' + this.decimalseparator + '-]', 'g'), '').replace(/^0+/, '');
    if (this.value[0] == this.decimalseparator)
      this.value = '0' + this.value;
    if (Number.isNaN(this.value))
      this.value = '0';
    this.inputControl.value = this.getFormattedValue();
    var previousDigits = previousValue.indexOf(this.decimalseparator) == -1 ? previousValue.length : previousValue.indexOf(this.decimalseparator);
    var currentDigits = this.value.indexOf(this.decimalseparator) == -1 ? this.value.length : this.value.indexOf(this.decimalseparator);
    if (this.includethousandseparator && !this.zeropadding) {
      if (currentDigits > previousDigits && currentDigits % 3 == 1) {
        this.editorPosition = this.editorPosition + 1;
      }
      else if (currentDigits < previousDigits && currentDigits % 3 == 0) {
        if (originalValue.substring(0, this.editorPosition) != this.getFormattedValue().substring(0, this.editorPosition)) {
          this.editorPosition = this.editorPosition - 1;
        }
      }
    }
    else if (this.zeropadding) {
      this.editorPosition = this.inputControl.value.length - originalValue.length + this.editorPosition;
    }
  }
  setCursorPosition(val) {
    this.inputControl.selectionStart = val;
    this.inputControl.selectionEnd = val;
  }
  getCursorPosition() {
    if (!this.inputControl)
      return; // No (input) element found
    if ('selectionStart' in this.inputControl) {
      // Standard-compliant browsers
      return this.inputControl.selectionStart;
    }
    else {
      //@ts-ignore
      var sel = document.selection;
      if (sel) {
        // IE
        //@ts-ignore
        this.inputControl.focus();
        var sel = sel.createRange();
        var selLen = sel.createRange().text.length;
        //@ts-ignore
        sel.moveStart('character', -this.inputControl.value.length);
        return sel.text.length - selLen;
      }
    }
  }
  getFormattedValue() {
    return this.getFormattedValue_impl(this.value);
  }
  getFormattedValue_impl(value) {
    var result = '';
    if (this.valueprefix != '')
      result = this.valueprefix + ' ';
    if (value.startsWith('-'))
      result = result + '-';
    const decimalSeparatorPosition = value.indexOf(this.decimalseparator);
    // now include integer part
    var integerPart = (decimalSeparatorPosition != -1 ? value.substring(0, decimalSeparatorPosition) : value).replace('-', '');
    if (this.zeropadding)
      integerPart = '0'.repeat(this.integerdigits - integerPart.length) + integerPart;
    if (this.includethousandseparator) {
      var pendingdigits = integerPart;
      var withSeparator = '';
      while (pendingdigits.length > 3) {
        withSeparator = this.thousandseparator + pendingdigits.substring(pendingdigits.length - 3) + withSeparator;
        pendingdigits = pendingdigits.substring(0, pendingdigits.length - 3);
      }
      withSeparator = pendingdigits + withSeparator;
      result = result + withSeparator;
    }
    else {
      result = result + integerPart;
    }
    // now include decimal part
    if (this.decimals > 0) {
      var decimalValue = decimalSeparatorPosition != -1 ? value.substring(value.indexOf(this.decimalseparator) + 1) : '';
      result = result + this.decimalseparator + decimalValue.substring(0, this.decimals);
      if (this.decimals - decimalValue.length > 0)
        result = result + '0'.repeat(this.decimals - decimalValue.length);
    }
    return result;
  }
  render() {
    var styleObj = {};
    if (!this.enabled) {
      styleObj.display = 'none';
    }
    styleObj.textAlign = 'right';
    var classString = this.inputclass;
    var readonlyClassString = this.readonlyclass + ' ' + this.inputclass;
    return (h("div", null, h("p", { class: "form-control-static", style: this.enabled ? { display: 'none' } : {} }, h("span", { class: readonlyClassString, "data-gx-readonly": "" }, this.value)), h("input", { class: 'form-control ' + classString, type: "text", ref: c => (this.inputControl = c), style: styleObj, onInput: () => this.onInputChange(), onBlur: () => this.onInputChange(), onChange: () => this.changeEvent.emit() })));
  }
  componentDidRender() {
    this.inputControl.value = this.getFormattedValue();
  }
  componentDidUpdate() {
    if (this.editorPosition != -1) {
      this.setCursorPosition(this.editorPosition);
      this.editorPosition = -1;
    }
  }
  static get watchers() { return {
    "value": ["onInput"]
  }; }
  static get style() { return k2btNumericInputCss; }
};
K2btNumericInput$1.digits = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9'];

const k2btNumericSliderCss = ":host{display:block}.K2BT_NumericSliderContainer{max-width:300px;display:flex;flex-direction:row}.K2BT_NumericSliderControl{margin-right:8px}input[type='range']{-webkit-appearance:none;width:100%;background:transparent;}input[type='range']:focus{outline:none;}input[type='range']::-webkit-slider-thumb{border:none;-webkit-appearance:none;height:16px;width:16px;border-radius:8px;background:#019f0c;cursor:pointer;margin-top:-4px}input[type='range']::-webkit-slider-thumb:hover{background:#01890a}input[type='range']::-moz-range-thumb{border:none;height:16px;width:16px;border-radius:8px;background:#019f0c;cursor:pointer;box-sizing:border-box}input[type='range']::-moz-range-thumb:hover{background:#01890a}input[type='range']::-ms-thumb{border:none;height:16px;width:16px;border-radius:3px;background:#019f0c;cursor:pointer}input[type='range']::-ms-thumb:hover{background:#01890a}input[type='range']::-webkit-slider-runnable-track{width:100%;height:8.4px;cursor:pointer;background:#ebebeb;border-radius:1.3px}input[type='range']::-moz-range-track{width:100%;height:8.4px;cursor:pointer;background:#ebebeb;border-radius:1.3px}input[type='range']::-moz-range-progress{background:#019f0c32;height:8.4px}input[type='range']::-ms-track{width:100%;height:8.4px;cursor:pointer;background:transparent;border-color:transparent;border-width:16px 0;color:transparent}input[type='range']::-ms-fill-lower{background:#019f0c32;border-radius:2.6px}input[type='range']:focus::-ms-fill-lower{background:#019f0c32}input[type='range']::-ms-fill-upper{background:#ebebeb;border:0.2px solid #010101;border-radius:2.6px;box-shadow:1px 1px 1px #000000, 0px 0px 1px #0d0d0d}input[type='range']:focus::-ms-fill-upper{background:#ebebeb}";

const K2btNumericSlider$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.inputEvent = createEvent(this, "input", 7);
    this.changeEvent = createEvent(this, "change", 7);
    this.value = '';
    this.readonlyclass = '';
    this.enabled = true;
    this.changeTimeout = null;
  }
  onAuxiliaryInput() {
    clearTimeout(this.changeTimeout);
    var component = this;
    this.value = this.auxiliaryInput.value;
    this.changeTimeout = setTimeout(function () {
      component.inputEvent.emit({ value: component.nativeInput.value });
    }, 300);
  }
  onSliderInput() {
    clearTimeout(this.changeTimeout);
    var component = this;
    this.value = this.nativeInput.value;
    this.changeTimeout = setTimeout(function () {
      component.inputEvent.emit({ value: component.nativeInput.value });
    }, 300);
  }
  render() {
    var inputClasses = 'K2BT_NumericSliderControl ' + this.sliderclass;
    var auxiliaryInputClasses = 'K2BT_NumericSliderAuxiliaryInput form-control ' + this.numberclass;
    var readonlyClassString = this.readonlyclass;
    return (h(Host, null, h("p", { class: "form-control-static", style: this.enabled ? { display: 'none' } : {} }, h("span", { class: readonlyClassString, "data-gx-readonly": "" }, this.value)), h("div", { class: "K2BT_NumericSliderContainer", style: !this.enabled ? { display: 'none' } : {} }, h("input", { type: "range", min: this.min, max: this.max, step: this.step, class: inputClasses, value: this.value, ref: el => (this.nativeInput = el), onInput: () => this.onSliderInput(), onChange: () => this.changeEvent.emit() }), h("span", { style: this.numbervisible && this.numberreadonly ? { minWidth: this.max.toString().length + 3 + 'ch', textAlign: 'right' } : { display: 'none' }, class: readonlyClassString, "data-gx-readonly": "" }, this.value), h("input", { type: "text", inputmode: "numeric", class: auxiliaryInputClasses, value: this.value, ref: el => (this.auxiliaryInput = el), onInput: () => this.onAuxiliaryInput(), onChange: () => this.changeEvent.emit(), size: this.max.toString().length, style: this.numbervisible && !this.numberreadonly ? {} : { display: 'none' } }))));
  }
  static get style() { return k2btNumericSliderCss; }
};

const K2btPillViewer$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.pills = [];
  }
  render() {
    return (h("div", { class: "K2BT_PillViewer" }, this.pills.map(p => (h("div", { class: p.Class ? 'K2BT_Pill ' + p.Class : 'K2BT_Pill', title: p.Text }, h("div", { class: "K2BT_PillImageContainer" }, h("img", { class: "K2BT_PillImage", src: p.IconUrl })), h("span", { class: "K2BT_PillText" }, p.Text))))));
  }
};

const K2btSignaturePad$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.changeEvent = createEvent(this, "change", 7);
    this.width = 300;
    this.height = 200;
    this.backgroundimageurl = '';
    this.backgroundcolor = '#FFFFFF';
    this.showclearbutton = true;
    this.clearbuttoncaption = 'Clear';
    this.canvas = null;
    this.ctx = null;
    this.isWriting = false;
    this.setValueDebouncer = null;
  }
  render() {
    var _a, _b;
    var style = {
      backgroundImage: ((_a = this.backgroundimageurl) !== null && _a !== void 0 ? _a : '') !== '' ? 'url(' + this.backgroundimageurl + ')' : null,
      backgroundColor: ((_b = this.backgroundcolor) !== null && _b !== void 0 ? _b : '') !== '' ? this.backgroundcolor : null,
    };
    console.log(style);
    return (h(Host, null, h("div", { class: "K2BT_SignaturePadContainer", style: style }, h("canvas", { height: this.height, width: this.width, class: "K2BT_SignaturePad", ref: canvas => {
        this.canvas = canvas;
        this.ctx = canvas.getContext('2d');
      } })), this.showclearbutton ? (h("button", { class: "K2BT_SignaturePadClearButton", onClick: this.clearCanvas.bind(this) }, this.clearbuttoncaption)) : ('')));
  }
  componentDidRender() {
    this.canvas.addEventListener('pointerdown', this.handlePointerDown.bind(this), { passive: true });
    document.addEventListener('pointerup', this.handlePointerUp.bind(this), { passive: true });
    this.canvas.addEventListener('pointermove', this.handlePointerMove.bind(this), { passive: true });
  }
  clearCanvas() {
    this.ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);
  }
  handlePointerDown(event) {
    this.isWriting = true;
    this.ctx.beginPath();
    const [positionX, positionY] = this.getCursorPosition(event);
    this.ctx.moveTo(positionX, positionY);
  }
  handlePointerUp() {
    var wasWriting = this.isWriting;
    this.isWriting = false;
    if (wasWriting) {
      if (this.setValueDebouncer !== null) {
        clearTimeout(this.setValueDebouncer);
      }
      this.setValueDebouncer = setTimeout((() => {
        this.setValueDebouncer = null;
        this.changeEvent.emit(this.canvas.toDataURL());
      }).bind(this), 200);
    }
  }
  handlePointerMove(event) {
    if (!this.isWriting)
      return;
    const [positionX, positionY] = this.getCursorPosition(event);
    this.ctx.lineTo(positionX, positionY);
    this.ctx.stroke();
  }
  getCursorPosition(event) {
    var positionX = event.clientX - event.target.getBoundingClientRect().x;
    var positionY = event.clientY - event.target.getBoundingClientRect().y;
    return [positionX, positionY];
  }
};

const K2btSplitDatepicker$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.inputEvent = createEvent(this, "dateinput", 7);
    this.changeEvent = createEvent(this, "datechange", 7);
    this.value = null;
    this.readonlyclass = '';
    this.enabled = true;
    this.validationerrorcode = K2btSplitDatepicker$1.VALIDATION_ERROR_NOERROR;
    this.yearControl = null;
    this.monthControl = null;
    this.dayControl = null;
    this.minuteControl = null;
    this.hourControl = null;
  }
  connectedCallback() {
    if (this.value) {
      this.selectedYear = this.value.getFullYear();
      this.selectedMonth = this.value.getMonth() + 1;
      this.selectedDay = this.value.getDate();
      this.selectedHour = this.value.getHours();
      this.selectedMinute = this.value.getMinutes();
    }
    else {
      this.selectedYear = 0;
      this.selectedMonth = 0;
      this.selectedDay = 0;
      this.selectedHour = -1;
      this.selectedMinute = -1;
      this.validationerrorcode = K2btSplitDatepicker$1.VALIDATION_ERROR_EMPTYYEAR;
    }
    this.validateDate();
  }
  render() {
    var readonlyClassString = this.readonlyclass;
    return (h(Host, null, h("p", { class: "form-control-static", style: this.enabled ? { display: 'none' } : {} }, h("span", { class: readonlyClassString, "data-gx-readonly": "" }, this.getReadonlyContents())), h("div", { style: !this.enabled ? { display: 'none' } : {} }, this.renderDatepickerContents())));
  }
  getReadonlyContents() {
    switch (this.fieldorder) {
      case K2btSplitDatepicker$1.ORDER_DMY:
        return `${this.normalizeToAmountOfDigits(this.selectedDay, 2)}/${this.getSelectedMonthString()}/${this.normalizeToAmountOfDigits(this.selectedYear, 4)} ${this.normalizeToAmountOfDigits(this.selectedHour, 2)}:${this.normalizeToAmountOfDigits(this.selectedMinute, 2)}`;
      case K2btSplitDatepicker$1.ORDER_YMD:
        return `${this.normalizeToAmountOfDigits(this.selectedYear, 4)}/${this.getSelectedMonthString()}/${this.normalizeToAmountOfDigits(this.selectedDay, 2)} ${this.normalizeToAmountOfDigits(this.selectedHour, 2)}:${this.normalizeToAmountOfDigits(this.selectedMinute, 2)}`;
      case K2btSplitDatepicker$1.ORDER_MDY:
        return `${this.getSelectedMonthString()}/${this.normalizeToAmountOfDigits(this.selectedDay, 2)}/${this.normalizeToAmountOfDigits(this.selectedYear, 4)} ${this.normalizeToAmountOfDigits(this.selectedHour, 2)}:${this.normalizeToAmountOfDigits(this.selectedMinute, 2)}`;
    }
  }
  getSelectedMonthString() {
    var month = this.selectedMonth;
    if (month === 0)
      return this.normalizeToAmountOfDigits(0, 2);
    switch (this.showmonthas) {
      case K2btSplitDatepicker$1.SHOWMONTHAS_NUMBER:
        return this.normalizeToAmountOfDigits(month, 2);
      case K2btSplitDatepicker$1.SHOWMONTHAS_NAME:
        //@ts-ignore
        return k2btools.getMonthName(month - 1);
      case K2btSplitDatepicker$1.SHOWMONTHAS_SHORTNAME:
        //@ts-ignore
        return k2btools.getMonthShortName(month - 1);
    }
  }
  normalizeToAmountOfDigits(value, digits) {
    return '0'.repeat(digits - value.toString().length) + value.toString();
  }
  renderDatepickerContents() {
    switch (this.fieldorder) {
      case K2btSplitDatepicker$1.ORDER_DMY:
        return (h("div", { class: "K2BT_SplitDatepicker" }, this.renderDayField(), this.renderMonthField(), this.renderYearField(), this.renderTimeFields()));
      case K2btSplitDatepicker$1.ORDER_YMD:
        return (h("div", { class: "K2BT_SplitDatepicker" }, this.renderYearField(), this.renderMonthField(), this.renderDayField(), this.renderTimeFields()));
      case K2btSplitDatepicker$1.ORDER_MDY:
        return (h("div", { class: "K2BT_SplitDatepicker" }, this.renderMonthField(), this.renderDayField(), this.renderYearField(), this.renderTimeFields()));
    }
  }
  renderTimeFields() {
    return (h("div", { class: "K2BT_SplitDatepickerTime" }, this.renderHourField(), this.renderMinuteField()));
  }
  renderYearField() {
    if (this.fieldcontroltype == K2btSplitDatepicker$1.FIELDCONTROLTYPE_COMBO) {
      return this.renderYearFieldAsCombo();
    }
    else {
      if (this.minimumdate) {
        return this.renderYearFieldAsSuggest();
      }
      else {
        return this.renderYearFieldAsInputWithNoSuggest();
      }
    }
  }
  renderYearFieldAsInputWithNoSuggest() {
    return (h("input", { type: "text", class: "form-control Attribute_Trn", size: Math.max(4, this.yearinvitemessage.length), placeholder: this.yearinvitemessage, ref: c => (this.yearControl = c), value: this.selectedYear != 0 ? this.selectedYear.toString() : '', onChange: () => {
        if (this.yearControl.value)
          this.selectedYear = parseInt(this.yearControl.value);
        else
          this.selectedYear = 0;
        if (isNaN(this.selectedYear))
          this.selectedYear = 0;
        this.validateDate();
      } }));
  }
  renderYearFieldAsSuggest() {
    const AMOUNT_OF_SUGGEST_ITEMS = 15;
    return (h("k2bt-enhancedsuggest", { inputSize: Math.max(4, this.yearinvitemessage.length), onSuggestValueChanged: this.processYearChange.bind(this), enableadditem: false, placeholder: this.yearinvitemessage, noresultsfoundtext: this.yearnotfoundmessage, value: this.selectedYear != 0 ? [this.selectedYear.toString()] : [], values: this.getCandidateYears(AMOUNT_OF_SUGGEST_ITEMS), maxSelectionSize: 1, suggestmaxitems: AMOUNT_OF_SUGGEST_ITEMS, ref: c => (this.yearControl = c), clientSideFiltering: "Disabled", onInputValueChanged: () => {
        this.yearControl.values = this.getCandidateYears(AMOUNT_OF_SUGGEST_ITEMS);
      } }));
  }
  renderYearFieldAsCombo() {
    var values = Array.from(Array(this.maximumdate.getFullYear() - this.minimumdate.getFullYear() + 1).keys()).map(i => {
      var year = i + this.minimumdate.getFullYear();
      return { value: year.toString(), description: this.normalizeToAmountOfDigits(year, 4) };
    });
    return (h("k2bt-enhancedcombo", { optionswidth: Math.max(4, this.yearinvitemessage.length) + 'ch !important', showemptyiteminlist: false, onChange: this.processYearChange.bind(this), includesearch: false, showselectionastags: false, enableadditem: false, includeemptyitem: true, emptyitemtext: this.yearinvitemessage, headermaxvisibleitems: 1, values: values, value: this.selectedYear != 0 ? [this.selectedYear.toString()] : [], maxSelectionSize: 1, ref: c => (this.yearControl = c) }));
  }
  processYearChange() {
    var _a, _b;
    if (((_b = (_a = this.yearControl.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) > 0)
      this.selectedYear = parseInt(this.yearControl.value[0]);
    else
      this.selectedYear = 0;
    if (isNaN(this.selectedYear) ||
      (this.maximumdate && this.selectedYear > this.maximumdate.getFullYear()) ||
      (this.minimumdate && this.selectedYear < this.minimumdate.getFullYear()))
      this.selectedYear = 0;
    this.validateDate();
  }
  getCandidateYears(max_candidates) {
    var _a, _b;
    var prefix = (_b = (_a = this.yearControl) === null || _a === void 0 ? void 0 : _a.searchvalue) !== null && _b !== void 0 ? _b : '';
    var start = this.getStartYearForSuggestions(prefix);
    max_candidates = Math.min(max_candidates, Math.pow(10, 4 - prefix.length));
    if (this.minimumdate || prefix.length >= 2)
      return Array.from(Array(max_candidates).keys())
        .map(i => i + start)
        .filter(year => this.maximumdate == null || year <= this.maximumdate.getFullYear())
        .map(y => {
        return { value: y.toString(), description: y.toString() };
      });
    else
      return [];
  }
  getStartYearForSuggestions(prefix) {
    var start = 0;
    if (prefix !== '') {
      const missingDigits = prefix.length <= 4 ? 4 - prefix.length : 0;
      start = parseInt(prefix) * Math.pow(10, missingDigits);
    }
    if (this.minimumdate != null)
      start = Math.max(start, this.minimumdate.getFullYear(), this.selectedYear);
    return start;
  }
  renderMonthField() {
    var values = this.getMonthValues();
    if (this.fieldcontroltype == K2btSplitDatepicker$1.FIELDCONTROLTYPE_COMBO) {
      return this.renderMonthAsCombo(values);
    }
    else {
      return this.renderMonthAsSuggest(values);
    }
  }
  renderMonthAsSuggest(values) {
    return (h("k2bt-enhancedsuggest", {
      //@ts-ignore
      inputSize: Math.max(Math.max(...values.map(v => v.description.length)), this.monthinvitemessage.length), onSuggestValueChanged: this.processMonthChange.bind(this), enableadditem: false, placeholder: this.monthinvitemessage, noresultsfoundtext: this.monthnotfoundmessage, value: this.selectedMonth != 0 ? [this.selectedMonth.toString()] : [], values: values, suggestmaxitems: 12, maxSelectionSize: 1, ref: c => (this.monthControl = c), onInputValueChanged: () => (this.monthControl.values = this.getMonthValues())
    }));
  }
  renderMonthAsCombo(values) {
    return (h("k2bt-enhancedcombo", { optionswidth: Math.max(Math.max(...values.map(v => v.description.length)), this.monthinvitemessage.length) + 'ch !important', showemptyiteminlist: false, onChange: this.processMonthChange.bind(this), includesearch: false, showselectionastags: false, enableadditem: false, includeemptyitem: true, emptyitemtext: this.monthinvitemessage, headermaxvisibleitems: 1, values: values, value: this.selectedMonth != 0 ? [this.selectedMonth.toString()] : [], maxSelectionSize: 1, ref: c => (this.monthControl = c) }));
  }
  processMonthChange() {
    var _a, _b;
    if (((_b = (_a = this.monthControl.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) > 0)
      this.selectedMonth = parseInt(this.monthControl.value[0]);
    else
      this.selectedMonth = 0;
    if (isNaN(this.selectedMonth) || this.selectedMonth > 12 || this.selectedMonth < 1)
      this.selectedMonth = 0;
    this.validateDate();
  }
  getMonthValues() {
    switch (this.showmonthas) {
      case K2btSplitDatepicker$1.SHOWMONTHAS_NUMBER:
        return Array.from(Array(12).keys()).map(i => {
          var month = i + 1;
          return { value: month.toString(), description: this.normalizeToAmountOfDigits(month, 2) };
        });
      case K2btSplitDatepicker$1.SHOWMONTHAS_NAME:
        return Array.from(Array(12).keys()).map(i => {
          var month = i + 1;
          //@ts-ignore
          return { value: month.toString(), description: k2btools.getMonthName(month - 1) };
        });
      case K2btSplitDatepicker$1.SHOWMONTHAS_SHORTNAME:
        return Array.from(Array(12).keys()).map(i => {
          var month = i + 1;
          //@ts-ignore
          return { value: month.toString(), description: k2btools.getMonthShortName(month - 1) };
        });
    }
  }
  renderDayField() {
    var values = Array.from(Array(31).keys()).map(i => {
      var day = i + 1;
      return { value: day.toString(), description: this.normalizeToAmountOfDigits(day, 2) };
    });
    if (this.fieldcontroltype == K2btSplitDatepicker$1.FIELDCONTROLTYPE_COMBO) {
      return this.renderDayFieldAsCombo(values);
    }
    else {
      return this.renderDayFieldAsSuggest(values);
    }
  }
  renderDayFieldAsSuggest(values) {
    return (h("k2bt-enhancedsuggest", { inputSize: Math.max(2, this.dayinvitemessage.length), onSuggestValueChanged: this.processDayChange.bind(this), enableadditem: false, placeholder: this.dayinvitemessage, noresultsfoundtext: this.daynotfoundmessage, value: this.selectedDay != 0 ? [this.selectedDay.toString()] : [], values: values, maxSelectionSize: 1, suggestmaxitems: 31, ref: c => (this.dayControl = c), onInputValueChanged: () => (this.dayControl.values = [...values]) }));
  }
  renderDayFieldAsCombo(values) {
    return (h("k2bt-enhancedcombo", { optionswidth: Math.max(2, this.dayinvitemessage.length) + 'ch !important', showemptyiteminlist: false, onChange: this.processDayChange.bind(this), includesearch: false, showselectionastags: false, enableadditem: false, includeemptyitem: true, emptyitemtext: this.dayinvitemessage, headermaxvisibleitems: 1, values: values, value: this.selectedDay != 0 ? [this.selectedDay.toString()] : [], maxSelectionSize: 1, ref: c => (this.dayControl = c) }));
  }
  processDayChange() {
    var _a, _b;
    if (((_b = (_a = this.dayControl.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) > 0)
      this.selectedDay = parseInt(this.dayControl.value[0]);
    else
      this.selectedDay = 0;
    if (isNaN(this.selectedDay) || this.selectedDay > 31 || this.selectedDay < 1)
      this.selectedDay = 0;
    this.validateDate();
  }
  getMaxDayForSelectedMonth() {
    var year = this.selectedYear;
    var month = this.selectedMonth;
    // This is the first date of next month (as month parameter is 0 based)
    const nextMonth = new Date(year, month, 1);
    nextMonth.setDate(nextMonth.getDate() - 1);
    return nextMonth.getDate();
  }
  renderHourField() {
    if (!this.showtimepart)
      return null;
    var values = this.getHourFieldValues();
    if (this.fieldcontroltype == K2btSplitDatepicker$1.FIELDCONTROLTYPE_COMBO) {
      return this.renderHourFieldAsCombo(values);
    }
    else {
      return this.renderHourFieldAsSuggest(values);
    }
  }
  getHourFieldValues() {
    return Array.from(Array(this.endhour + 1 - this.starthour).keys()).map(i => {
      var hour = i + this.starthour; // +1 to avoid "empty" values
      if (this.hourformat == K2btSplitDatepicker$1.HOURFORMAT_24H)
        return { value: (hour + 1).toString(), description: this.normalizeToAmountOfDigits(hour, 2) };
      else {
        var ampmHour = hour;
        if (hour === 0)
          ampmHour = 12;
        if (hour > 12)
          ampmHour = hour - 12;
        return { value: (hour + 1).toString(), description: ampmHour.toString() + ' ' + (hour >= 12 ? 'PM' : 'AM') };
      }
    });
  }
  renderHourFieldAsSuggest(values) {
    return (h("k2bt-enhancedsuggest", {
      //@ts-ignore
      inputSize: Math.max(Math.max(...values.map(v => v.description.length)), this.hourinvitemessage.length), onSuggestValueChanged: this.processHourChange.bind(this), enableadditem: false, placeholder: this.hourinvitemessage, noresultsfoundtext: this.hournotfoundmessage, values: values, suggestmaxitems: 24, value: this.selectedHour != -1 ? [(this.selectedHour + 1).toString()] : [], maxSelectionSize: 1, ref: c => (this.hourControl = c), onInputValueChanged: () => (this.hourControl.values = [...values])
    }));
  }
  renderHourFieldAsCombo(values) {
    return (h("k2bt-enhancedcombo", { optionswidth: Math.max(Math.max(...values.map(v => v.description.length)), this.hourinvitemessage.length) + 'ch !important', showemptyiteminlist: false, onChange: this.processHourChange.bind(this), includesearch: false, showselectionastags: false, enableadditem: false, includeemptyitem: true, emptyitemvalue: "-1", emptyitemtext: this.hourinvitemessage, headermaxvisibleitems: 1, values: values, value: this.selectedHour != -1 ? [(this.selectedHour + 1).toString()] : [], maxSelectionSize: 1, ref: c => (this.hourControl = c) }));
  }
  processHourChange() {
    var _a, _b;
    // Decrease 1 because of values being 1-based and not 0-based
    if (((_b = (_a = this.hourControl.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) > 0)
      this.selectedHour = parseInt(this.hourControl.value[0]) - 1;
    else
      this.selectedYear = 0;
    this.validateDate();
  }
  renderMinuteField() {
    if (!this.showtimepart)
      return null;
    var values = [];
    for (var m = 0; m < 60; m += this.minutestep) {
      // starting in 1 to avoid "empty" values
      values.push({ value: (m + 1).toString(), description: this.normalizeToAmountOfDigits(m, 2) });
    }
    if (this.fieldcontroltype == K2btSplitDatepicker$1.FIELDCONTROLTYPE_COMBO) {
      return this.renderMinuteFieldAsCombo(values);
    }
    else {
      return this.renderMinuteFieldAsSuggest(values);
    }
  }
  renderMinuteFieldAsSuggest(values) {
    return (h("k2bt-enhancedsuggest", {
      //@ts-ignore
      inputSize: Math.max(Math.max(...values.map(v => v.description.length)), this.minuteinvitemessage.length), onSuggestValueChanged: this.processMinuteChange.bind(this), suggestmaxitems: 60, enableadditem: false, placeholder: this.minuteinvitemessage, noresultsfoundtext: this.minutenotfoundmessage, values: values, value: this.selectedMinute != -1 ? [(this.selectedMinute + 1).toString()] : [], maxSelectionSize: 1, ref: c => (this.minuteControl = c), onInputValueChanged: () => (this.minuteControl.values = [...values])
    }));
  }
  processMinuteChange() {
    var _a, _b;
    // Decrease 1 because of values being 1-based and not 0-based
    if (((_b = (_a = this.minuteControl.value) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) > 0)
      this.selectedMinute = parseInt(this.minuteControl.value[0]) - 1;
    else
      this.selectedMinute = 0;
    this.validateDate();
  }
  renderMinuteFieldAsCombo(values) {
    return (h("k2bt-enhancedcombo", { optionswidth: Math.max(Math.max(...values.map(v => v.description.length)), this.minuteinvitemessage.length) + 'ch !important', showemptyiteminlist: false, onChange: this.processMinuteChange.bind(this), includesearch: false, showselectionastags: false, enableadditem: false, includeemptyitem: true, emptyitemvalue: "-1", emptyitemtext: this.minuteinvitemessage, headermaxvisibleitems: 1, values: values, value: this.selectedMinute != -1 ? [(this.selectedMinute + 1).toString()] : [], maxSelectionSize: 1, ref: c => (this.minuteControl = c) }));
  }
  componentDidRender() {
    var _a, _b, _c, _d, _e;
    if (this.fieldcontroltype == K2btSplitDatepicker$1.FIELDCONTROLTYPE_SUGGEST) {
      //@ts-ignore
      if ((_a = this.yearControl) === null || _a === void 0 ? void 0 : _a.updateDescription)
        this.yearControl.updateDescription();
      //@ts-ignore
      if ((_b = this.monthControl) === null || _b === void 0 ? void 0 : _b.updateDescription)
        this.monthControl.updateDescription();
      //@ts-ignore
      if ((_c = this.dayControl) === null || _c === void 0 ? void 0 : _c.updateDescription)
        this.dayControl.updateDescription();
      //@ts-ignore
      if ((_d = this.hourControl) === null || _d === void 0 ? void 0 : _d.updateDescription)
        this.hourControl.updateDescription();
      //@ts-ignore
      if ((_e = this.minuteControl) === null || _e === void 0 ? void 0 : _e.updateDescription)
        this.minuteControl.updateDescription();
    }
  }
  validateDate() {
    this.validationerrorcode = K2btSplitDatepicker$1.VALIDATION_ERROR_NOERROR;
    // Validate not empty
    if (this.validationerrorcode === K2btSplitDatepicker$1.VALIDATION_ERROR_NOERROR && this.selectedDay == 0)
      this.validationerrorcode = K2btSplitDatepicker$1.VALIDATION_ERROR_EMPTYDAY;
    if (this.validationerrorcode === K2btSplitDatepicker$1.VALIDATION_ERROR_NOERROR && this.selectedMonth == 0)
      this.validationerrorcode = K2btSplitDatepicker$1.VALIDATION_ERROR_EMPTYMONTH;
    if (this.validationerrorcode === K2btSplitDatepicker$1.VALIDATION_ERROR_NOERROR && this.selectedYear == 0)
      this.validationerrorcode = K2btSplitDatepicker$1.VALIDATION_ERROR_EMPTYYEAR;
    if (this.showtimepart) {
      if (this.validationerrorcode === K2btSplitDatepicker$1.VALIDATION_ERROR_NOERROR && this.selectedHour == -1)
        this.validationerrorcode = K2btSplitDatepicker$1.VALIDATION_ERROR_EMPTYHOUR;
      if (this.validationerrorcode === K2btSplitDatepicker$1.VALIDATION_ERROR_NOERROR && this.selectedMinute == -1)
        this.validationerrorcode = K2btSplitDatepicker$1.VALIDATION_ERROR_EMPTYMINUTE;
    }
    // Validate month boundary
    var day = this.selectedDay;
    const maxDayForMonth = this.getMaxDayForSelectedMonth();
    if (this.validationerrorcode === K2btSplitDatepicker$1.VALIDATION_ERROR_NOERROR && day > maxDayForMonth)
      this.validationerrorcode = K2btSplitDatepicker$1.VALIDATION_ERROR_INVALIDDAYFORMONTH;
    const candidateDate = this.showtimepart
      ? new Date(this.selectedYear, this.selectedMonth - 1, day, this.selectedHour, this.selectedMinute)
      : new Date(this.selectedYear, this.selectedMonth - 1, day);
    // Validate accepted date ranges
    if (this.validationerrorcode === K2btSplitDatepicker$1.VALIDATION_ERROR_NOERROR && this.minimumdate && candidateDate < this.minimumdate)
      this.validationerrorcode = K2btSplitDatepicker$1.VALIDATION_ERROR_DATETOOLOW;
    if (this.validationerrorcode === K2btSplitDatepicker$1.VALIDATION_ERROR_NOERROR && this.maximumdate && candidateDate > this.maximumdate)
      this.validationerrorcode = K2btSplitDatepicker$1.VALIDATION_ERROR_DATETOOHIGH;
    if (this.validationerrorcode === K2btSplitDatepicker$1.VALIDATION_ERROR_NOERROR)
      this.value = candidateDate;
    else
      this.value = null;
    this.inputEvent.emit({});
    this.changeEvent.emit({});
  }
};
K2btSplitDatepicker$1.ORDER_DMY = 'DMY';
K2btSplitDatepicker$1.ORDER_YMD = 'YMD';
K2btSplitDatepicker$1.ORDER_MDY = 'MDY';
K2btSplitDatepicker$1.SHOWMONTHAS_NAME = 'Name';
K2btSplitDatepicker$1.SHOWMONTHAS_SHORTNAME = 'ShortName';
K2btSplitDatepicker$1.SHOWMONTHAS_NUMBER = 'Number';
K2btSplitDatepicker$1.HOURFORMAT_AMPM = 'AM/PM';
K2btSplitDatepicker$1.HOURFORMAT_24H = '24H';
K2btSplitDatepicker$1.FIELDCONTROLTYPE_COMBO = 'ComboBox';
K2btSplitDatepicker$1.FIELDCONTROLTYPE_SUGGEST = 'Suggest';
K2btSplitDatepicker$1.VALIDATION_ERROR_NOERROR = '0';
K2btSplitDatepicker$1.VALIDATION_ERROR_EMPTYDAY = '1';
K2btSplitDatepicker$1.VALIDATION_ERROR_EMPTYMONTH = '2';
K2btSplitDatepicker$1.VALIDATION_ERROR_EMPTYYEAR = '3';
K2btSplitDatepicker$1.VALIDATION_ERROR_INVALIDDAYFORMONTH = '4';
K2btSplitDatepicker$1.VALIDATION_ERROR_DATETOOLOW = '5';
K2btSplitDatepicker$1.VALIDATION_ERROR_DATETOOHIGH = '6';
K2btSplitDatepicker$1.VALIDATION_ERROR_EMPTYHOUR = '7';
K2btSplitDatepicker$1.VALIDATION_ERROR_EMPTYMINUTE = '8';

const k2btToggleBarCss = ":host{display:block}";

const K2btToggleBar$1 = class extends HTMLElement$1 {
  constructor() {
    super();
    this.__registerHost();
    this.selectionErrorEvent = createEvent(this, "selectionError", 7);
    this.inputEvent = createEvent(this, "input", 7);
    this.changeEvent = createEvent(this, "change", 7);
    this.newRecordClickedEvent = createEvent(this, "newRecordClicked", 7);
    this.SmallChip = 'SmallChip';
    this.MediumChip = 'MediumChip';
    this.Rectangle = 'Rectangle';
    this.value = null;
    this.values = [];
    this.includeemptyitem = true;
    this.emptyitemtext = '(none)';
    this.noresultsfoundtext = 'No results found';
    this.enableadditem = true;
    this.additemcaption = 'New record';
    this.enabled = true;
    this.readonlyclass = '';
    this.togglestyle = this.MediumChip;
    this.maxSelectionSize = 1;
    this.selectedElement = null;
  }
  onImageError(e) {
    e.target.classList.add('K2BT_ToggleBarIconInvisible');
  }
  isCollection() {
    return this.maxSelectionSize != 1;
  }
  selectionIsFull() {
    return this.isCollection() && this.maxSelectionSize != 0 && this.maxSelectionSize <= this.value.length;
  }
  itemIsSelected(item) {
    var _a, _b;
    return ((_b = (_a = this.value) === null || _a === void 0 ? void 0 : _a.filter(selectedItem => selectedItem.toString().trim() == item.value.trim()).length) !== null && _b !== void 0 ? _b : 0) > 0;
  }
  setValue(value) {
    var _a, _b, _c;
    if (this.maxSelectionSize == 1) {
      this.value = [value];
    }
    else {
      if (((_b = (_a = this.value) === null || _a === void 0 ? void 0 : _a.filter(v => v.trim() === value.trim()).length) !== null && _b !== void 0 ? _b : 0) === 0) {
        if (!this.selectionIsFull())
          this.value = ((_c = this.value) !== null && _c !== void 0 ? _c : []).concat([value]);
        else {
          this.errorCode = K2btToggleBar$1.ERROR_SELECTION_FULL;
          this.selectionErrorEvent.emit({});
        }
      }
      else
        this.value = this.value.filter(v => v.trim() !== value.trim());
    }
    this.inputEvent.emit();
    this.changeEvent.emit(value);
  }
  getToggleContent(values, containsDetails, containsIcons, containsTrailingText) {
    if (values.length == 0) {
      return (h("div", { class: "K2BT_ToggleBar_NoItems", style: !this.enabled ? { display: 'none' } : {} }, this.noresultsfoundtext));
    }
    else {
      const atomic = values.filter(v => { var _a, _b; return ((_b = (_a = v.items) === null || _a === void 0 ? void 0 : _a.length) !== null && _b !== void 0 ? _b : 0) == 0; });
      const currentLevelCategories = values.filter(v => { var _a; return ((_a = v.items) === null || _a === void 0 ? void 0 : _a.length) > 0; });
      var containerClasses = this.GetContainerClasses();
      return (h("div", { class: containerClasses, style: !this.enabled ? { display: 'none' } : {} }, atomic.length > 0 ? (h("div", { class: "K2BT_ToggleBarItemContainer" }, atomic.map(item => this.getItemContent(item, containsIcons, containsTrailingText, containsDetails)))) : (''), currentLevelCategories.map(c => {
        return (h("div", { class: "K2BT_ToggleBar_CategoryContainer" }, h("div", { class: "K2BT_ToggleBarCategoryName" }, c.description), this.getToggleContent(c.items, containsDetails, containsIcons, containsTrailingText)));
      })));
    }
  }
  getItemContent(item, containsIcons, containsTrailingText, containsDetails) {
    return (h("div", { class: this.itemIsSelected(item) ? 'K2BT_ToggleBarItem K2BT_ToggleBarItemSelected' : 'K2BT_ToggleBarItem', ref: c => {
        if (this.itemIsSelected(item))
          this.selectedElement = c;
      }, onClick: () => this.setValue(item.value.trim()) }, this.getIconHTMLIfNecessary(containsIcons, item), h("div", { class: "K2BT_ToggleBarItem_TextSection" }, h("div", { class: "K2BT_ToggleBarItem_TextContainer" }, h("span", { class: "K2BT_ToggleBarDescription" }, item.description), containsTrailingText ? h("span", { class: "K2BT_ToggleBarTrailingText" }, item.trailingText) : ''), h("div", { class: "K2BT_ToggleBarItem_TextContainer" }, containsDetails ? h("span", { class: "K2BT_ToggleBarSubtitle" }, item.detail) : ''))));
  }
  getIconHTMLIfNecessary(containsIcons, item) {
    return containsIcons ? (h("div", { class: "K2BT_ToggleBarIconContainer" }, h("img", { class: "K2BT_ToggleBarIcon", src: item.imageUrl, onError: e => this.onImageError(e) }))) : ('');
  }
  GetContainerClasses() {
    var containerClasses = 'K2BT_ToggleBar_Container';
    if (this.togglestyle == this.SmallChip) {
      containerClasses += ' K2BT_ToggleBarSmallChip';
    }
    else if (this.togglestyle == this.Rectangle) {
      containerClasses += ' K2BT_ToggleBarRectangle';
    }
    return containerClasses;
  }
  getSelectedItemsContent() {
    if (this.value == null)
      return this.emptyitemtext;
    else {
      var vals = ControlInfoValue.getAtomicValues_impl(this.values).filter(value => this.itemIsSelected(value));
      if (vals.length > 0)
        return vals.map(v => v.description).join(', ');
      else
        return this.emptyitemtext;
    }
  }
  render() {
    var containsDetails = ControlInfoValue.containsDetails(ControlInfoValue.getAtomicValues_impl(this.values));
    var containsIcons = ControlInfoValue.containsIcons(ControlInfoValue.getAtomicValues_impl(this.values));
    var containsTrailingText = ControlInfoValue.containsTrailingText(ControlInfoValue.getAtomicValues_impl(this.values));
    var valueList = this.getValues();
    return (h(Host, null, h("p", { class: "form-control-static", style: this.enabled ? { display: 'none' } : {} }, h("span", { class: !this.enabled ? this.readonlyclass : this.readonlyclass + ' gx-invisible', "data-gx-readonly": "" }, this.getSelectedItemsContent())), this.getToggleContent(valueList, containsDetails, containsIcons, containsTrailingText)));
  }
  getValues() {
    var valueList = [...this.values];
    if (this.includeemptyitem && !this.isCollection()) {
      valueList.unshift({
        value: '',
        description: this.emptyitemtext,
        imageUrl: '',
        detail: '',
        trailingText: '',
        badgeClass: '',
        items: [],
        tooltip: ''
      });
    }
    return valueList;
  }
  componentDidLoad() {
    var currentValues = ControlInfoValue.getAtomicValues_impl(this.values);
    if (!this.includeemptyitem && currentValues.filter(v => this.itemIsSelected(v)).length == 0) {
      if (currentValues.length > 0) {
        this.setValue(currentValues[0].value.trim());
      }
    }
  }
  static get style() { return k2btToggleBarCss; }
};
K2btToggleBar$1.ERROR_SELECTION_FULL = 'SELECTION_FULL';

const K2btBaseColorPicker = /*@__PURE__*/proxyCustomElement(K2btBaseColorPicker$1, [0,"k2bt-base-color-picker",{"columns":[2],"containerclass":[1],"maxSelectionSize":[2,"max-selection-size"],"enabled":[4],"errorCode":[1,"error-code"],"colorOptions":[16],"selectedIds":[16]}]);
const K2btCalendarActionMenu = /*@__PURE__*/proxyCustomElement(K2btCalendarActionMenu$1, [0,"k2bt-calendar-action-menu",{"actions":[16],"isOpen":[32]},[[8,"click","closeMenu"]]]);
const K2btCalendarDayInMonthPicker = /*@__PURE__*/proxyCustomElement(K2btCalendarDayInMonthPicker$1, [0,"k2bt-calendar-day-in-month-picker",{"selectedDate":[16],"weekstartday":[1]}]);
const K2btCalendarDayView = /*@__PURE__*/proxyCustomElement(K2btCalendarDayView$1, [0,"k2bt-calendar-day-view",{"calendars":[16],"startHour":[2,"start-hour"],"endHour":[2,"end-hour"],"year":[2],"month":[2],"day":[2],"readonly":[4],"draftItemDescription":[1,"draft-item-description"],"showhours":[4],"alldayrows":[2],"draftItem":[32],"currentDate":[32]}]);
const K2btCalendarFullView = /*@__PURE__*/proxyCustomElement(K2btCalendarFullView$1, [0,"k2bt-calendar-full-view",{"showheader":[4],"showcalendarnavigation":[4],"showperiodnavigation":[4],"showviewselection":[4],"showcalendarselection":[4],"starthour":[2],"endhour":[2],"readonly":[4],"todaycaption":[1],"daycaption":[1],"weekcaption":[1],"workweekcaption":[1],"monthcaption":[1],"yearcaption":[1],"seemorecaption":[1],"itemsperdayinmonthview":[2],"dayviewenabled":[4],"weekviewenabled":[4],"workweekviewenabled":[4],"monthviewenabled":[4],"weekstartday":[1],"dateFrom":[1040],"dateTo":[1040],"currentview":[32],"calendars":[32],"selectedCalendars":[32],"advancedSelectorOpen":[32]},[[8,"click","closeMenu"]]]);
const K2btCalendarMonthView = /*@__PURE__*/proxyCustomElement(K2btCalendarMonthView$1, [0,"k2bt-calendar-month-view",{"year":[1026],"month":[1026],"readonly":[4],"calendars":[16],"seemorecaption":[1],"itemsperday":[2],"weekstartday":[1],"currentDate":[32]}]);
const K2btCalendarPeriodView = /*@__PURE__*/proxyCustomElement(K2btCalendarPeriodView$1, [0,"k2bt-calendar-period-view",{"calendars":[16],"startHour":[2,"start-hour"],"endHour":[2,"end-hour"],"dateFrom":[16],"dateTo":[16],"readonly":[4],"draftItemDescription":[1,"draft-item-description"],"draftItem":[32]}]);
const K2btEditCollection = /*@__PURE__*/proxyCustomElement(K2btEditCollection$1, [0,"k2bt-edit-collection",{"value":[16],"maxSelectionSize":[2,"max-selection-size"],"datatype":[1],"length":[2],"integers":[2],"decimals":[2],"enabled":[4],"inputclass":[1],"readonlyclass":[1],"invitemessage":[1]}]);
const K2btEnhancedcombo = /*@__PURE__*/proxyCustomElement(K2btEnhancedcombo$1, [0,"k2bt-enhancedcombo",{"value":[16],"values":[1],"includesearch":[4],"includeemptyitem":[4],"emptyitemvalue":[1],"emptyitemtext":[1],"noresultsfoundtext":[1],"open":[4],"enableadditem":[4],"additemcaption":[1],"clearselectioncaption":[1],"selectallcaption":[1],"searchfieldplaceholder":[1],"searchvalue":[1],"enabled":[4],"maxSelectionSize":[2,"max-selection-size"],"headermaxvisibleitems":[2],"readonlyclass":[1],"errorCode":[1,"error-code"],"showselectionastags":[4],"showiconsintags":[4],"displayaslist":[4],"enableclearselectionaction":[4],"enableselectallaction":[4],"activeElementValue":[1,"active-element-value"],"showemptyiteminlist":[4],"optionswidth":[1]},[[0,"keydown","processKeydown"],[0,"focusin","onFocusIn"],[8,"click","closeMenu"]]]);
const K2btEnhancedsuggest = /*@__PURE__*/proxyCustomElement(K2btEnhancedsuggest$1, [0,"k2bt-enhancedsuggest",{"value":[16],"values":[1],"noresultsfoundtext":[1],"open":[4],"enableadditem":[4],"additemcaption":[1],"enabled":[4],"readonlyclass":[1],"searchvalue":[1],"placeholder":[1],"selectedValueDescription":[1,"selected-value-description"],"maxSelectionSize":[2,"max-selection-size"],"errorCode":[1,"error-code"],"suggestmaxitems":[2],"activeElementValue":[1,"active-element-value"],"showiconsintags":[4],"emptyitemtext":[1],"clientSideFiltering":[1,"client-side-filtering"],"inputSize":[2,"input-size"],"waitingInformationFromServer":[32]},[[2,"input","processInput"],[0,"keydown","processKeydown"],[8,"click","closeMenu"]]]);
const K2btImageRegionSelector = /*@__PURE__*/proxyCustomElement(K2btImageRegionSelector$1, [0,"k2bt-image-region-selector",{"imageWithRegionsDefinition":[16],"selectedIds":[16],"maxSelectionSize":[2,"max-selection-size"],"errorCode":[1,"error-code"],"enabled":[4]}]);
const K2btMaskedInput = /*@__PURE__*/proxyCustomElement(K2btMaskedInput$1, [0,"k2bt-masked-input",{"mask":[1],"numeric":[4],"enabled":[4],"inputclass":[1],"readonlyclass":[1],"value":[1],"uppercase":[4]}]);
const K2btMegaMenu = /*@__PURE__*/proxyCustomElement(K2btMegaMenu$1, [0,"k2bt-mega-menu",{"items":[16],"openSublevels":[32],"activeSublevelCode":[32]}]);
const K2btNumericInput = /*@__PURE__*/proxyCustomElement(K2btNumericInput$1, [0,"k2bt-numeric-input",{"decimals":[2],"integerdigits":[2],"includethousandseparator":[4],"includesign":[4],"usermustenterdecimalseparator":[4],"decimalseparator":[1],"thousandseparator":[1],"valueprefix":[1],"zeropadding":[4],"inputclass":[1],"readonlyclass":[1],"value":[1],"enabled":[4]},[[2,"keydown","handleKeydown"]]]);
const K2btNumericSlider = /*@__PURE__*/proxyCustomElement(K2btNumericSlider$1, [0,"k2bt-numeric-slider",{"max":[2],"min":[2],"step":[2],"numbervisible":[4],"numberreadonly":[4],"numberclass":[1],"sliderclass":[1],"value":[1],"readonlyclass":[1],"enabled":[4]}]);
const K2btPillViewer = /*@__PURE__*/proxyCustomElement(K2btPillViewer$1, [0,"k2bt-pill-viewer",{"pills":[16]}]);
const K2btSignaturePad = /*@__PURE__*/proxyCustomElement(K2btSignaturePad$1, [0,"k2bt-signature-pad",{"width":[2],"height":[2],"backgroundimageurl":[1],"backgroundcolor":[1],"showclearbutton":[4],"clearbuttoncaption":[1]}]);
const K2btSplitDatepicker = /*@__PURE__*/proxyCustomElement(K2btSplitDatepicker$1, [0,"k2bt-split-datepicker",{"showtimepart":[4],"minimumdate":[16],"maximumdate":[16],"dayinvitemessage":[1],"monthinvitemessage":[1],"yearinvitemessage":[1],"hourinvitemessage":[1],"minuteinvitemessage":[1],"daynotfoundmessage":[1],"monthnotfoundmessage":[1],"yearnotfoundmessage":[1],"hournotfoundmessage":[1],"minutenotfoundmessage":[1],"showmonthas":[1],"fieldcontroltype":[1],"fieldorder":[1],"hourformat":[1],"starthour":[2],"endhour":[2],"minutestep":[2],"value":[16],"readonlyclass":[1],"enabled":[4],"validationerrorcode":[1],"selectedYear":[32],"selectedMonth":[32],"selectedDay":[32],"selectedHour":[32],"selectedMinute":[32]}]);
const K2btToggleBar = /*@__PURE__*/proxyCustomElement(K2btToggleBar$1, [0,"k2bt-toggle-bar",{"value":[16],"values":[16],"includeemptyitem":[4],"emptyitemtext":[1],"noresultsfoundtext":[1],"enableadditem":[4],"additemcaption":[1],"enabled":[4],"readonlyclass":[1],"togglestyle":[1],"maxSelectionSize":[2,"max-selection-size"],"errorCode":[1,"error-code"]}]);
const defineCustomElements = (opts) => {
  if (typeof customElements !== 'undefined') {
    [
      K2btBaseColorPicker,
  K2btCalendarActionMenu,
  K2btCalendarDayInMonthPicker,
  K2btCalendarDayView,
  K2btCalendarFullView,
  K2btCalendarMonthView,
  K2btCalendarPeriodView,
  K2btEditCollection,
  K2btEnhancedcombo,
  K2btEnhancedsuggest,
  K2btImageRegionSelector,
  K2btMaskedInput,
  K2btMegaMenu,
  K2btNumericInput,
  K2btNumericSlider,
  K2btPillViewer,
  K2btSignaturePad,
  K2btSplitDatepicker,
  K2btToggleBar
    ].forEach(cmp => {
      if (!customElements.get(cmp.is)) {
        customElements.define(cmp.is, cmp, opts);
      }
    });
  }
};

export { K2btBaseColorPicker, K2btCalendarActionMenu, K2btCalendarDayInMonthPicker, K2btCalendarDayView, K2btCalendarFullView, K2btCalendarMonthView, K2btCalendarPeriodView, K2btEditCollection, K2btEnhancedcombo, K2btEnhancedsuggest, K2btImageRegionSelector, K2btMaskedInput, K2btMegaMenu, K2btNumericInput, K2btNumericSlider, K2btPillViewer, K2btSignaturePad, K2btSplitDatepicker, K2btToggleBar, defineCustomElements };
